------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 534 $
--    $Date: 2012-10-05 19:46:28 +0200 (Fri, 05 Oct 2012) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Framework_Config;      use Framework_Config;
with Resource_Set;          use Resource_Set;
with Tasks;                 use Tasks;
with Task_Groups;           use Task_Groups;
with Parameters;            use Parameters;
with Parameters.extended;   use Parameters.extended;
use Parameters.User_Defined_Parameters_Table_Package;
with Offsets;               use Offsets;
with Offsets.extended;      use Offsets.extended;
use Offsets.Offsets_Table_Package;
with Ada.Numerics.Aux;      use Ada.Numerics.Aux;
with sets;
with scheduler.fixed_priority; use scheduler.fixed_priority;
with scheduler.fixed_priority.rm; use scheduler.fixed_priority.rm;
with scheduler.fixed_priority.dm; use scheduler.fixed_priority.dm;
with scheduler.fixed_priority.hpf; use scheduler.fixed_priority.hpf;
with scheduler.dynamic_priority; use scheduler.dynamic_priority;
with scheduler.dynamic_priority.edf; use scheduler.dynamic_priority.edf;
with scheduler.dynamic_priority.llf; use scheduler.dynamic_priority.llf;
with scheduler.dynamic_priority.d_over; use scheduler.dynamic_priority.d_over;
with scheduler.dynamic_priority.muf; use scheduler.dynamic_priority.muf;
with Call_Framework_Interface; use Call_Framework_Interface;




package feasibility_test.processor_utilization is


   function Processor_Utilization_Over_Deadline
     (My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String)
      return           Double;

   function Processor_Utilization_Over_Period
     (My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String)
      return           Double;





 procedure Bound_On_Processor_Utilization
     (My_Scheduler   : in Generic_Scheduler_ptr;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Double;
      Msg            : in out Unbounded_String);

procedure  Bound_On_Processor_Utilization
     (My_Scheduler   : in Dynamic_Priority_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Double;
      Msg            : in out Unbounded_String);

procedure Bound_On_Processor_Utilization
     (My_Scheduler   : in Fixed_Priority_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Double;
      Msg            : in out Unbounded_String);






   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in generic_Scheduler_ptr;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String;
      Output         : in Output_Format := String_Output);

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in D_Over_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String;
      Output         : in Output_Format := String_Output);

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in Edf_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String;
      Output         : in Output_Format := String_Output);

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in Llf_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String;
      Output         : in Output_Format := String_Output);

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in Muf_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String;
      Output         : in Output_Format := String_Output);

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in Dm_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String;
      Output         : in Output_Format := String_Output);

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in Rm_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String;
      Output         : in Output_Format := String_Output);




end feasibility_test.processor_utilization;



