------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded;    use Ada.Strings.Unbounded;
with Parameters;               use Parameters;
with Parameters.extended;      use Parameters.extended;
use Parameters.Framework_Parameters_Table_Package;
with Framework_Config;         use Framework_Config;
with Systems;                  use Systems;
with Tables;
with Call_Framework_Interface; use Call_Framework_Interface;
with id_generators;            use id_generators;

package Call_Framework is

   Boolean_framework_parameter_name : array (1 .. 26) of Unbounded_String :=
     (To_Unbounded_String ("schedule_with_offsets"),
      To_Unbounded_String ("schedule_with_precedencies"),
      To_Unbounded_String ("schedule_with_resources"),
      To_Unbounded_String ("schedule_with_jitters"),
      To_Unbounded_String ("schedule_with_crpd"),
      To_Unbounded_String ("minimize_preemption"),
      To_Unbounded_String ("predictable"),
      To_Unbounded_String ("context_switch_overhead"),
      To_Unbounded_String ("start_of_task_capacity"),
      To_Unbounded_String ("end_of_task_capacity"),
      To_Unbounded_String ("write_to_buffer"),
      To_Unbounded_String ("read_from_buffer"),
      To_Unbounded_String ("running_task"),
      To_Unbounded_String ("task_activation"),
      To_Unbounded_String ("send_message"),
      To_Unbounded_String ("receive_message"),
      To_Unbounded_String ("allocate_resource"),
      To_Unbounded_String ("release_resource"),
      To_Unbounded_String ("wait_for_resource"),
      To_Unbounded_String ("wait_for_memory"),
      To_Unbounded_String ("preemption"),
      To_Unbounded_String ("address_space_activation"),
      To_Unbounded_String ("schedule_with_task_groups"),
      To_Unbounded_String ("worst_case"),
      To_Unbounded_String ("best_case"),
      To_Unbounded_String ("average_case"));

   Integer_framework_parameter_name : array (1 .. 2) of Unbounded_String :=
     (To_Unbounded_String ("period"),
      To_Unbounded_String ("seed_value"));

   String_framework_parameter_name : array (1 .. 2) of Unbounded_String :=
     (To_Unbounded_String ("feasibility_test_name"),
      To_Unbounded_String ("feasibility_test_name"));

   Invalid_Number_Of_Parameter : exception;
   Unexpected_Parameter : exception;
   Parameter_Type_Error : exception;

   -- Initialize the framework. Should be called before any
   -- use of the framework
   --
   procedure initialize (Aadl_Labels : in Boolean);

   procedure Sequential_Framework_Request
     (Sys      : in out System;
      Request  : in Framework_Request_Table;
      Response : in out Framework_Response_Table;
      Order    : in Perform_Order;
      Output   : in Output_Format);

   procedure Sequential_Framework_Request
     (Sys      : in out System_Ptr;
      Request  : in Framework_Request_Table;
      Response : in out Framework_Response_Table;
      Order    : in Perform_Order;
      Output   : in Output_Format);

   procedure Sequential_Framework_Request
     (Sys      : in out System;
      Request  : in Framework_Request_Table;
      Response : in out Framework_Response_Table);

   procedure Parallel_Framework_Request
     (Sys      : in out System;
      Request  : in Framework_Request_Table;
      Response : in out Framework_Response_Table;
      Order    : in Perform_Order := Total_Order;
      Output   : in Output_Format := String_Output);

   procedure Distributed_Framework_Request
     (Sys      : in out System;
      Request  : in Framework_Request_Table;
      Response : in out Framework_Response_Table;
      Order    : in Perform_Order := Total_Order;
      Output   : in Output_Format := String_Output);

   -------------------------------------------------------------------------

   -- Global variable of the framework
   --

   -- Unique id generator of Cheddar XML objects
   --
   framework_id : id;

   -------------------------------------------------------------------------

end Call_Framework;
