------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 548 $
--    $Date: 2012-10-12 01:48:51 +0200 (Fri, 12 Oct 2012) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework_Interface;     use Call_Framework_Interface;
with Framework_Config;             use Framework_Config;
with Systems;                      use Systems;
with Task_Set;                     use Task_Set;
with Processors;                   use Processors;
with Ada.Strings.Unbounded;        use Ada.Strings.Unbounded;
with Buffers;                      use Buffers;
use Buffers.Buffer_Roles_Package;
with Scheduling_Analysis;          use Scheduling_Analysis;
with Scheduling_Analysis.extended; use Scheduling_Analysis.extended;
with Scheduling_Analysis.extended.buffer_analysis; use Scheduling_Analysis.extended.buffer_analysis;
use Scheduling_Analysis.Buffer_Size_Package;
use Scheduling_Analysis.Waiting_Time_Package;
with indexed_tables;

package Call_Memory_Framework is

   package Buffer_Result_Package is new indexed_tables (
      Buffer_Size_Table,
      Buffer_Ptr,
      Framework_Config.Max_Buffers,
      0,
      Put,
      Initialize,
      Buffers.Put_Name,
      Get_Name,
      XML_root_String,
      XML_String);
   use Buffer_Result_Package;

   subtype Buffer_Result_Table is Buffer_Result_Package.indexed_table;
   subtype Buffer_Result_Range is Buffer_Result_Package.indexed_table_range;

   -- Last computed buffer size
   --
   Buff : Buffer_Result_Table;

   -- Compute global memory requirement (stack, buffer, text, ....)
   --
   procedure Compute_Memory_Requirement_Analysis
     (Sys                       : in System;
      Result                    : in out Unbounded_String;
      Update_Address_Spaces_Set : in Boolean;
      Output                    : in Output_Format := String_Output);

   -- Perform analysis on buffer requirement from a scheduling
   -- simulation
   --
   procedure Compute_Buffer_Scheduling_Simulation
     (Sys         : in System;
      Result      : in out Unbounded_String;
      A_Processor : in Generic_Processor_Ptr;
      Output      : in Output_Format := String_Output);

   -- Perform analysis on buffer requirement from analytical
   -- equation
   --
   procedure Compute_Buffer_Feasibility_Tests
     (Sys         : in System;
      Result      : in out Unbounded_String;
      A_Processor : in Generic_Processor_Ptr;
      Output      : in Output_Format := String_Output);

end Call_Memory_Framework;
