------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
--
--
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Framework_Config;      use Framework_Config;
with Caches;                use Caches;
with sets;


package Cache_Access_Profile_Set is

   ------------------------------------------------------
   -- CACHE ACCESS PROFILE
   ------------------------------------------------------
   package Cache_Access_Profile_Set is new Sets (max_element    => Framework_Config.Max_Tasks,
                                                 element        => Cache_Access_Profile_Ptr,
                                                 free           => Free,
                                                 copy           => Copy,
                                                 put            => Put,
                                                 xml_string     => XML_String,
                                                 xml_ref_string => XML_Ref_String);

   use Cache_Access_Profile_Set;

   type Cache_Access_Profiles_Set is new Cache_Access_Profile_Set.set with private;

   subtype Cache_Access_Profiles_Range is Cache_Access_Profile_Set.element_range;
   subtype Cache_Access_Profiles_Iterator is Cache_Access_Profile_Set.iterator;


   --
   -- Exception
   --
   Cache_Access_Profile_Not_Found : exception;

   Cache_Access_Profile_Must_Be_Defined : exception;

   Invalid_Parameter : exception;
   --
   -- Add
   --
   procedure Add_Cache_Access_Profile
     (my_cache_access_profiles	: in out Cache_Access_Profiles_Set;
      a_cache_access_profile	: in out Cache_Access_Profile_Ptr;
      name			: in Unbounded_String;
      ucbs			: in Cache_Blocks_Table;
      ecbs			: in Cache_Blocks_Table);

   procedure Add_Cache_Access_Profile
     (my_cache_access_profiles	: in out Cache_Access_Profiles_Set;
      name			: in Unbounded_String;
      ucbs			: in Cache_Blocks_Table;
      ecbs			: in Cache_Blocks_Table);

   --
   -- Update
   --
    procedure Update_Cache_Access_Profile
     (my_cache_access_profiles	: in out Cache_Access_Profiles_Set;
      name			: in Unbounded_String;
      ucbs			: in Cache_Blocks_Table;
      ecbs			: in Cache_Blocks_Table);

   --
   -- Search
   --
   function Search_Cache_Access_Profile
     (my_cache_access_profiles 	: in Cache_Access_Profiles_Set;
      name          		: in Unbounded_String)
      return          	Cache_Access_Profile_Ptr;

   --
   -- Delete
   --
   procedure Delete_Cache_Access_Profile
     (my_cache_access_profiles	: in out Cache_Access_Profiles_Set;
      name 			: in Unbounded_String);


private
   type Cache_Access_Profiles_Set is new Cache_Access_Profile_Set.set with null record;

end Cache_Access_Profile_Set;
