------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
--
--
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Framework_Config;      use Framework_Config;
with Caches;                use Caches;
with sets;


package Cache_Block_Set is

   ------------------------------------------------------
   -- CACHE BLOCKS
   ------------------------------------------------------
   package Cache_Block_Set is new Sets (max_element    => Framework_Config.Max_Cache_Size,
                                        element        => Cache_Block_Ptr,
                                        free           => Free,
                                        copy           => Copy,
                                        put            => Put,
                                        xml_string     => XML_String,
                                        xml_ref_string => XML_Ref_String);

   use Cache_Block_Set;

   type Cache_Blocks_Set is new Cache_Block_Set.set with private;

   subtype Cache_Blocks_Range is Cache_Block_Set.element_range;
   subtype Cache_Blocks_Iterator is Cache_Block_Set.iterator;

   --
   -- Exception
   --
   Cache_Block_Not_Found : exception;

   Cache_Block_Must_Be_Defined : exception;

   Invalid_Parameter : exception;
   --
   -- Add
   --
   procedure Add_Cache_Block
     (my_cache_blocks		: in out Cache_Blocks_Set;
      a_cache_block		: in out Cache_Block_Ptr;
      name			: in Unbounded_String;
      cache_block_number	: in Natural);

   procedure Add_Cache_Block
     (my_cache_blocks		: in out Cache_Blocks_Set;
      name			: in Unbounded_String;
      cache_block_number	: in Natural);

   --
   -- Update
   --
   procedure Update_Cache_Block
     (my_cache_blocks		: in out Cache_Blocks_Set;
      name			: in Unbounded_String;
      cache_block_number	: in Natural);

   --
   -- Search
   --
   function Search_Cache_Block
     (my_cache_blocks 	: in Cache_Blocks_Set;
      name          	: in Unbounded_String)
      return          	Cache_Block_Ptr;

   function Search_Cache_Block_By_Id
     (my_cache_blocks 	: in Cache_Blocks_Set;
      id          	: in Unbounded_String)
      return          	Cache_Block_Ptr;

   --
   -- Delete
   --
   procedure Delete_Cache_Block
     (my_cache_blocks	: in out Cache_Blocks_Set;
      name 		: in Unbounded_String);


private
   type Cache_Blocks_Set is new Cache_Block_Set.set with null record;

end Cache_Block_Set;
