with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Framework_Config;      use Framework_Config;
with CFG_Edges;		    use CFG_Edges;

with sets;


package CFG_Edge_Set is

   ----------------------------------------------------
   -- In this package, we work with both CFG_Edge_Table and CFG_Edge_Set.
   -- The reason for that is because:
   -- + In the system, we use CFG_Edge_Set
   -- + In the task, we use CFG_Edge_Table
   -- We did by this way because this is the current implementation for other
   -- models of Cheddars such as processor and core_unit.
   ----------------------------------------------------


   ----------------------------------------------------------
   -- Definition of basic block set
   ----------------------------------------------------------

   package CFG_Edge_Set is new Sets (
                                     max_element    => Framework_Config.Max_CFG_Edges,
                                     element        => CFG_Edge_Ptr,
                                     free           => Free,
                                     copy           => Copy,
                                     put            => Put,
                                     xml_string     => XML_String,
                                     xml_ref_string => XML_Ref_String);

   use CFG_Edge_Set;

   type CFG_Edges_Set is new CFG_Edge_Set.set with private;

   subtype CFG_Edges_Range is CFG_Edge_Set.element_range;
   subtype CFG_Edges_Iterator is CFG_Edge_Set.iterator;

   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------

   -- Raised when a given CFG node is not in a CFG node set
   --
   CFG_Edge_Not_Found : exception;

   -- Raised when parameters provided to Add_Basic_Block are wrong
   --
   Invalid_Parameter : exception;

   ----------------------------------------------------
   -- CHECK
   ----------------------------------------------------

   procedure Check_CFG_Edge
     (name 			: in Unbounded_String;
      node			: in Unbounded_String;
      next_node			: in Unbounded_String);

   ----------------------------------------------------
   -- ADD
   ----------------------------------------------------
   procedure Add_CFG_Edge
     (my_cfg_edges		: in out CFG_Edges_Set;
      a_cfg_edge		: in out CFG_Edge_Ptr;
      name 			: in Unbounded_String;
      node			: in Unbounded_String;
      next_node			: in Unbounded_String);

   procedure Add_CFG_Edge
     (my_cfg_edges		: in out CFG_Edges_Set;
      name 			: in Unbounded_String;
      node			: in Unbounded_String;
      next_node			: in Unbounded_String);

   ----------------------------------------------------
   -- UPDATE
   ----------------------------------------------------

   procedure Update_CFG_Edge
     (my_cfg_edges		: in out CFG_Edges_Set;
      name 			: in Unbounded_String;
      node			: in Unbounded_String;
      next_node			: in Unbounded_String);

   ----------------------------------------------------
   -- SEARCH
   ----------------------------------------------------

   function Search_CFG_Edge
     (my_cfg_edges	: in CFG_Edges_Set;
      name          	: in Unbounded_String)
      return		CFG_Edge_Ptr;

   --Search CFG Edge by cheddar_private_id
   function Search_CFG_Edge_By_Id
     (my_cfg_edges 	: in CFG_Edges_Set;
      id 		: in Unbounded_String)
      return            CFG_Edge_Ptr;

   ----------------------------------------------------
   -- DELETE
   ----------------------------------------------------
   procedure Delete_CFG_Edge
     (my_cfg_edges		: in out CFG_Edges_Set;
      name 			: in Unbounded_String);

private
   type CFG_Edges_Set is new CFG_Edge_Set.set with null record;

end CFG_Edge_Set;
