with Ada.Exceptions;		use Ada.Exceptions;
with CFG_Nodes;			use CFG_Nodes;
with CFG_Nodes;			use CFG_Nodes.CFG_Nodes_Table_Package;
with Objects;           	use Objects;
with Objects.extended;  	use Objects.extended;
with Text_IO;			use Text_IO;
with Ada.Integer_Text_IO;       use Ada.Integer_Text_IO;
with Integer_Arrays;		use Integer_Arrays;
with Translate;         	use Translate;
with CFG_Nodes.Extended;        use CFG_Nodes.Extended;
with initialize_framework; 	use initialize_framework;
with Tables;
with sets;


package body CFG_Node_Set is

   -- In this package, we work with both CFG_Node_Table and CFG_Node_Set.
   -- The reason for that is because:
   -- + In the system, we use CFG_Node_Set
   -- + In the task, we use CFG_Node_Table
   -- We did by this way because this is the current implementation for other
   -- models of Cheddars such as processor and core_unit.

   ----------------------------------------------------
   -- CHECK
   ----------------------------------------------------

   procedure Check_CFG_Node
     (name			: in Unbounded_String;
      node_type			: in CFG_Node_Type;
      graph_type		: in CFG_Graph_Type)
   is
   begin
      if (name = "") then
         Raise_Exception
           (Invalid_Parameter'Identity,
            To_String
              (Lb_CFG_Node (Current_Language) &
                 Lb_Mandatory (Current_Language)));
      end if;

      if not Is_A_Valid_Identifier (Name) then
         Raise_Exception
           (Invalid_Parameter'Identity,
            To_String
              (Lb_CFG_Node (Current_Language) &
                 Name &
                 " : " &
                 Lb_CFG_Node_Name (Current_Language) &
                 Lb_Colon &
                 Lb_Invalid_Identifier (Current_Language)));
      end if;

   end Check_CFG_Node;

   ----------------------------------------------------
   -- ADD
   ----------------------------------------------------
   procedure Add_CFG_Node
     (my_cfg_nodes		: in out CFG_Nodes_Set;
      a_cfg_node		: in out CFG_Node_Ptr;
      name			: in Unbounded_String;
      node_type			: in CFG_Node_Type;
      graph_type		: in CFG_Graph_Type)
   is
      my_iterator             	: CFG_Nodes_Iterator;
   begin
      check_initialize;

      Check_CFG_Node(name,
                     node_type,
                     graph_type);

      if (get_number_of_elements (my_cfg_nodes) > 0) then
         reset_iterator(my_cfg_nodes,my_iterator);
         loop
            current_element(my_cfg_nodes,a_cfg_node,my_iterator);
            if(name = a_cfg_node.name) then
               Raise_Exception
                 (Invalid_Parameter'Identity,
                  To_String
                    (Lb_CFG_Node (Current_Language) &
                       " " &
                       Name &
                       " : " &
                       Lb_CFG_Node_Name (Current_Language) &
                       Lb_Already_Defined (Current_Language)));
            end if;
            exit when is_last_element (my_cfg_nodes, my_iterator);
            next_element (my_cfg_nodes, my_iterator);
         end loop;
      end if;

      a_cfg_node				:= new CFG_Node;
      a_cfg_node.name 				:= name;
      a_cfg_node.node_type 			:= node_type;
      a_cfg_node.graph_type			:= graph_type;

      add(my_cfg_nodes,a_cfg_node);

   exception
      when full_set =>
         Raise_Exception
           (Invalid_Parameter'Identity,
            To_String (Lb_Can_Not_Define_More_CFG_Node (Current_Language)));
   end Add_CFG_Node;

   procedure Add_CFG_Node
     (my_cfg_nodes		: in out CFG_Nodes_Set;
      name			: in Unbounded_String;
      node_type			: in CFG_Node_Type;
      graph_type		: in CFG_Graph_Type)
   is
      a_cfg_node 		: CFG_Node_Ptr;
   begin
      Add_CFG_Node(my_cfg_nodes    => my_cfg_nodes,
                   a_cfg_node      => a_cfg_node,
                   name            => name,
                   node_type       => node_type,
                   graph_type      => graph_type);
   end Add_CFG_Node;

   ------------------------------------------------------
   -- SEARCH
   ------------------------------------------------------
   function Search_CFG_Node
     (my_cfg_nodes	: in CFG_Nodes_Set;
      name          	: in Unbounded_String)
      return		CFG_Node_Ptr

   is
      my_iterator 	: CFG_Nodes_Iterator;
      a_cfg_node  	: CFG_Node_Ptr;
      result      	: CFG_Node_Ptr;
      found 		: Boolean := False;
   begin
      if not is_empty(my_cfg_nodes) then
         reset_iterator(my_cfg_nodes,my_iterator);
         loop
            current_element (my_cfg_nodes, a_cfg_node, my_iterator);
            if (a_cfg_node.name = name) then
               found  := True;
               result := a_cfg_node;
            end if;

            exit when is_last_element (my_cfg_nodes,my_iterator);
            next_element (my_cfg_nodes,my_iterator);
         end loop;

      end if;

      if not Found then
         Raise_Exception
           (CFG_Node_Not_Found'Identity,
            To_String (Lb_CFG_Node_Name (Current_Language) & "=" & Name));
      end if;

      return Result;

   end Search_CFG_Node;

   function Search_CFG_Node_By_Id
     (my_cfg_nodes	: in CFG_Nodes_Set;
      id          	: in Unbounded_String)
      return		CFG_Node_Ptr

   is
      my_iterator 	: CFG_Nodes_Iterator;
      a_cfg_node  	: CFG_Node_Ptr;
      result      	: CFG_Node_Ptr;
      found 		: Boolean := False;
   begin
      if not is_empty(my_cfg_nodes) then
         reset_iterator(my_cfg_nodes,my_iterator);
         loop
            current_element (my_cfg_nodes, a_cfg_node, my_iterator);
            if (a_cfg_node.cheddar_private_id = id) then
               found  := True;
               result := a_cfg_node;
            end if;

            exit when is_last_element (my_cfg_nodes,my_iterator);
            next_element (my_cfg_nodes,my_iterator);
         end loop;
      end if;

      if not Found then
         Raise_Exception
           (CFG_Node_Not_Found'Identity,
            To_String (Lb_CFG_Node_Id (Current_Language) & "=" & id));
      end if;

      return Result;

   end Search_CFG_Node_By_Id;

   ------------------------------------------------------
   -- DELETE
   ------------------------------------------------------
   procedure Delete_CFG_Node
     (my_cfg_nodes		: in out CFG_Nodes_Set;
      name 			: in Unbounded_String)
   is
      a_cfg_node			: CFG_Node_Ptr;
   begin
      a_cfg_node := Search_CFG_Node(my_cfg_nodes, name);
      delete (my_cfg_nodes, a_cfg_node);
   end Delete_CFG_Node;

   ------------------------------------------------------
   -- UPDATE
   ------------------------------------------------------
   procedure Update_CFG_Node
     (my_cfg_nodes		: in out CFG_Nodes_Set;
      name			: in Unbounded_String;
      node_type			: in CFG_Node_Type;
      graph_type		: in CFG_Graph_Type)
   is
      a_cfg_node		: CFG_Node_Ptr;
   begin
      Check_CFG_Node(name       => name,
                     node_type  => node_type,
                     graph_type => graph_type);

      a_cfg_node := Search_CFG_Node(my_cfg_nodes,name);

      Delete(my_cfg_nodes,a_cfg_node);

      Add_CFG_Node(my_cfg_nodes    => my_cfg_nodes,
                   a_cfg_node      => a_cfg_node,
                   name            => name,
                   node_type       => node_type,
                   graph_type      => graph_type);

   end Update_CFG_Node;

   ------------------------------------------------------
   -- NEXT, PREVIOUS
   ------------------------------------------------------
   procedure Add_Next_CFG_Node
     (my_cfg_nodes		: in out CFG_Nodes_Set;
      name			: in Unbounded_String;
      next_node_name		: in Unbounded_String)
   is
      a_cfg_node		: CFG_Node_Ptr;
      a_next_cfg_node		: CFG_Node_Ptr;
   begin
      a_cfg_node := Search_CFG_Node(my_cfg_nodes,name);
      a_next_cfg_node := Search_CFG_Node(my_cfg_nodes,next_node_name);

      add(CFG_Node_Ext_Ptr(a_cfg_node).next_nodes,a_next_cfg_node);
   end Add_Next_CFG_Node;

   procedure Add_Next_CFG_Node_By_Id
     (my_cfg_nodes		: in out CFG_Nodes_Set;
      a_cfg_node		: in out CFG_Node_Ptr;
      next_node_id		: in Unbounded_String)
   is
      a_next_cfg_node		: CFG_Node_Ptr;
   begin
      a_next_cfg_node := Search_CFG_Node_By_Id(my_cfg_nodes,next_node_id);
      add(CFG_Node_Ext_Ptr(a_cfg_node).next_nodes,a_next_cfg_node);
   end Add_Next_CFG_Node_By_Id;

   procedure Add_Previous_CFG_Node
     (my_cfg_nodes		: in out CFG_Nodes_Set;
      name			: in Unbounded_String;
      previous_node_name	: in Unbounded_String)
   is
      a_cfg_node		: CFG_Node_Ptr;
      a_next_cfg_node		: CFG_Node_Ptr;
   begin
      a_cfg_node := Search_CFG_Node(my_cfg_nodes,name);
      a_next_cfg_node := Search_CFG_Node(my_cfg_nodes,previous_node_name);

      add(CFG_Node_Ext_Ptr(a_cfg_node).previous_nodes,a_next_cfg_node);
   end Add_Previous_CFG_Node;

   procedure Add_Previous_CFG_Node_By_Id
     (my_cfg_nodes		: in out CFG_Nodes_Set;
      a_cfg_node		: in out CFG_Node_Ptr;
      previous_node_id		: in Unbounded_String)
   is
      a_next_cfg_node		: CFG_Node_Ptr;
   begin
      a_next_cfg_node := Search_CFG_Node_By_Id(my_cfg_nodes,previous_node_id);
      add(CFG_Node_Ext_Ptr(a_cfg_node).previous_nodes,a_next_cfg_node);
   end Add_Previous_CFG_Node_By_Id;

   procedure Set_Previous_CFG_Nodes
     (my_cfg_nodes 		: in out CFG_Nodes_Set)
   is
      my_iterator             	: CFG_Nodes_Iterator;
      a_cfg_node		: CFG_Node_Ptr;
   begin
      reset_iterator(my_cfg_nodes,my_iterator);
      loop
         current_element(my_cfg_nodes,a_cfg_node,my_iterator);

         for i in 0..CFG_Node_Ext_Ptr(a_cfg_node).next_nodes.Nb_Entries-1 loop

            Add_Previous_CFG_Node(my_cfg_nodes 		=> my_cfg_nodes,
                                  name            	=> CFG_Node_Ext_Ptr(a_cfg_node).next_nodes.Entries(i).name,
                                  previous_node_name  	=> a_cfg_node.name);
         end loop;

         exit when is_last_element (my_cfg_nodes, my_iterator);
         next_element (my_cfg_nodes, my_iterator);
      end loop;

   end Set_Previous_CFG_Nodes;

end CFG_Node_Set;
