with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Framework_Config;      use Framework_Config;
with CFGs;		    use CFGs;

with sets;
with CFG_Nodes; use CFG_Nodes;
with CFG_Edges; use CFG_Edges;

package CFG_Set is

   ----------------------------------------------------------
   -- Definition of basic block set
   ----------------------------------------------------------
   package CFG_Set is new Sets (
                                max_element    => Framework_Config.Max_CFGs,
                                element        => CFG_Ptr,
                                free           => Free,
                                copy           => Copy,
                                put            => Put,
                                xml_string     => XML_String,
                                xml_ref_string => XML_Ref_String);

   use CFG_Set;

   type CFGs_Set is new CFG_Set.set with private;

   subtype CFGs_Range is CFG_Set.element_range;
   subtype CFGs_Iterator is CFG_Set.iterator;

   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------

   -- Raised when a given CFG node is not in a CFG node set
   --
   CFG_Not_Found : exception;

   -- Raised when parameters provided to Add_Basic_Block are wrong
   --
   Invalid_Parameter : exception;

   ----------------------------------------------------
   -- CHECK
   ----------------------------------------------------

   procedure Check_CFG
     (name			: in Unbounded_String;
      cfg_nodes_tbl		: in CFG_Nodes_Table;
      cfg_edges_tbl		: in CFG_Edges_Table);

   ----------------------------------------------------
   -- ADD
   ----------------------------------------------------
   procedure Add_CFG
     (my_cfgs		: in out CFGs_Set;
      a_cfg		: in out CFG_Ptr);
   -- Add a CFG node pointer to a set

   procedure Add_CFG
     (my_cfgs			: in out CFGs_Set;
      a_cfg			: in out CFG_Ptr;
      name			: in Unbounded_String;
      cfg_nodes_tbl		: in CFG_Nodes_Table;
      cfg_edges_tbl		: in CFG_Edges_Table);

   procedure Add_CFG
     (my_cfgs			: in out CFGs_Set;
      name			: in Unbounded_String;
      cfg_nodes_tbl		: in CFG_Nodes_Table;
      cfg_edges_tbl		: in CFG_Edges_Table);

   ------------------------------------------------------
   -- SEARCH
   ------------------------------------------------------
   function Search_CFG
     (my_cfgs		: in CFGs_Set;
      name          	: in Unbounded_String)
      return		CFG_Ptr;
   --Search CFG node by name

   function Search_CFG_By_Id
     (my_cfgs 		: in CFGs_Set;
      id 		: in Unbounded_String)
      return            CFG_Ptr;
   --Search CFG node by cheddar_private_id

private
   type CFGs_Set is new CFG_Set.set with null record;

end CFG_Set;
