------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 534 $
--    $Date: 2012-10-05 19:46:28 +0200 (Fri, 05 Oct 2012) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Framework_Config;      use Framework_Config;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with deployments;use deployments;
with unbounded_strings;       use unbounded_strings;
with Scheduler_Interface;     use Scheduler_Interface;
with objects;use objects;
use objects.Generic_Object_Set_Package;
with sets;




package deployment_Set is

   ----------------------------------------------------------
   -- Definition of a set
   ----------------------------------------------------------

   package Generic_deployment_Set is new Sets (
      Max_Element => Framework_Config.Max_deployments,
      Element => generic_deployment_Ptr,
      Free => Free,
      Copy => Copy,
      Put => Put,
      xml_string => XML_String,
      xml_ref_string => XML_ref_String);
   use Generic_deployment_Set;

   type deployments_Set is new Generic_deployment_Set.set with private;

   subtype deployments_Range is Generic_deployment_Set.element_range;
   subtype deployments_Iterator is Generic_deployment_Set.iterator;

   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------
   -- Raised when a given deployment is not in a deployment set
   --
   deployment_Not_Found : exception;


   -- Raised when parameters provided to Add_deployment are wrong
   --
   Invalid_Parameter : exception;


   procedure Add_deployment 
     (My_deployments	: in out deployments_set;
      A_deployment          : in out generic_deployment_Ptr;
      Name              : in Unbounded_String;
resource_entities : in Generic_Objects_Set;
consumer_entities : in Generic_Objects_Set;
      allocation_description   : in Unbounded_String;
      capacity       : in Integer :=0;
      period         : in Integer :=0;
      priority       : in Integer :=0;
      Is_Preemptive     : in Preemptives_Type := preemptive;
      Quantum           : in Integer          := 0;
      File_Name         : in Unbounded_String := empty_string;
      A_Scheduler       : in Schedulers_Type  := No_Scheduling_Protocol;
      automaton_name    : in Unbounded_String := empty_string);


   procedure Add_deployment 
     (My_deployments	: in out deployments_set;
      Name              : in Unbounded_String;
resource_entities : in Generic_Objects_Set;
consumer_entities : in Generic_Objects_Set;
      allocation_description   : in Unbounded_String;
      capacity       : in Integer :=0;
      period         : in Integer :=0;
      priority       : in Integer :=0;
      Is_Preemptive     : in Preemptives_Type := preemptive;
      Quantum           : in Integer          := 0;
      File_Name         : in Unbounded_String := empty_string;
      A_Scheduler       : in Schedulers_Type  := No_Scheduling_Protocol;
      automaton_name    : in Unbounded_String := empty_string);


   procedure Check_deployment
     (My_deployments       : in deployments_Set;
      Name              : in Unbounded_String;
resource_entities : in Generic_Objects_Set;
consumer_entities : in Generic_Objects_Set;
      allocation_file   : in Unbounded_String;
      capacity       : in Integer;
      period         : in Integer;
      priority       : in Integer;
      Is_Preemptive     : in Preemptives_Type;
      Quantum           : in Integer;
      File_Name         : in Unbounded_String;
      A_Scheduler       : in Schedulers_Type;
      automaton_name    : in Unbounded_String);

   ------------------------------------------------------
   -- Read information from a set
   ------------------------------------------------------

   function Search_deployment
     (My_deployments 	: in deployments_Set;
      Name              : in Unbounded_String)
      return              Generic_deployment_Ptr;

   function Search_deployment_By_Processor_Name
     (My_deployments : in deployments_Set;
      Name              : in Unbounded_String)
      return              Generic_deployment_Ptr;

   function Search_deployment_By_Resource_Task_Name
     (My_deployments : in deployments_Set;
      Name              : in Unbounded_String)
      return              Generic_deployment_Ptr;

   function Search_deployment_By_Consumer_Task_Name
     (My_deployments : in deployments_Set;
      Name              : in Unbounded_String)
      return              Generic_deployment_Ptr;

   -- Delete a deployment in the set
   --
   procedure Delete_Deployment
     (my_deployments			: in out Deployments_Set;
      name 			: in Unbounded_String);



private

   type deployments_Set is new Generic_deployment_Set.set with null record;

end deployment_Set;

