------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 534 $
--    $Date: 2012-10-05 19:46:28 +0200 (Fri, 05 Oct 2012) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Framework_Config;      use Framework_Config;
with Event_Analyzers;       use Event_Analyzers;
with sets;

package Event_Analyzer_Set is

   ----------------------------------------------------------
   -- Definition of a set
   ----------------------------------------------------------

   package Generic_Event_Analyzer_Set is new sets (
      max_element    => Framework_Config.Max_Event_Analyzers,
      element        => Event_Analyzer_Ptr,
      free           => Free,
      copy           => Copy,
      put            => Put,
      xml_string     => XML_String,
      xml_ref_string => XML_String);

   use Generic_Event_Analyzer_Set;

   type Event_Analyzers_Set is new Generic_Event_Analyzer_Set.set with private;

   subtype Event_Analyzers_Range is Generic_Event_Analyzer_Set.element_range;
   subtype Event_Analyzers_Iterator is Generic_Event_Analyzer_Set.iterator;

   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------

   -- Raised when a given event analyzer is not in a event analyzer set
   --
   Event_Analyzer_Not_Found : exception;

   -- Raised when parameters provided to Add_event_analyzer are wrong
   --
   Invalid_Parameter : exception;

   ----------------------------------------------------------
   -- I/O operations
   ----------------------------------------------------------

   function Export_Aadl_Properties
     (My_Event_Analyzers : in Event_Analyzers_Set;
      Number_Of_Ht       : in Natural)
      return               Unbounded_String;

   ----------------------------------------------------------
   -- Procedure to proceed modification on a set
   ----------------------------------------------------------

   procedure Check_Event_Analyzer
     (My_Event_Analyzers : in Event_Analyzers_Set;
      Name          : in Unbounded_String;
      File_Name          : in Unbounded_String);

   procedure Add_Event_Analyzer
     (My_Event_Analyzers : in out Event_Analyzers_Set;
      Name               : in Unbounded_String;
      File_Name          : in Unbounded_String);

   ------------------------------------------------------
   -- Read information from a set
   ------------------------------------------------------

   -- Get the task pointer of the task named "Name"
   --
   function Search_Event_Analyzer
     (My_Event_Analyzers : in Event_Analyzers_Set;
      Name               : in Unbounded_String)
      return               Event_Analyzer_Ptr;

private

   type Event_Analyzers_Set is new Generic_Event_Analyzer_Set.set with null
record;

end Event_Analyzer_Set;
