------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Expressions;            use Expressions;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Unchecked_Deallocation;
with Interpreter;            use Interpreter;
use Interpreter.Sets_Type_Package;

package Event_Analyzers.extended is

   type extended_Event_Analyzer is new event_analyzer with record

   -- Event Analyzer behavior in string format
   --
      String_Behavior : Unbounded_String;

      -- Statements for each scheduling
      -- section (pointers given by the parser)
      --
      Root_Statement_Pointer : Section_Table;

      -- Variables table for the parsed parametric file
      -- (given by the parser)
      --
      Variables_Table : Variables_Table_Type;

      -- Table of set statements given by the parser
      --
      Sets_Table : Sets_Table_Type;
   end record;

   type extended_Event_Analyzer_Ptr is access all extended_Event_Analyzer'Class;


   procedure Free is new Unchecked_Deallocation (
      extended_Event_Analyzer'Class,
      extended_Event_Analyzer_Ptr);

   function Copy
     (My_Event_Analyzer : in extended_Event_Analyzer_Ptr)
      return              extended_Event_Analyzer_Ptr;


end Event_Analyzers.extended;
