------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- the LISyC Team, University of Western Britanny.
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of Cheddar 
-- and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
-----------------------------------------------------------------------------
-- Last update : 
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (Sun, 29 Jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Network;
USE Network;


PACKAGE Network.Predefined IS


   TYPE Predefined_Network IS NEW Generic_Network WITH PRIVATE;


   PROCEDURE Initialize (
         A_Network : IN OUT Predefined_Network);

   PROCEDURE Finalize (
         A_Network : IN OUT Predefined_Network);

   PROCEDURE Adjust (
         A_Network : IN OUT Predefined_Network);


   PROCEDURE Register_Message (
         A_Network     : IN OUT Predefined_Network;
         Message_Name  : IN     Unbounded_String;
         Response_Time : IN     Double);


   PROCEDURE Register_Messages (
         A_Network     : IN OUT Predefined_Network;
         Number        : IN     Response_Range;
         Response_Time : IN     Response_Table);


   PROCEDURE Dispatched_Read_From_File (
         My_Network : IN OUT Predefined_Network;
         File_Name  : IN     Unbounded_String);

   PROCEDURE Dispatched_Write_To_File (
         My_Network : IN     Predefined_Network;
         File_Name  : IN     Unbounded_String);

   PROCEDURE Dispatched_Reset (
         My_Network : IN OUT Predefined_Network);

   PROCEDURE Dispatched_Put (
         My_Network : IN     Predefined_Network);


   PROCEDURE Dispatched_Transmission_Delay (
         My_Network    : IN     Predefined_Network;
         My_Messages   : IN     Messages_Set;
         Response_Time :    OUT Response_Table);






PRIVATE


   TYPE Predefined_Network IS NEW Generic_Network WITH
      RECORD
         Number_Of_Messages : Response_Range := 0;
         Rt                 : Response_Table;
      END RECORD;


END Network.Predefined;
