-------------------------------------------------------------
-------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 297 $
--    $Date: 2010-11-23 13:39:03 +0100 (mar. 23 nov. 2010) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.IO_Exceptions;      use Ada.IO_Exceptions;
with GNAT.Current_Exception; use GNAT.Current_Exception;
with unbounded_strings;      use unbounded_strings;
with GNAT.Command_Line;      use GNAT.Command_Line;
with GNAT.OS_Lib;            use GNAT.OS_Lib;
with Text_IO;                use Text_IO;
with Version;                use Version;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Systems;                use Systems;
with Task_Set;               use Task_Set;
use Task_Set.Generic_Task_Set;
with Processor_Set;          use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Address_Space_Set;      use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Buffer_Set;             use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Call_Framework;         use Call_Framework;
with Tasks;                  use Tasks;
with unbounded_strings;      use unbounded_strings;
use unbounded_strings.unbounded_string_list_package;
with Translate;              use Translate;

package body applicability_constraint.all_tasks_are_periodic_or_sporadic is

   --Beginning of first unvariant part
   function type_of (t : Generic_Task_Ptr) return unbounded_string_list is
      list : unbounded_string_list;
      type elem_ptrs_range is range 1 .. 100;
      type unbounded_string_ptr_array is
        array (elem_ptrs_range) of unbounded_string_ptr;
      elem_ptrs : unbounded_string_ptr_array;
      i         : elem_ptrs_range := 1;
   begin
      initialize (list);
      --End of first unvariant part
      --For each entity's types and subtypes add :
      --the only part to modify is the to_unbounded_string(S:String) parameter
      elem_ptrs (i)     := new Unbounded_String;
      elem_ptrs (i).all := To_Unbounded_String ("OBJECTS.GENERIC_OBJECT");
      add (list, elem_ptrs (i));
      i := i + 1;

      elem_ptrs (i)     := new Unbounded_String;
      elem_ptrs (i).all := To_Unbounded_String ("TASKS.GENERIC_TASK");
      add (list, elem_ptrs (i));
      i := i + 1;

      elem_ptrs (i) := new Unbounded_String;

      case t.task_type is
         when Periodic_Type =>
            elem_ptrs (i).all := To_Unbounded_String ("TASKS.PERIODIC_TASK");
         when Aperiodic_Type =>
            elem_ptrs (i).all := To_Unbounded_String ("TASKS.APERIODIC_TASK");
         when Sporadic_Type =>
            elem_ptrs (i).all := To_Unbounded_String ("TASKS.SPORADIC_TASK");
         when Poisson_Type =>
            elem_ptrs (i).all := To_Unbounded_String ("TASKS.POISSON_TASK");
         when Scheduling_Task_Type =>
            elem_ptrs (i).all :=
               To_Unbounded_String ("TASKS.SCHEDULING_TASK");
         when Parametric_Type =>
            elem_ptrs (i).all :=
               To_Unbounded_String ("TASKS.PARAMETRIC_TASK");
         when Frame_Task_Type =>
            null;
      end case;
      add (list, elem_ptrs (i));
      --Beginning of second unvariant part
      return list;
   end type_of;
   --End of second unvariant part

   function R8_QUERY1_Condition (t : Generic_Task_Ptr) return Boolean is
   begin

      return not ((element_in_list
                      (To_Unbounded_String ("TASKS.PERIODIC_TASK"),
                       type_of (t))) or
                  (element_in_list
                      (To_Unbounded_String ("TASKS.SPORADIC_TASK"),
                       type_of (t))));

   end R8_QUERY1_Condition;

   function R8 (Sys : System) return Boolean is
   begin
      Context := Sys;
      return (get_number_of_elements
                 (select_and_copy (Sys.Tasks, R8_QUERY1_Condition'Access)) =
              Generic_Task_Set.element_range (0));

   end R8;

   function R8_Txt return Unbounded_String is
   begin

      return ("The constraint R8 is not met:") &
             unbounded_lf &
             ("tasks must be periodic or sporadic.") &
             unbounded_lf;

   end R8_Txt;

end applicability_constraint.all_tasks_are_periodic_or_sporadic;
