------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 297 $
--    $Date: 2010-11-23 13:39:03 +0100 (mar. 23 nov. 2010) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.IO_Exceptions;      use Ada.IO_Exceptions;
with GNAT.Current_Exception; use GNAT.Current_Exception;
with Unbounded_Strings;      use Unbounded_Strings;
with GNAT.Command_Line;      use GNAT.Command_Line;
with GNAT.OS_Lib;            use GNAT.OS_Lib;
with Text_IO;                use Text_IO;
with Version;                use Version;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Systems;                use Systems;
with Task_Set;               use Task_Set;
with Processor_Set;          use Processor_Set;
with Address_Space_Set;      use Address_Space_Set;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Resources;              use Resources;
use Resources.resource_accesses;
with Buffer_Set;             use Buffer_Set;
with Call_Framework;         use Call_Framework;
with Framework_Config;       use Framework_Config;
with Tasks;                  use Tasks;
use Task_Set.Generic_Task_Set;

package body applicability_constraint.Ceiling_Priority_Assignment is

   function R13_QUERY2_TEMP (r : Generic_Resource_Ptr) return Boolean is
      res       : Boolean := False;
      item      : resource_accesses_Range;
      range_end : resource_accesses_Range;
   begin

      range_end := r.critical_sections.Nb_Entries;
      item      := 0;
      loop
         if (r.protocol /= No_Protocol) and (r.protocol /= Priority_Inheritance_Protocol) then
            if Search_Task (Context.Tasks, (r.critical_sections.Entries (item).Item)).
                 priority >
              -- Priority_Constrained_Resource_Ptr (r).ceiling_priority
 		Generic_Resource_Ptr (r).priority
            then
               res := True;
            end if;
         end if;
         item := item + 1;

         exit when item >= range_end;
      end loop;
      return res;

   end R13_QUERY2_TEMP;

   function R12_QUERY1_Condition (r : Generic_Resource_Ptr) return Boolean is
   begin

      return ((r.protocol = Priority_Ceiling_Protocol) or
              (r.protocol = Immediate_Priority_Ceiling_Protocol)) and
             R13_QUERY2_TEMP (r);

   end R12_QUERY1_Condition;

   function R12 (Sys : System) return Boolean is
   begin
      Context := Sys;
      return (Get_Number_Of_Elements
                 (Select_And_Copy
                     (Context.Resources,
                      R12_QUERY1_Condition'Access)) =
              Resources_Range (0));

   end R12;


   function R12_Txt  return Unbounded_String is
   begin
     
      return ("The constraint R12 is not met:")& unbounded_lf &
	      ("if PCP or IPCP are used, data’s Ceiling priority must be")& unbounded_lf &
 ("higher or equal to all priorities of data-component dependent tasks.") & unbounded_lf ;

   end R12_Txt;

end applicability_constraint.Ceiling_Priority_Assignment;
