------------------------------------------------------
--------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 297 $
--    $Date: 2010-11-23 13:39:03 +0100 (mar. 23 nov. 2010) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.IO_Exceptions;      use Ada.IO_Exceptions;
with GNAT.Current_Exception; use GNAT.Current_Exception;
with unbounded_strings;      use unbounded_strings;
with GNAT.Command_Line;      use GNAT.Command_Line;
with GNAT.OS_Lib;            use GNAT.OS_Lib;
with Text_IO;                use Text_IO;
with Version;                use Version;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Systems;                use Systems;
with Task_Set;               use Task_Set;
with Processor_Set;          use Processor_Set;
with Address_Space_Set;      use Address_Space_Set;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Resources;              use Resources;
with Buffer_Set;             use Buffer_Set;
with Call_Framework;         use Call_Framework;
with Tasks;                  use Tasks;
use Task_Set.Generic_Task_Set;

package body applicability_constraint.Pip_No_Deadlock is

   function R13_QUERY3_TEMP
     (r2   : Generic_Resource_Ptr;
      r1   : Generic_Resource_Ptr)
      return Boolean
   is
      res : Boolean := False;
   begin

      return res;

   end R13_QUERY3_TEMP;

   function R13_QUERY2_Condition
     (r2   : Generic_Resource_Ptr;
      r1   : Generic_Resource_Ptr)
      return Boolean
   is
   begin

      return ((r2.protocol = Priority_Inheritance_Protocol) and
              (r2 /= r1) and
              R13_QUERY3_TEMP (r2, r1));

   end R13_QUERY2_Condition;

   function R13_QUERY1_Condition (r1 : Generic_Resource_Ptr) return Boolean is
   begin

      return (get_number_of_elements
                 (select_and_copy
                     (Context.Resources,
                      R13_QUERY2_Condition'Access,
                      r1)) >
              Resources_Range (0));

   end R13_QUERY1_Condition;

   function R13 (Sys : System) return Boolean is
   begin
      Context := Sys;
      return (get_number_of_elements
                 (select_and_copy
                     (Context.Resources,
                      R13_QUERY1_Condition'Access)) =
              Resources_Range (0));

   end R13;

   function R13_Txt return Unbounded_String is
   begin

      return ("The constraint R13 is not met:") &
             unbounded_lf &
             (
"if PIP is used, dependent tasks cannot shared more than one data component.")
             &
             unbounded_lf;

   end R13_Txt;

end applicability_constraint.Pip_No_Deadlock;
