------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 297 $
--    $Date: 2010-11-23 13:39:03 +0100 (mar. 23 nov. 2010) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.IO_Exceptions;      use Ada.IO_Exceptions;
with GNAT.Current_Exception; use GNAT.Current_Exception;
with Unbounded_Strings;      use Unbounded_Strings;
with GNAT.Command_Line;      use GNAT.Command_Line;
with GNAT.OS_Lib;            use GNAT.OS_Lib;
with Text_IO;                use Text_IO;
with Version;                use Version;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Systems;                use Systems;
with Task_Set;               use Task_Set;
use Task_Set.Generic_Task_Set;
with Processor_Set;          use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Address_Space_Set;      use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Buffer_Set;             use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Call_Framework;         use Call_Framework;
with Tasks;                  use Tasks;
with Unbounded_Strings;      use Unbounded_Strings;
use Unbounded_Strings.Unbounded_String_List_Package;
with Unbounded_Strings;      use Unbounded_Strings;
use Unbounded_Strings.Unbounded_String_List_Package;
with Processors; use Processors;
with Scheduler; use  Scheduler;
with Scheduler_Interface;                                              use 
  Scheduler_Interface;
with translate; use translate;

package body applicability_constraint.Uniprocessor_Scheduling_Protocol is

   function R2 (Sys : System) return Boolean is
      p : Generic_Processor_Ptr;
      p2 : Mono_Core_Processor_Ptr;
   begin

      	 Get_Element_Number (Sys.Processors, p, Processors_Range (0));
	 p2 :=Mono_Core_Processor_Ptr (p);
      return  (p2.core.scheduling.scheduler_type =
              Earliest_Deadline_First_Protocol) or
		(p2.core.scheduling.scheduler_type =
              Rate_Monotonic_Protocol) or
		(p2.core.scheduling.scheduler_type =
              Posix_1003_Highest_Priority_First_Protocol) or
		(p2.core.scheduling.scheduler_type =
              Deadline_Monotonic_Protocol);

   end R2;

   function R2_Txt return Unbounded_String is
   begin
     
      return ("The environmental constraint R2 is not met:")& unbounded_lf &
	      ("scheduling policy must be in Fixed Priorities, Rate Monotonic,")& unbounded_lf &
 	      ("or Earliest Deadline First or Deadline Monotonic.") & unbounded_lf ;

   end R2_Txt;

end applicability_constraint.Uniprocessor_Scheduling_Protocol;
