
with Translate;           use Translate;
with unbounded_strings;   use unbounded_strings;
with Scheduling_Analysis; use Scheduling_Analysis;
use Scheduling_Analysis.Double_Tasks_Parameters_Package;
with feasibility_test.processor_utilization;
use feasibility_test.processor_utilization;
with scheduler_interface; use scheduler_interface;
with systems;
use systems;
with feasibility_test.periodic_task_worst_case_response_time_dynamic_priority; 
use feasibility_test.periodic_task_worst_case_response_time_dynamic_priority; 
with feasibility_test.periodic_task_worst_case_response_time_fixed_priority;
use feasibility_test.periodic_task_worst_case_response_time_fixed_priority;
with Scheduler.dynamic_Priority;
use Scheduler.dynamic_Priority;
with Scheduler.Fixed_Priority.Dm;                       
use Scheduler.Fixed_Priority.Dm;
with Scheduler.Fixed_Priority.Rm;                       
use Scheduler.Fixed_Priority.Rm;
with Scheduler.Fixed_Priority.Hpf;                      
use Scheduler.Fixed_Priority.Hpf;
with Cache_Set; 		use Cache_Set;
with Caches; 			use Caches;
with Call_Framework_Interface; 	use Call_Framework_Interface;
with Scheduler.Fixed_Priority; 	use Scheduler.Fixed_Priority;



package body feasibility_test.periodic_task_worst_case_response_time  is


   procedure Compute_Response_Time
     (My_Scheduler   			: in generic_Scheduler_ptr;
      My_Tasks       			: in out Tasks_Set;
      Processor_Name 			: in Unbounded_String;
      Msg            			: in out Unbounded_String;
      Response_Time  			: out Response_Time_Table;
      With_CRPD 			: in Boolean 				:= false;
      CRPD_Computation_Approach 	: in CRPD_Computation_Approach_Type 	:= ECB_Only;
      Block_Reload_Time 		: in Natural 				:= 0;
      My_Cache_Access_Profiles	     	: in Cache_Access_Profiles_Set 		:= no_cache_access_profile)
   is
   begin
      case my_scheduler.parameters.scheduler_type is
         when Rate_Monotonic_Protocol =>
            Compute_response_Time(rm_scheduler(my_scheduler.All), 
                                  my_tasks, 
                                  processor_name, 
                                  msg, 
                                  response_time,
                                  With_CRPD,
                                  CRPD_Computation_Approach,
                                  Block_Reload_Time, 
                                  My_Cache_Access_Profiles);
         when Deadline_Monotonic_Protocol =>
            Compute_response_Time(dm_scheduler(my_scheduler.All), 
                                  my_tasks, 
                                  processor_name, 
                                  msg, 
                                  response_time,
                                  With_CRPD,
                                  CRPD_Computation_Approach,
                                  Block_Reload_Time, 
                                  My_Cache_Access_Profiles);
         when Posix_1003_Highest_Priority_First_Protocol =>
            Compute_response_Time(hpf_scheduler(my_scheduler.All), 
                                  my_tasks, 
                                  processor_name, 
                                  msg, 
                                  response_time,
                                  With_CRPD,
                                  CRPD_Computation_Approach,
                                  Block_Reload_Time, 
                                  My_Cache_Access_Profiles);
         when Earliest_Deadline_First_Protocol |
              Least_Laxity_First_Protocol |
              D_Over_Protocol |
              Maximum_Urgency_First_Based_On_Laxity_Protocol |
              Maximum_Urgency_First_Based_On_Deadline_Protocol =>
            Compute_response_Time(dynamic_priority_scheduler(my_scheduler.All), 
                                  my_tasks,
                                  processor_name, 
                                  msg, 
                                  response_time);
         when others =>
            raise Invalid_Scheduler;
      end case;
   end Compute_Response_Time;
   
  
end feasibility_test.periodic_task_worst_case_response_time;



