with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Messages;              use Messages;
with Message_Set;           use Message_Set;
use Message_Set.Generic_Message_Set;
with Buffers;               use Buffers;
with Buffer_Set;            use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;          use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Network_Set;           use Network_Set;
use Network_Set.Generic_Network_Set;
with Event_Analyzer_Set;    use Event_Analyzer_Set;
use Event_Analyzer_Set.Generic_Event_Analyzer_Set;
with deployment_Set;          use deployment_Set;
with unbounded_strings;     use unbounded_strings;
use unbounded_strings.unbounded_string_list_package;
with CFG_Node_Set.Atomic_Operation_Set; use CFG_Node_Set.Atomic_Operation_Set;
with Atomic_Operations; 		use Atomic_Operations;
with Partitioning_Algorithm_Set;		use Partitioning_Algorithm_Set;



package Partition_Algorithms  is

   type Partition_Algorithm is tagged record
      CFG_Nodes       	: CFG_Nodes_Set;
      Partitioning_Algorithms : Partitioning_Algorithms_Set;
      Atomic_Operations	      : Atomic_Operations_Set;
      Partitioning_Algorithm :Partitioning_Algorithm_Ptr;
   end record;

   type AO_Ptr is access Partition_Algorithm;

   procedure Initialize (A_AO : in out Partition_Algorithm);

   procedure Initialize (A_AO : in AO_Ptr);

   procedure Delete_Partition_Algorithms
     (A_AO : in out Partition_Algorithm;
      A_Partitioning_Algorithm : Partitioning_Algorithm_Ptr);

   function XML_String
     (obj   : in Partition_Algorithm;
      level : in Natural := 0)
      return  Unbounded_String;
   function XML_String
     (obj   : in AO_Ptr;
      level : in Natural := 0)
      return  Unbounded_String;

   -- Display a system to the screen in different languages
   --
   procedure Put_Xml
     (A_AO               : in Partition_Algorithm);


   -- I/O sub-programs : read/write a system in different languages
   --


   procedure Read_From_Xml_File
     (A_AO  : in out Partition_Algorithm;
      Dir_List  : in unbounded_string_list;
      File_Name : in Unbounded_String);
   procedure Read_From_Xml_File
     (A_AO  : in out Partition_Algorithm;
      Dir_List  : in unbounded_string_list;
      File_Name : in String);

   procedure Write_To_Xml_File
     (A_AO                : in Partition_Algorithm;
      File_Name               : in Unbounded_String);
   procedure Write_To_Xml_File
     (A_AO                : in Partition_Algorithm;
      File_Name               : in String);

end Partition_Algorithms ;
