------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Finalization;
with Systems;           use Systems;
with Processor_Set;     use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with unbounded_strings; use unbounded_strings;
use unbounded_strings.unbounded_string_list_package;

package AADL_Parsers is

   -- AADL parser
   --
   type AADL_Project_Parser is new Ada.Finalization.Controlled with private;

   -- Methods for all Cheddar AADL parsers
   --
   procedure Initialize_Project_Parser
     (Handler : in out AADL_Project_Parser);

   function Get_Parsed_System (Handler : AADL_Project_Parser) return System;

   procedure Initialize (Handler : in out AADL_Project_Parser);

   procedure Parse
     (Handler           : in out AADL_Project_Parser;
      Dir_List          : unbounded_string_list;
      Project_File_List : unbounded_string_list);

   -- Exception raised by clients of AADL_Parser when something goes
   -- wrong during AADL parsing
   --
   AADL_Read_Error : exception;

private

   type AADL_Project_Parser is new Ada.Finalization.Controlled with record
      Parsed_System : System;
   end record;

end AADL_Parsers;
