------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 395 $
--    $Date: 2011-04-11 20:43:38 +0200 (lun., 11 avr. 2011) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Sax.Exceptions;
with Sax.Locators;
with Sax.Readers;
with Sax.Attributes;
with Unicode.CES;
with Systems;                           use Systems;
with Processor_Set;                     use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Scheduling_Analysis;               use Scheduling_Analysis;
with Multiprocessor_Services_Interface; use Multiprocessor_Services_Interface;
use Multiprocessor_Services_Interface.Scheduling_Result_Per_Processor_Package;

package Xml_generic_Parsers.architecture is

   type Xml_architecture_Parser is new Xml_Generic_Parser with private;

   -- Override methods of the Reader to custom the treatement the
   -- parser have to do
   --
   procedure Start_Document (Handler : in out Xml_architecture_Parser);
   procedure Start_Element
     (Handler       : in out Xml_architecture_Parser;
      Namespace_Uri : Unicode.CES.Byte_Sequence := "";
      Local_Name    : Unicode.CES.Byte_Sequence := "";
      Qname         : Unicode.CES.Byte_Sequence := "";
      Atts          : Sax.Attributes.Attributes'Class);
   procedure End_Element
     (Handler       : in out Xml_architecture_Parser;
      Namespace_Uri : Unicode.CES.Byte_Sequence := "";
      Local_Name    : Unicode.CES.Byte_Sequence := "";
      Qname         : Unicode.CES.Byte_Sequence := "");

   procedure Initialize (Handler : in out Xml_architecture_Parser);
   function Get_Parsed_System
     (Handler : in Xml_architecture_Parser)
      return    System;

private

   -- At the end of the parser execution, the architecture
   -- is stored in the object defined below
   --
   type Xml_architecture_Parser is new Xml_Generic_Parser with record
      Parsed_System : System;
   end record;

end Xml_generic_Parsers.architecture;
