-------------------------------------------------------
-------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 395 $
--    $Date: 2011-04-11 20:43:38 +0200 (lun., 11 avr. 2011) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Sax.Exceptions;
with Sax.Locators;
with Sax.Readers;
with Sax.Attributes;
with Unicode.CES;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package Xml_generic_Parsers is

   ----------------------------------------------------------------------------
   ----------------    Mandatory code for all XML parser of Cheddar
   ----------------------------------------------------------------------------

   -- Exception raised by clients of Xml_Parses when something goes
   -- wrong during XML parsing
   --
   Xml_Read_Error : exception;

   -- Types and constants to parser multiple values between tags
   --
   Max_Xml_Tag_Parameters : constant Natural := 10000;
   type parameter_list_type is
     array (1 .. Max_Xml_Tag_Parameters) of Unbounded_String;

   -- Generic XML parser
   --
   type Xml_Generic_Parser is new Sax.Readers.Reader with record

   -- Parser locator
   --
      Locator : Sax.Locators.Locator;

      -- Store values between each tag
      --
      Current_Parameter : Integer;
      Parameter_List    : parameter_list_type;

      -- Variable for type conversion
      --
      ok : Boolean;

   end record;

   -- Methods for all Cheddar XML parsers
   --
   procedure Characters
     (Handler : in out Xml_Generic_Parser;
      Ch      : Unicode.CES.Byte_Sequence);
   procedure Error
     (Handler : in out Xml_Generic_Parser;
      Except  : Sax.Exceptions.Sax_Parse_Exception'Class);
   procedure Fatal_Error
     (Handler : in out Xml_Generic_Parser;
      Except  : Sax.Exceptions.Sax_Parse_Exception'Class);
   procedure Warning
     (Handler : in out Xml_Generic_Parser;
      Except  : Sax.Exceptions.Sax_Parse_Exception'Class);

   -- Method to parse Index value
   --
   procedure End_Element
     (Handler       : in out Xml_Generic_Parser;
      val           : in out Unbounded_String;
      Namespace_Uri : Unicode.CES.Byte_Sequence := "";
      Local_Name    : Unicode.CES.Byte_Sequence := "";
      Qname         : Unicode.CES.Byte_Sequence := "");
   procedure End_Element
     (Handler       : in out Xml_Generic_Parser;
      val           : in out Integer;
      Namespace_Uri : Unicode.CES.Byte_Sequence := "";
      Local_Name    : Unicode.CES.Byte_Sequence := "";
      Qname         : Unicode.CES.Byte_Sequence := "");

end Xml_generic_Parsers;
