------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Interpreter;                       use Interpreter;
use Interpreter.Sets_Type_Package;
with Systems;                           use Systems;
with Processors;                        use Processors;
with Buffer_Set;                        use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Message_Set;                       use Message_Set;
use Message_Set.Generic_Message_Set;
with Event_Analyzers;                   use Event_Analyzers;
with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
with Framework_Config;                  use Framework_Config;
with Scheduling_Analysis;               use Scheduling_Analysis;
with Scheduling_Analysis.extended;      use Scheduling_Analysis.extended;
with Task_Dependencies;                 use Task_Dependencies;
with indexed_tables;
with Unchecked_Deallocation;
with Processor_Set;                     use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Address_Space_Set;                 use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Time_Unit_Events;                  use Time_Unit_Events;
use Time_Unit_Events.Time_Unit_Package;
with Task_Set;                          use Task_Set;
use Task_Set.Generic_Task_Set;
with Resources;                         use Resources;
use Resources.resource_accesses;
with Resource_Set;                      use Resource_Set;
use Resource_Set.Generic_Resource_Set;

package partitioning_Services is

   
  
   --------------------------------------------------------------
   -- Partitionning tools  : these subprograms allow users to
   -- split tasks on a set of processors
   --------------------------------------------------------------

   -- All processors should have the same scheduler
   --
   Processors_Should_Have_The_Same_Scheduler : exception;

   -- We need more processors to do partition of this task set
   --
   No_Such_Processors : exception;

   -- can we applied scheduling partition algorithms with this scheduler ?
   --
   Invalid_Scheduler_Multiprocessors : exception;

   procedure Partition_General_Task
     (Src_Processors : in Processors_Set;
      Src_Tasks      : in Tasks_Set;
      Msg            : in out Unbounded_String;
      Result_Tasks   : in out Tasks_Set);

   procedure Partition_Small_Task
     (Src_Processors : in Processors_Set;
      Src_Tasks      : in Tasks_Set;
      Msg            : in out Unbounded_String;
      Result_Tasks   : in out Tasks_Set);

   procedure Partition_Small_Task
     (Src_Processors   : in Processors_Set;
      Src_Tasks        : in Tasks_Set;
      Msg              : in out Unbounded_String;
      Result_Tasks     : in out Tasks_Set;
      Processor_Result : out Processors_Iterator);

   procedure Partition_Small_Task
     (Src_Processors   : in Processors_Set;
      Result_Tasks     : in out Tasks_Set;
      Processor_Result : out Processors_Iterator);

   procedure Rm_General_Tasks
     (Src_Processors : in Processors_Set;
      Result_Tasks   : in out Tasks_Set);

   procedure Partition_Next_Fit
     (Src_Processors : in Processors_Set;
      Src_Tasks      : in Tasks_Set;
      Msg            : in out Unbounded_String;
      Result_Tasks   : in out Tasks_Set);

   procedure Partition_First_Fit
     (Src_Processors : in Processors_Set;
      Src_Tasks      : in Tasks_Set;
      Msg            : in out Unbounded_String;
      Result_Tasks   : in out Tasks_Set);

   procedure Partition_Best_Fit
     (Src_Processors : in Processors_Set;
      Src_Tasks      : in Tasks_Set;
      Msg            : in out Unbounded_String;
      Result_Tasks   : in out Tasks_Set);

   procedure Validate_Multiprocessor_Tasks
     (Src_Processors : in Processors_Set;
      Src_Tasks      : in Tasks_Set);

end partitioning_Services;


