with Ada.Strings.Unbounded; 		use Ada.Strings.Unbounded;
with Text_IO;             		use Text_IO;
with unbounded_strings;   		use unbounded_strings;
with Scheduling_Analysis; 		use Scheduling_Analysis;
with Priority_Assignment.Audsley_OPA;	use Priority_Assignment.Audsley_OPA;
with Task_Set; 				use Task_Set;
with Tasks; 				use Tasks;
with Tasks.Extended; 			use Tasks.Extended;
with Integer_Arrays; 			use Integer_Arrays;
with Priority_Assignment.Utility; 	use Priority_Assignment.Utility;
with Cache_Set; 			use Cache_Set;
with Cache_Block_Set;			use Cache_Block_Set;
with Cache_Access_Profile_Set;		use Cache_Access_Profile_Set;

package Priority_Assignment.Audsley_OPA_CRPD is

   procedure Check_Input_Data
     (my_tasks : in Tasks_Set;
      my_cache_access_profiles : in Cache_Access_Profiles_Set);

   type CRPD_Inteference_Computation_Complexity is (ECB_Only,
                                                    PT_Simplified,
                                                    PT_Binomial_Coefficient,
                                                    Tree);
   --------------------------------
   --Perform the Audsley's optimal priority assignment with CRPD taken into account
   --Return the task set with assigned task priorities
   --------------------------------

   -- High level call to Audsley OPA CRPD
   --
   procedure Set_Priority_According_To_Audsley_OPA_CRPD
     (My_Tasks       		: in out Tasks_Set;
      My_Cache_Access_Profiles 	: in Cache_Access_Profiles_Set;
      Computation_Complexity    : in CRPD_Inteference_Computation_Complexity := PT_Simplified;
      Processor_Name  		: in Unbounded_String := empty_string);

   -- Lower level call to Audsley OPA CRPD
   -- Perform computation and assign priorities to task
   --
   procedure OPA_CRPD
     (my_tasks 			: in out Tasks_Set;
      my_cache_access_profiles 	: in Cache_Access_Profiles_Set;
      complexity		: in CRPD_Inteference_Computation_Complexity := PT_Simplified);


end Priority_Assignment.Audsley_OPA_CRPD;
