----------------------------------------------------------------------
----------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Qs_Tools; use Qs_Tools;

package body Queueing_System.Simulation.Mp1 is

   procedure Initialize
     (A_Queueing_System : in out Mp1_Queueing_System_Simulation)
   is
   begin
      Initialize (Generic_Queueing_System_Simulation (A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Mp1;
   end Initialize;

   function Compute_Service_Time
     (A_Queueing_System : in Mp1_Queueing_System_Simulation;
      Seed              : in Generator)
      return              Double
   is
      Prev_Act : Double := 0.0;
      Next_Act : Double := 0.0;

      Prev_Ri_Index : Resp_Time_Consumer_Range;
      Next_Ri_Index : Resp_Time_Consumer_Range;

      Prev_Ri : Double := 0.0;
      Next_Ri : Double := 0.0;

      Mst          : constant Double  :=
        1.0 / A_Queueing_System.Service_Rate.entries (0).data;
      T_Arr        : constant Double  := A_Queueing_System.T_Arr;
      Empty_Buffer : constant Boolean := A_Queueing_System.Empty_Buffer;
      Result       : Double           := 0.0;
   begin
      Prev_Act := Double'Floor (T_Arr / Mst);
      Next_Act := Prev_Act + 1.0;

      Prev_Ri_Index :=
        Resp_Time_Consumer_Range (my_mod
                                     (Prev_Act,
                                      Double (
        A_Queueing_System.Consumer_Resp_Time.nb_entries)));

      Next_Ri_Index :=
        Resp_Time_Consumer_Range (my_mod
                                     (Next_Act,
                                      Double (
        A_Queueing_System.Consumer_Resp_Time.nb_entries)));

      Prev_Ri :=
        A_Queueing_System.Consumer_Resp_Time.entries (Prev_Ri_Index).data;
      Next_Ri :=
        A_Queueing_System.Consumer_Resp_Time.entries (Next_Ri_Index).data;

      if Empty_Buffer then
         if T_Arr < (Prev_Act * Mst + Prev_Ri) then
            Result := (Prev_Act * Mst + Prev_Ri) - T_Arr;
         else
            Result := (Next_Act * Mst + Next_Ri) - T_Arr;
         end if;
      else
         Result := Next_Ri + Mst - Prev_Ri;
      end if;

      return Result;
   end Compute_Service_Time;

   function Compute_Inter_Arrival_Time
     (A_Queueing_System : in Mp1_Queueing_System_Simulation;
      Seed              : in Generator)
      return              Double
   is
      Result       : Double          := 0.0;
      Arrival_Rate : constant Double :=
        A_Queueing_System.Arrival_Rate.entries (0).data;
   begin
      Result := Get_Exponential_Time (1.0 / Arrival_Rate, Seed);
      return Result;
   end Compute_Inter_Arrival_Time;

end Queueing_System.Simulation.Mp1;
