--------------------------------------------------
------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
package body Queueing_System.theoretical.Pp1 is

   procedure Initialize
     (A_Queueing_System : in out Pp1_Queueing_System_theoretical)
   is
   begin
      Initialize (Generic_Queueing_System_theoretical (A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Pp1;
   end Initialize;

   procedure Qs_Maximum_Waiting_Time
     (A_Queueing_System : in Pp1_Queueing_System_theoretical;
      Result            : in out Double)
   is

      Service_Rate : constant Double :=
        A_Queueing_System.Service_Rate.entries (0).data;
      Nb_Arrival   : constant Double := A_Queueing_System.Nb_Arrival;
   begin
      if A_Queueing_System.Harmonic then
         Result := (2.0 * Nb_Arrival) * (1.0 / Service_Rate);
      else
         Result := (2.0 * Nb_Arrival + 1.0) * (1.0 / Service_Rate);
      end if;
   end Qs_Maximum_Waiting_Time;

   procedure Qs_Maximum_Number_Customer
     (A_Queueing_System : in Pp1_Queueing_System_theoretical;
      Result            : in out Double)
   is

      Nb_Arrival : constant Double  := A_Queueing_System.Nb_Arrival;
      Harmonic   : constant Boolean := A_Queueing_System.Harmonic;
   begin
      if Harmonic then
         Result := 2.0 * Nb_Arrival;
      else
         Result := 2.0 * Nb_Arrival + 1.0;
      end if;
   end Qs_Maximum_Number_Customer;

   procedure Qs_Average_Waiting_Time
     (A_Queueing_System : in Pp1_Queueing_System_theoretical;
      Result            : in out Double)
   is
   begin
      raise Not_Implemented;
   end Qs_Average_Waiting_Time;

   procedure Qs_Average_Number_Customer
     (A_Queueing_System : in Pp1_Queueing_System_theoretical;
      Result            : in out Double)
   is
   begin
      raise Not_Implemented;
   end Qs_Average_Number_Customer;

   function Get_Probability_Of_State
     (A_Queueing_System : in Pp1_Queueing_System_theoretical;
      N                 : in Natural)
      return              Double
   is
   begin
      raise Not_Implemented;
      return 0.0;
   end Get_Probability_Of_State;

end Queueing_System.theoretical.Pp1;
