-----------------------------------------
---------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

package body Automaton.extended is

   function Search_transition
     (a_list : in Transition_Lists;
      Name   : in Unbounded_String)
      return   Transition_Ptr
   is
      result       : Transition_Ptr := null;
      A_transition : Transition_Ptr;
      List_Ite     : Transition_Lists_Iterator;

   begin

      if not is_empty (a_list) then

         reset_head_iterator (a_list, List_Ite);
         loop
            current_element (a_list, A_transition, List_Ite);

            if A_transition.name = Name then
               result := A_transition;
            end if;

            if is_tail_element (a_list, List_Ite) then
               exit;
            end if;
            next_element (a_list, List_Ite);
         end loop;

      end if;

      if result /= null then
         return result;
      else
         raise transition_not_found;
      end if;

   end Search_transition;

   function Search_state
     (a_list : in State_Lists;
      Name   : in Unbounded_String)
      return   State_Ptr
   is
      result   : State_Ptr := null;
      A_state  : State_Ptr;
      List_Ite : State_Lists_Iterator;

   begin

      if not is_empty (a_list) then

         reset_head_iterator (a_list, List_Ite);
         loop
            current_element (a_list, A_state, List_Ite);

            if A_state.name = Name then
               result := A_state;
            end if;

            if is_tail_element (a_list, List_Ite) then
               exit;
            end if;
            next_element (a_list, List_Ite);
         end loop;

      end if;

      if result /= null then
         return result;
      else
         raise state_not_found;
      end if;

   end Search_state;

   function Search_initial_state (a_list : in State_Lists) return State_Ptr is
      result   : State_Ptr := null;
      A_state  : State_Ptr;
      List_Ite : State_Lists_Iterator;

   begin

      if not is_empty (a_list) then

         reset_head_iterator (a_list, List_Ite);
         loop
            current_element (a_list, A_state, List_Ite);

            if A_state.is_initial then
               result := A_state;
            end if;

            if is_tail_element (a_list, List_Ite) then
               exit;
            end if;
            next_element (a_list, List_Ite);
         end loop;

      end if;

      if result /= null then
         return result;
      else
         raise state_not_found;
      end if;

   end Search_initial_state;

end Automaton.extended;
