------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Interpreter;                       use Interpreter;
use Interpreter.Sets_Type_Package;
with Systems;                           use Systems;
with Processors;                        use Processors;
with Buffer_Set;                        use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Message_Set;                       use Message_Set;
use Message_Set.Generic_Message_Set;
with Event_Analyzers;                   use Event_Analyzers;
with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
with Framework_Config;                  use Framework_Config;
with Scheduling_Analysis;               use Scheduling_Analysis;
with Scheduling_Analysis.extended;      use Scheduling_Analysis.extended;
with Task_Dependencies;                 use Task_Dependencies;
with indexed_tables;
with Unchecked_Deallocation;
with Processor_Set;                     use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Address_Space_Set;                 use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Time_Unit_Events;                  use Time_Unit_Events;
use Time_Unit_Events.Time_Unit_Package;
with Task_Set;                          use Task_Set;
use Task_Set.Generic_Task_Set;
with Resources;                         use Resources;
use Resources.resource_accesses;
with Resource_Set;                      use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Multiprocessor_Services_Interface; use Multiprocessor_Services_Interface;
use Multiprocessor_Services_Interface.Scheduling_Result_Per_Processor_Package;

package Multiprocessor_Services is

   procedure Free (Sched : in out Scheduling_Table_Ptr);

   procedure Display_Scheduling (Sched : in Scheduling_Table_Ptr);

   ------------------------------------------------------
   -- Write/read the event table into/from a XML file
   ------------------------------------------------------

   procedure Write_To_Xml_File
     (Sched     : in Scheduling_Table_Ptr;
      Sys       : in System;
      File_Name : in String);

   procedure Write_To_Xml_File
     (Sched     : in Scheduling_Table_Ptr;
      Sys       : in System;
      File_Name : in Unbounded_String);

   procedure Read_From_Xml_File
     (Sched     : in out Scheduling_Table_Ptr;
      Sys       : in System;
      File_Name : in String);

   procedure Read_From_Xml_File
     (Sched     : in out Scheduling_Table_Ptr;
      Sys       : in System;
      File_Name : in Unbounded_String);


   -----------------------------------------------------------------
   --  This function is the major entry of the scheduling simulator
   -----------------------------------------------------------------

   procedure Build_Multiprocessor_Scheduling
     (Sys : in system; 
      Result                     : in out Scheduling_Table_Ptr;
      Event_To_Generate          : in Time_Unit_Event_Type_Boolean_Table;
      Last_Time                  : in Natural;
      With_Offsets               : in Boolean := True;
      With_Precedencies          : in Boolean := True;
      With_Resources             : in Boolean := True;
      With_jitters               : in Boolean := True;
      With_minimize_preemption   : in Boolean := True;
      With_Task_Specific_Seed    : in Boolean := True;
      Global_Seed_Value          : in Integer := 0;
      Predictable_Global_Seed    : in Boolean := True;
      With_CRPD                  : in Boolean := False);

   procedure initialize_Multiprocessor_Scheduling
     (Sys : in system; 
      Result                  : in out Scheduling_Table_Ptr;
      Event_To_Generate       : in Time_Unit_Event_Type_Boolean_Table;
      Last_Time               : in Natural;
      With_Offsets            : in Boolean := True;
      With_Precedencies       : in Boolean := True;
      With_Resources          : in Boolean := True;
      With_jitters          : in Boolean := True;
      With_Task_Specific_Seed : in Boolean := True;
      Global_Seed_Value       : in Integer := 0;
      Predictable_Global_Seed : in Boolean := True;
      With_CRPD		      : in Boolean := False);

   --------------------------------------------------------------
   -- User's defined event anayser
   -- From a scheduling table, this function do an
   -- user's defined analysis (computed with "event analyzers"
   --------------------------------------------------------------

   procedure Run_An_Event_Analyzer
     (Sys : in system; 
      Sched             : in Scheduling_Table_Ptr;
      An_Event_Analyzer : in Event_Analyzer_Ptr;
      Result            : in out Unbounded_String);


end Multiprocessor_Services;
