------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Xml_Tag;           use Xml_Tag;
with double_util;       use double_util;
with Translate;         use Translate;
with Ada.Numerics.Aux;  use Ada.Numerics.Aux;
with unbounded_strings; use unbounded_strings;



package body Scheduler.Dynamic_Priority.Llf is

   procedure Initialize (A_Scheduler : in out Llf_Scheduler) is
   begin
      Reset (A_Scheduler);
      A_Scheduler.parameters.scheduler_type := Least_Laxity_First_Protocol;
   end Initialize;

   function Copy
     (A_Scheduler : in Llf_Scheduler)
      return        Generic_Scheduler_Ptr
   is
      Ptr : Llf_Scheduler_Ptr;

   begin

      Ptr := new Llf_Scheduler;

      Ptr.parameters         := A_Scheduler.parameters;
      Ptr.Previously_Elected := A_Scheduler.Previously_Elected;

      return Generic_Scheduler_Ptr (Ptr);

   end Copy;

   procedure Do_Election
     (My_Scheduler       : in out Llf_Scheduler;
      Si                 : in out Scheduling_Information;
      Result             : in out Scheduling_Sequence_Ptr;
      Msg                : in out Unbounded_String;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      Address_Space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean;
      With_jitters     : in Boolean;
      With_minimize_preemption     : in Boolean;
      Event_To_Generate  : in Time_Unit_Event_Type_Boolean_Table;
      Elected            : in out Tasks_Range;
      No_Task            : in out Boolean)
   is

      Smallest_Laxity : Natural     := Natural'Last;
      Laxity          : Natural;
      I               : Tasks_Range := 0;
	is_ready : boolean := false;
   begin


  loop


  if not Si.Tcbs (i).already_run_at_current_time then
  if (Si.Tcbs (I).Tsk.cpu_name = Processor_Name) then
      if Check_Core_Assignment(my_scheduler, Si.Tcbs (I))
            then

            Laxity :=
              Dynamic_Priority_Tcb_Ptr (Si.Tcbs (I)).tsk.Deadline -
              Si.Tcbs (I).Rest_Of_Capacity;

            if (Si.Tcbs (I).Wake_Up_Time <= Current_Time) and
               (Laxity < Smallest_Laxity) and
               (Si.Tcbs (I).Rest_Of_Capacity /= 0)
            then
                    if With_Resources then
                        Check_Resource
                          (My_Scheduler,
                           Si,
                           Result,
                           Current_Time,
                           Si.Tcbs (I),
                           is_ready,
                           Event_To_Generate);
                if is_ready then

               Check_jitter(Si.Tcbs (I), Current_Time, Si.Tcbs(I).is_jitter_ready );
               if (With_jitters  = False) or (Si.Tcbs (I).is_jitter_ready)
               then

               if (With_Offsets = False) or
                  Check_Offset (Si.Tcbs (I), Current_Time)
               then
                  if (With_Precedencies = False) or
                     Check_Precedencies
                        (Si,
                         My_Dependencies,
                         Current_Time,
                         Si.Tcbs (I))
                  then

                     Smallest_Laxity := Laxity;
                     Elected         := I;
                  end if;
               end if;
            end if;
         end if;
         end if;
         end if;
         end if;
         end if;
         end if;

         I := I + 1;
         exit when Si.Tcbs (I) = null;
      end loop;

      if Smallest_Laxity = Natural'Last then
         No_Task := True;
      else
         No_Task := False;
      end if;

   end Do_Election;


end Scheduler.Dynamic_Priority.Llf;
