------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 395 $
--    $Date: 2011-04-11 20:43:38 +0200 (lun., 11 avr. 2011) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

package Scheduler.Fixed_Priority.aperiodic_server is

   type aperiodic_server_Scheduler is abstract new Fixed_Priority_Scheduler
   with private;
   type aperiodic_server_Scheduler_Ptr is access all
     aperiodic_server_Scheduler'Class;

   procedure Specific_Scheduler_Initialization
     (My_Scheduler       : in out aperiodic_server_Scheduler;
      Si                 : in out Scheduling_Information;
      Processor_Name     : in Unbounded_String;
      address_space_name : in Unbounded_String;
      My_Tasks           : in out Tasks_Set;
      my_schedulers      : in Scheduler_table;
      My_Resources       : in out Resources_Set;
      My_Buffers         : in out Buffers_Set;
      My_Messages        : in Messages_Set;
      Msg                : in out Unbounded_String);

   procedure Do_Election
     (My_Scheduler       : in out aperiodic_server_Scheduler;
      Si                 : in out Scheduling_Information;
      Result             : in out Scheduling_Sequence_Ptr;
      Msg                : in out Unbounded_String;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      Address_Space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean;
      With_jitters     : in Boolean;
      With_minimize_preemption     : in Boolean;
      Event_To_Generate  : in Time_Unit_Event_Type_Boolean_Table;
      Elected            : in out Tasks_Range;
      No_Task            : in out Boolean);

   procedure Check_Before_Scheduling
     (My_Scheduler   : in aperiodic_server_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String);



private

   type aperiodic_server_Scheduler is abstract new Fixed_Priority_Scheduler
   with
      record
         periodic_Highest_Priority  : Natural     := Natural'First;
         aperiodic_Highest_Priority : Natural     := Natural'First;
         periodic_Elected           : Tasks_Range := 0;
         aperiodic_Elected          : Tasks_Range := 0;
		 
		 current_server_wake_time : Natural;
		 current_server_capacity  : Natural;
      end record;

end Scheduler.Fixed_Priority.aperiodic_server;
