------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim., 29 juin 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

package Scheduler.Hierarchical.cyclic is

   type Hierarchical_cyclic_Scheduler is new Hierarchical_Scheduler with
      private;
   type Hierarchical_cyclic_Scheduler_Ptr is access all Hierarchical_Scheduler'
     Class;

   procedure Initialize
     (A_Scheduler : in out Hierarchical_cyclic_Scheduler);

   function Copy
     (A_Scheduler : in Hierarchical_cyclic_Scheduler)
      return        Generic_Scheduler_Ptr;

   procedure Check_Before_Scheduling
     (My_Scheduler   : in Hierarchical_cyclic_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String);

   function Build_Resource
     (My_Scheduler : in Hierarchical_cyclic_Scheduler;
      A_Resource   : Generic_Resource_Ptr)
      return         Shared_Resource_Ptr;

   function Build_Tcb
     (My_Scheduler : in Hierarchical_cyclic_Scheduler;
      A_Task       : Generic_Task_Ptr)
      return         Tcb_Ptr;

   procedure Specific_Scheduler_Initialization
     (My_Scheduler       : in out Hierarchical_cyclic_Scheduler;
      Si                 : in out Scheduling_Information;
      Processor_Name     : in Unbounded_String;
      address_space_name : in Unbounded_String;
      My_Tasks           : in out Tasks_Set;
      my_schedulers      : in Scheduler_table;
      My_Resources       : in out Resources_Set;
      My_Buffers         : in out Buffers_Set;
      My_Messages        : in Messages_Set;
      Msg                : in out Unbounded_String);

   procedure Do_Election
     (My_Scheduler       : in out Hierarchical_cyclic_Scheduler;
      Si                 : in out Scheduling_Information;
      Result             : in out Scheduling_Sequence_Ptr;
      Msg                : in out Unbounded_String;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      Address_Space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean;
      With_jitters     : in Boolean;
      With_minimize_preemption     : in Boolean;
      Event_To_Generate  : in Time_Unit_Event_Type_Boolean_Table;
      Elected            : in out Tasks_Range;
      No_Task            : in out Boolean);

private

   type Hierarchical_cyclic_Scheduler is new Hierarchical_Scheduler with
      record

        -- Current activated address space
   	--
        address_space_index : scheduler_table_range := 0;
      end record;

end Scheduler.Hierarchical.cyclic;
