------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 422 $
--    $Date: 2011-04-20 20:59:09 +0200 (mer., 20 avr. 2011) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Xml_Tag;           use Xml_Tag;
with double_util;       use double_util;
with Translate;         use Translate;
with unbounded_strings; use unbounded_strings;
with systems; use systems;
with ada.tags; use ada.tags;
with text_io; use text_io;
with Scheduling_Analysis; use Scheduling_Analysis;
use Scheduling_Analysis.Double_Tasks_Parameters_Package;
with systems;
use systems;



package body Scheduler.multiprocessor_specific.edzl is




   function Build_Tcb
     (My_Scheduler : in multiprocessor_edzl_Scheduler;
      A_Task       : Generic_Task_Ptr)
      return         Tcb_Ptr
   is
      A_Tcb : edzl_Tcb_Ptr;
   begin
      A_Tcb := new edzl_Tcb;
      Initialize (Tcb (A_Tcb.all), A_Task);
      Initialize (A_Tcb.all);
      return Tcb_Ptr (A_Tcb);
   end Build_Tcb;

   procedure Initialize (A_Tcb : in out edzl_Tcb) is 
   begin
      A_Tcb.Dynamic_Deadline := A_Tcb.Tsk.deadline + A_Tcb.Wake_Up_Time;
   end Initialize;


   procedure Check_Before_Scheduling
     (My_Scheduler   : in multiprocessor_edzl_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String)
   is
   begin
      null;
   end Check_Before_Scheduling;




   procedure Specific_Scheduler_Initialization
     (My_Scheduler       : in out multiprocessor_edzl_Scheduler;
      Si                 : in out Scheduling_Information;
      Processor_Name     : in Unbounded_String;
      address_space_name : in Unbounded_String;
      My_Tasks           : in out Tasks_Set;
      my_schedulers      : in Scheduler_table;
      My_Resources       : in out Resources_Set;
      My_Buffers         : in out Buffers_Set;
      My_Messages        : in Messages_Set;
      Msg                : in out Unbounded_String)
   is
   begin
      null;
   end Specific_Scheduler_Initialization;

      

   procedure Initialize(A_Scheduler : in out multiprocessor_edzl_Scheduler) is 
   begin
      Reset (A_Scheduler);
      A_Scheduler.parameters.scheduler_type :=
        Earliest_Deadline_First_Protocol;
   end Initialize;

   function Copy
     (A_Scheduler : in multiprocessor_edzl_Scheduler)
      return        Generic_Scheduler_Ptr
   is
      Ptr : multiprocessor_edzl_Scheduler_ptr;

   begin

      Ptr := new multiprocessor_edzl_Scheduler;

      Ptr.parameters         := A_Scheduler.parameters;
      Ptr.Previously_Elected := A_Scheduler.Previously_Elected;

      return Generic_Scheduler_Ptr (Ptr);

   end Copy;

   procedure Do_Election
     (My_Scheduler       : in out multiprocessor_edzl_Scheduler;
      Si                 : in out Scheduling_Information;
      Result             : in out Scheduling_Sequence_Ptr;
      Msg                : in out Unbounded_String;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      Address_Space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean;
      With_jitters     : in Boolean;
      With_minimize_preemption     : in Boolean;
      Event_To_Generate  : in Time_Unit_Event_Type_Boolean_Table;
      Elected            : in out Tasks_Range;
      No_Task            : in out Boolean)
   is

      Smallest_Deadline : Natural     := Natural'Last;
      I                 : Tasks_Range := 0;
	is_ready : boolean := false;
	previous_task_can_be_run : boolean :=false;
      Laxity          : double;
	has_zero_laxity : boolean  :=false;

   begin

         No_Task := True;



      loop


  if (Si.Tcbs (I).Tsk.cpu_name = Processor_Name) then

       Laxity :=
              double(edzl_Tcb_Ptr (Si.Tcbs (I)).tsk.Deadline - current_time - Si.Tcbs (I).Rest_Of_Capacity);

        edzl_Tcb_ptr (Si.Tcbs (i)).Dynamic_Deadline  :=
              Si.Tcbs (i).wake_up_time +
              Si.Tcbs (i).Tsk.deadline;
end if;

  if not Si.Tcbs (i).already_run_at_current_time then
  if (Si.Tcbs (I).Tsk.cpu_name = Processor_Name) then
      if Check_Core_Assignment(my_scheduler, Si.Tcbs (I))
            then
            if (Si.Tcbs (I).Wake_Up_Time <= Current_Time) and
               (Si.Tcbs (I).Rest_Of_Capacity /= 0)
            then
                     if With_Resources then
                        Check_Resource
                          (My_Scheduler,
                           Si,
                           Result,
                           Current_Time,
                           Si.Tcbs (I),
			   is_ready,
                           Event_To_Generate);
		if is_ready then
               Check_jitter(Si.Tcbs (I), Current_Time, Si.Tcbs(I).is_jitter_ready );
               if (With_jitters  = False) or (Si.Tcbs (I).is_jitter_ready)
               then
               if (With_Offsets = False) or
                  Check_Offset (Si.Tcbs (I), Current_Time)
               then

                  if (With_Precedencies = False) or
                     Check_Precedencies
                        (Si,
                         My_Dependencies,
                         Current_Time,
                         Si.Tcbs (I))
                  then

			if i = my_scheduler.previously_elected then
				previous_task_can_be_run:=true;
			end if;
			if laxity <= 0.0
			then
				has_zero_laxity:=true;
				elected:=i;
         No_Task := False;
			end if;

                  if (not has_zero_laxity) and 
			(edzl_Tcb_Ptr (Si.Tcbs (I)).Dynamic_Deadline < Smallest_Deadline) then
                     Smallest_Deadline :=
                       edzl_Tcb_Ptr (Si.Tcbs (I)).Dynamic_Deadline;
                     Elected           := I;
         No_Task := False;
                  end if;
                  end if;
               end if;
            end if;
            end if;
            end if;
            end if;
            end if;
            end if;
         end if;

         I := I + 1;
         exit when Si.Tcbs (I) = null;
      end loop;




-- By default, as task are sorted in the set according to their name
-- when we have two tasks with the same absolute deadline, we choose the first one
-- in the task set, i.e. the task with the smallest name.
-- This strategy can be useful has it provides a simple mean to introduce a
-- tie break as a kind of fixed priority.
-- However, it may introduce an extra preemption.
-- If we want to reduce preemption number as much as possible, in this case
-- we select the previous task ... in this task can be run again !
--
if with_minimize_preemption and previous_task_can_be_run then
       if edzl_Tcb_ptr (Si.Tcbs (my_scheduler.previously_elected)).Dynamic_Deadline = Smallest_Deadline
		then elected:=my_scheduler.previously_elected;
	end if;
end if;


   end Do_Election;




end Scheduler.multiprocessor_specific.edzl;
