------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 422 $
--    $Date: 2011-04-20 20:59:09 +0200 (mer., 20 avr. 2011) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with unbounded_strings; use unbounded_strings;
with Text_IO;           use Text_IO;

package body Scheduler.multiprocessor_specific.pfair.PF is

   procedure Initialize
     (A_Scheduler : in out multiprocessor_pfair_pf_Scheduler)
   is
   begin
      Reset (A_Scheduler);
      A_Scheduler.parameters.scheduler_type := Proportionate_Fair_PF_Protocol;
   end Initialize;

   function Copy
     (A_Scheduler : in multiprocessor_pfair_pf_Scheduler)
      return        Generic_Scheduler_Ptr
   is
      Ptr : multiprocessor_pfair_pf_Scheduler_Ptr;

   begin

      Ptr := new multiprocessor_pfair_pf_Scheduler;

      Ptr.parameters         := A_Scheduler.parameters;
      Ptr.Previously_Elected := A_Scheduler.Previously_Elected;

      return Generic_Scheduler_Ptr (Ptr);

   end Copy;

   procedure Do_Election
     (My_Scheduler       : in out multiprocessor_pfair_pf_Scheduler;
      Si                 : in out Scheduling_Information;
      Result             : in out Scheduling_Sequence_Ptr;
      Msg                : in out Unbounded_String;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      Address_Space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean;
      With_jitters     : in Boolean;
      With_minimize_preemption     : in Boolean;
      Event_To_Generate  : in Time_Unit_Event_Type_Boolean_Table;
      Elected            : in out Tasks_Range;
      No_Task            : in out Boolean)
   is

      i                 : Tasks_Range := 0;
      Smallest_Deadline : Natural     := Natural'Last;
      is_ready : boolean := false;

   begin

      -- Compute scheduling parameters of the current quantum for each task
      --
      Compute_Quantum_Scheduling_parameters
        (My_Scheduler,
         Processor_Name,
         Si);

      i := 0;
      loop
         if not Si.Tcbs (i).already_run_at_current_time then
            if (Si.Tcbs (i).Tsk.cpu_name = Processor_Name) then
               if Check_Core_Assignment(My_Scheduler, Si.Tcbs (i))
               then
                  if (Si.Tcbs (i).Wake_Up_Time <= Current_Time) and
                     (Si.Tcbs (i).Rest_Of_Capacity /= 0)
                  then
               Check_jitter(Si.Tcbs (I), Current_Time, Si.Tcbs(I).is_jitter_ready );
               if (With_jitters  = False) or (Si.Tcbs (I).is_jitter_ready)
               then
                     if (With_Offsets = False) or
                        Check_Offset (Si.Tcbs (i), Current_Time)
                     then
                        if (With_Precedencies = False) or
                           Check_Precedencies
                              (Si,
                               My_Dependencies,
                               Current_Time,
                               Si.Tcbs (i))
                        then
                     if With_Resources then
                        Check_Resource
                          (My_Scheduler,
                           Si,
                           Result,
                           Current_Time,
                           Si.Tcbs (I),
                           is_ready,
                           Event_To_Generate);
                if is_ready then


                           if (Natural (Pfair_Tcb_Ptr (Si.Tcbs (i)).quantum_r)
                               <=
                               Current_Time) and
                              (Natural (Pfair_Tcb_Ptr (Si.Tcbs (i)).quantum_d)
                               <=
                               Smallest_Deadline)
                           then
                              Smallest_Deadline :=
                                Natural (
                                Pfair_Tcb_Ptr (Si.Tcbs (i)).quantum_d);
                              Elected           := i;
                           end if;

                        end if;
                     end if;
                     end if;
                     end if;
                     end if;
                  end if;
               end if;
               end if;
            end if;

         i := i + 1;
         exit when Si.Tcbs (i) = null;
      end loop;

      if Smallest_Deadline = Natural'Last then
         No_Task := True;
      else
         No_Task := False;
      end if;

   end Do_Election;

end Scheduler.multiprocessor_specific.pfair.PF;
