------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 422 $
--    $Date: 2011-04-20 20:59:09 +0200 (mer., 20 avr. 2011) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Translate; use Translate;

package body Scheduler.multiprocessor_specific.pfair is

   function Build_Tcb
     (My_Scheduler : in multiprocessor_pfair_Scheduler;
      A_Task       : Generic_Task_Ptr)
      return         Tcb_Ptr
   is
      A_Tcb : Pfair_Tcb_Ptr;
   begin
      A_Tcb := new Pfair_Tcb;
      Initialize (Tcb (A_Tcb.all), A_Task);
      Initialize (A_Tcb.all);
      return Tcb_Ptr (A_Tcb);
   end Build_Tcb;

   procedure Initialize (A_Tcb : in out Pfair_Tcb) is
   begin
      A_Tcb.quantum_d := 0.0;
      A_Tcb.quantum_r := 0.0;
      A_Tcb.quantum_b := 0.0;
   end Initialize;

   procedure Specific_Scheduler_Initialization
     (My_Scheduler       : in out multiprocessor_pfair_Scheduler;
      Si                 : in out Scheduling_Information;
      Processor_Name     : in Unbounded_String;
      address_space_name : in Unbounded_String;
      My_Tasks           : in out Tasks_Set;
      my_schedulers      : in Scheduler_table;
      My_Resources       : in out Resources_Set;
      My_Buffers         : in out Buffers_Set;
      My_Messages        : in Messages_Set;
      Msg                : in out Unbounded_String)
   is
   begin
      null;
   end Specific_Scheduler_Initialization;



   procedure Check_Before_Scheduling
     (My_Scheduler   : in multiprocessor_pfair_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String)
   is

   begin
      Periodic_Control (My_Tasks, Processor_Name);
      Have_Deadlines_Equal_Than_Periods_Control (My_Tasks, Processor_Name);
   end Check_Before_Scheduling;


   procedure Compute_Quantum_Scheduling_parameters
     (My_Scheduler   : in multiprocessor_pfair_Scheduler;
      Processor_Name : in Unbounded_String;
      Si             : in out Scheduling_Information)
   is

      i : Tasks_Range;

   begin

      -- Compute PFair dynamic priorities
      --
      i := 0;
      loop

         if (Si.Tcbs (i).Tsk.cpu_name = Processor_Name) then

            Pfair_Tcb_Ptr (Si.Tcbs (i)).quantum_d  :=
               Double'Ceiling
                 (Double (Si.Tcbs (i).Used_Capacity + 1) /
                  (Double (Si.Tcbs (i).Tsk.capacity) /
                   Double (Periodic_Task_Ptr (Si.Tcbs (i).Tsk).period)));
            Pfair_Tcb_Ptr (Si.Tcbs (i)).quantum_r  :=
               Double'Ceiling
                 (Double (Si.Tcbs (i).Used_Capacity) /
                  (Double (Si.Tcbs (i).Tsk.capacity) /
                   Double (Periodic_Task_Ptr (Si.Tcbs (i).Tsk).period)));
            Pfair_Tcb_Ptr (Si.Tcbs (i)).quantum_b  :=
              Pfair_Tcb_Ptr (Si.Tcbs (i)).quantum_d -
              Pfair_Tcb_Ptr (Si.Tcbs (i)).quantum_r;

         end if;

         i := i + 1;
         exit when Si.Tcbs (i) = null;
      end loop;

   end Compute_Quantum_Scheduling_parameters;

end Scheduler.multiprocessor_specific.pfair;
