------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 422 $
--    $Date: 2011-04-20 20:59:09 +0200 (mer., 20 avr. 2011) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with unbounded_strings; use unbounded_strings;

package Scheduler.multiprocessor_specific.pfair is

   type Pfair_Tcb is new Tcb with record
      quantum_d : Double;
      quantum_r : Double;
      quantum_b : Double;
   end record;

   type Pfair_Tcb_Ptr is access all Pfair_Tcb'Class;

   procedure Initialize (A_Tcb : in out Pfair_Tcb);

   -------------------------------------------------

   type multiprocessor_pfair_Scheduler is abstract new
     multiprocessor_Scheduler with private;
   type multiprocessor_pfair_Scheduler_Ptr is access all
     multiprocessor_pfair_Scheduler'Class;

   function Build_Tcb
     (My_Scheduler : in multiprocessor_pfair_Scheduler;
      A_Task       : Generic_Task_Ptr)
      return         Tcb_Ptr;

   procedure Specific_Scheduler_Initialization
     (My_Scheduler       : in out multiprocessor_pfair_Scheduler;
      Si                 : in out Scheduling_Information;
      Processor_Name     : in Unbounded_String;
      address_space_name : in Unbounded_String;
      My_Tasks           : in out Tasks_Set;
      my_schedulers      : in Scheduler_table;
      My_Resources       : in out Resources_Set;
      My_Buffers         : in out Buffers_Set;
      My_Messages        : in Messages_Set;
      Msg                : in out Unbounded_String);

   procedure Check_Before_Scheduling
     (My_Scheduler   : in multiprocessor_pfair_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String);

   procedure Compute_Quantum_Scheduling_parameters
     (My_Scheduler   : in multiprocessor_pfair_Scheduler;
      Processor_Name : in Unbounded_String;
      Si             : in out Scheduling_Information);

private

   type multiprocessor_pfair_Scheduler is abstract new
     multiprocessor_Scheduler with null record;

end Scheduler.multiprocessor_specific.pfair;
