------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with unbounded_strings; use unbounded_strings;
with Translate;         use Translate;

package body Scheduler.Time_Sharing_Based_On_Cpu_Usage is

   procedure Initialize
     (A_Scheduler : in out Time_Sharing_Based_On_Cpu_Usage_Scheduler)
   is
   begin
      Reset (A_Scheduler);
      A_Scheduler.parameters.scheduler_type :=
        Time_Sharing_Based_On_Cpu_Usage_Protocol;
   end Initialize;

   function Copy
     (A_Scheduler : in Time_Sharing_Based_On_Cpu_Usage_Scheduler)
      return        Generic_Scheduler_Ptr
   is
      Ptr : Time_Sharing_Based_On_Cpu_Usage_Scheduler_Ptr;

   begin

      Ptr := new Time_Sharing_Based_On_Cpu_Usage_Scheduler;

      Ptr.parameters         := A_Scheduler.parameters;
      Ptr.Previously_Elected := A_Scheduler.Previously_Elected;

      return Generic_Scheduler_Ptr (Ptr);

   end Copy;



   procedure Put
     (My_Scheduler : in Time_Sharing_Based_On_Cpu_Usage_Scheduler)
   is
   begin
      Put (Generic_Scheduler (My_Scheduler));
   end Put;

   procedure Check_Before_Scheduling
     (My_Scheduler   : in Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String)
   is

   begin
      null;
   end Check_Before_Scheduling;

   procedure Do_Election
     (My_Scheduler       : in out Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      Si                 : in out Scheduling_Information;
      Result             : in out Scheduling_Sequence_Ptr;
      Msg                : in out Unbounded_String;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      Address_Space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean;
           With_jitters     : in Boolean;
      With_minimize_preemption     : in Boolean;
      Event_To_Generate  : in Time_Unit_Event_Type_Boolean_Table;
      Elected            : in out Tasks_Range;
      No_Task            : in out Boolean)
   is

      Less_Executed_Task : Natural     := Natural'Last;
      I                  : Tasks_Range := 0;
      is_ready : boolean := false;

   begin

      loop
  if not Si.Tcbs (i).already_run_at_current_time then
  if (Si.Tcbs (I).Tsk.cpu_name = Processor_Name) then
       if Check_Core_Assignment(My_Scheduler, Si.Tcbs (i))
               then
            if (Si.Tcbs (I).Wake_Up_Time <= Current_Time) and
               (Si.Tcbs (I).Used_Cpu < Less_Executed_Task) and
               (Si.Tcbs (I).Rest_Of_Capacity /= 0)
            then
               Check_jitter(Si.Tcbs (I), Current_Time, Si.Tcbs(I).is_jitter_ready );
               if (With_jitters  = False) or (Si.Tcbs (I).is_jitter_ready)
               then
               if (With_Offsets = False) or
                  Check_Offset (Si.Tcbs (I), Current_Time)
               then
		if With_Resources then
                        Check_Resource
                          (My_Scheduler,
                           Si,
                           Result,
                           Current_Time,
                           Si.Tcbs (I),
                           is_ready,
                           Event_To_Generate);
                if is_ready then
                  if (With_Precedencies = False) or
                     Check_Precedencies
                        (Si,
                         My_Dependencies,
                         Current_Time,
                         Si.Tcbs (I))
                  then

                     Less_Executed_Task := Si.Tcbs (I).Used_Cpu;
                     Elected            := I;
                  end if;
               end if;
            end if;
            end if;
            end if;
            end if;
            end if;
            end if;
         end if;

         I := I + 1;
         exit when Si.Tcbs (I) = null;
      end loop;

      if Less_Executed_Task = Natural'Last then
         No_Task := True;
      else
         No_Task := False;
      end if;

   end Do_Election;

   procedure Specific_Scheduler_Initialization
     (My_Scheduler       : in out Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      Si                 : in out Scheduling_Information;
      Processor_Name     : in Unbounded_String;
      address_space_name : in Unbounded_String;
      My_Tasks           : in out Tasks_Set;
      my_schedulers      : in Scheduler_table;
      My_Resources       : in out Resources_Set;
      My_Buffers         : in out Buffers_Set;
      My_Messages        : in Messages_Set;
      Msg                : in out Unbounded_String)
   is

   begin
      null;
   end Specific_Scheduler_Initialization;

   procedure Compute_Activation_Time
     (My_Scheduler : in Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      Si           : in out Scheduling_Information;
      Elected      : in Tasks_Range;
      Value        : in out Natural)
   is

   begin
      raise Invalid_Scheduler;
   end Compute_Activation_Time;


end Scheduler.Time_Sharing_Based_On_Cpu_Usage;
