------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 97 $
--    $Date: 2007-07-20 15:17:15 +0200 (Fri, 20 Jul 2007) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with section_Set; use section_Set;
use section_Set.package_Generic_section_Set;
with Automaton;   use Automaton;
use Automaton.Package_Automaton_Status;

package Scheduler.user_defined.interpreted.Automata is

   type Automata_User_Defined_Scheduler is new
     interpreted_user_defined_Scheduler with private;
   type Automata_User_Defined_Scheduler_Ptr is access all
     Automata_User_Defined_Scheduler'Class;

   procedure Initialize
     (A_Scheduler : in out Automata_User_Defined_Scheduler);

   function Copy
     (A_Scheduler : in Automata_User_Defined_Scheduler)
      return        Generic_Scheduler_Ptr;

   procedure Set_Automaton_Name
     (My_Scheduler : in out Automata_User_Defined_Scheduler;
      To_Set       : in Unbounded_String);

   function Get_Automaton_Name
     (My_Scheduler : in Automata_User_Defined_Scheduler)
      return         Unbounded_String;

   procedure Put (My_Scheduler : in Automata_User_Defined_Scheduler);

   procedure Do_Election
     (My_Scheduler       : in out Automata_User_Defined_Scheduler;
      Si                 : in out Scheduling_Information;
      Result             : in out Scheduling_Sequence_Ptr;
      Msg                : in out Unbounded_String;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      Address_Space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean;
      With_jitters     : in Boolean;
      With_minimize_preemption     : in Boolean;
      Event_To_Generate  : in Time_Unit_Event_Type_Boolean_Table;
      Elected            : in out Tasks_Range;
      No_Task            : in out Boolean);

   procedure Specific_Scheduler_Initialization
     (My_Scheduler       : in out Automata_User_Defined_Scheduler;
      Si                 : in out Scheduling_Information;
      Processor_Name     : in Unbounded_String;
      address_space_name : in Unbounded_String;
      My_Tasks           : in out Tasks_Set;
      My_Schedulers      : in Scheduler_table;
      My_Resources       : in out Resources_Set;
      My_Buffers         : in out Buffers_Set;
      My_Messages        : in Messages_Set;
      Msg                : in out Unbounded_String);

   -- Check the current state of the automaton : is the transition reachable ?
   --  from the current automaton location
   --
   procedure Check_Reachability
     (My_Scheduler                     : in out
     Automata_User_Defined_Scheduler;
      A_Transition                     : in out Transition_Status;
      Automaton_Name_Of_The_Transition : in Unbounded_String);

   -- Check synchronization : is the transition awaiting another one ?
   --
   procedure Check_Synchronization_Constraints
     (My_Scheduler      : in out Automata_User_Defined_Scheduler;
      A_Transition      : in out Transition_Status;
      Si                : in out Scheduling_Information;
      Result            : in out Scheduling_Sequence_Ptr;
      Msg               : in out Unbounded_String;
      Current_Time      : in Natural;
      Processor_Name    : in Unbounded_String;
      My_Dependencies   : in Tasks_Dependencies_Ptr;
      With_Offsets      : in Boolean;
      With_Precedencies : in Boolean;
      With_Resources    : in Boolean;
      Event_To_Generate : in Time_Unit_Event_Type_Boolean_Table;
      Elected           : in out Tasks_Range;
      No_Task           : in out Boolean);

   -- Check the guard : is the guard true ?
   --
   procedure Check_Guard_Constraints
     (My_Scheduler : in out Automata_User_Defined_Scheduler;
      A_Transition : in out Transition_Status);

   -- Check the delay statements : can we wake up the transition ?
   --
   procedure Check_Delay_Constraints
     (My_Scheduler                     : in out
     Automata_User_Defined_Scheduler;
      A_Transition                     : in out Transition_Status;
      Automaton_Name_Of_The_Transition : in Unbounded_String;
      Current_Time                     : in Natural);

   -- This sub-program fires a transition
   --
   procedure Fire_A_Transition
     (My_Scheduler                     : in out
     Automata_User_Defined_Scheduler;
      A_Transition                     : in out Transition_Status;
      Automaton_Name_Of_The_Transition : in Unbounded_String;
      Si                               : in out Scheduling_Information;
      Result                           : in out Scheduling_Sequence_Ptr;
      Msg                              : in out Unbounded_String;
      Current_Time                     : in Natural;
      Processor_Name                   : in Unbounded_String;
      My_Dependencies                  : in Tasks_Dependencies_Ptr;
      With_Offsets                     : in Boolean;
      With_Precedencies                : in Boolean;
      With_Resources                   : in Boolean;
      Event_To_Generate                : in Time_Unit_Event_Type_Boolean_Table;
      Elected                          : in out Tasks_Range;
      No_Task                          : in out Boolean;
      An_Election_Section_Was_Run      : in out Boolean);

private

   type Automata_User_Defined_Scheduler is new
     interpreted_user_defined_Scheduler with
      record

      -- Name of the automaton which is supposed to model the scheduler of the
      -- processor
      --
         Automaton_Name : Unbounded_String;

         -- Stores the set of sections with their code
         --
         Root_Statement_Pointer : sections_Set;

         -- The shedulers of each address space
         --
         Local_Scheduler : Scheduler_table;

         -- Variable which store simulation data
         -- (status of automata and transitions)
         --
         Transition_Data : Transition_Status_Table;
         Automaton_Data  : Automaton_Status_Table;
      end record;

end Scheduler.user_defined.interpreted.Automata;
