------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 97 $
--    $Date: 2007-07-20 15:17:15 +0200 (Fri, 20 Jul 2007) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Expressions; use Expressions;
use Expressions.Variables_Type_Package;
with Interpreter; use Interpreter;
use Interpreter.Sets_Type_Package;
with Statements;  use Statements;

package Scheduler.user_defined.interpreted is

   type interpreted_user_defined_Scheduler is abstract new
     user_defined_Scheduler with private;
   type interpreted_user_defined_Scheduler_Ptr is access all
     interpreted_user_defined_Scheduler'Class;

   -- Procedure to load/save interpreter variables
   -- that are allowed to be read or written by sc programmers
   --
   procedure Save_Read_Write_Interpreter_Variables
     (My_Scheduler   : in interpreted_user_defined_Scheduler;
      Si             : in out Scheduling_Information;
      Processor_Name : in Unbounded_String);
   procedure Load_Read_Write_Interpreter_Variables
     (My_Scheduler   : in out interpreted_user_defined_Scheduler;
      Si             : in Scheduling_Information;
      Processor_Name : in Unbounded_String);

   -- When a task is run by the scheduling simulator,
   -- its state has changed. These changes must be stored in the interpreter
   -- variables : this procedure update interpreter variables of the
   -- elected task
   --
   procedure update_interpreter_variables_after_task_execution
     (My_Scheduler       : in out interpreted_user_defined_Scheduler;
      Si                 : in out Scheduling_Information;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      address_space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean);

   procedure dispatch_return_statement
     (My_Scheduler       : in out interpreted_user_defined_Scheduler;
      Current            : in Generic_Statement_Ptr;
      Si                 : in out Scheduling_Information;
      Current_Time       : in Natural;
      Processor_Name     : in Unbounded_String;
      address_space_Name : in Unbounded_String;
      My_Dependencies    : in Tasks_Dependencies_Ptr;
      With_Offsets       : in Boolean;
      With_Precedencies  : in Boolean;
      With_Resources     : in Boolean;
      Elected            : in out Tasks_Range;
      No_Task            : in out Boolean);

   procedure Compute_Activation_Time
     (My_Scheduler : in interpreted_user_defined_Scheduler;
      Si           : in out Scheduling_Information;
      Elected      : in Tasks_Range;
      Value        : in out Natural);

private

   type interpreted_user_defined_Scheduler is abstract new
     user_defined_Scheduler with
      record

      -- Variables table for the parsed scheduler
      -- (given by the parser)
      --
         Variables_Table : Variables_Table_Type;

         -- Table of set statements given by the parser
         --
         Sets_Table : Sets_Table_Type;

      end record;

end Scheduler.user_defined.interpreted;
