------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 559 $
--    $Date: 2012-10-13 13:58:21 +0200 (Sat, 13 Oct 2012) $
--    $Author: gaudel $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_IO;           use Text_IO;
with Ada.Exceptions;    use Ada.Exceptions;
with unbounded_strings; use unbounded_strings;


package body Scheduler_interface.extended is


   function Export_Aadl_Properties
     (My_Scheduler : in Scheduling_parameters;
      Number_Of_Ht : in Natural)
      return         Unbounded_String
   is
      Result : Unbounded_String := empty_string;

   begin

      for I in 1 .. Number_Of_Ht loop
         Result := Result & ASCII.HT;
      end loop;

      Result := Result &
                To_Unbounded_String
                   ("Scheduling_Protocol => " &
                    My_Scheduler.scheduler_type'img &
                    ";") &
                unbounded_lf;

      for I in 1 .. Number_Of_Ht loop
         Result := Result & ASCII.HT;
      end loop;

      Result := Result &
                To_Unbounded_String
                   ("Cheddar_Properties::Scheduler_Quantum => " &
                    My_Scheduler.quantum'img &
                    " ms ;") &
                unbounded_lf;

      for I in 1 .. Number_Of_Ht loop
         Result := Result & ASCII.HT;
      end loop;

      if My_Scheduler.preemptive_type = preemptive then
         Result := Result &
                   To_Unbounded_String
                      ("Cheddar_Properties::Preemptive_Scheduler => True;") &
                   unbounded_lf;
      else
         Result := Result &
                   To_Unbounded_String
                      ("Cheddar_Properties::Preemptive_Scheduler => False;") &
                   unbounded_lf;
      end if;

      for I in 1 .. Number_Of_Ht loop
         Result := Result & ASCII.HT;
      end loop;

      Result := Result &
                To_Unbounded_String
                   ("Cheddar_Properties::Automaton_Name => """ &
                    To_String (My_Scheduler.automaton_name) &
                    """;") &
                unbounded_lf;

      for I in 1 .. Number_Of_Ht loop
         Result := Result & ASCII.HT;
      end loop;

   Result := Result &
                To_Unbounded_String
                   ("Cheddar_Properties::Source_Text => """ &
                    To_String (My_Scheduler.user_defined_scheduler_source_file_name) &
                    """;") &
                unbounded_lf;


      return Result;

   end Export_Aadl_Properties;



end Scheduler_interface.extended;


