------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 297 $
--    $Date: 2010-11-23 13:39:03 +0100 (mar., 23 nov. 2010) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Translate;    use Translate;
with Buffer_Set;   use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resources;    use Resources;


package body Scheduling_Analysis.extended.buffer_analysis is




   function Compute_Average_Buffer_Size_From_Simulation
     (Buff : in Buffer_Size_Table_Ptr)
      return Double
   is
      Sum : Double := 0.0;

   begin

      for I in 0 .. Buff.nb_entries - 1 loop
         Sum := Sum + Double (Buff.entries (I).size);
      end loop;

      return Sum / Double (Buff.nb_entries);

   end Compute_Average_Buffer_Size_From_Simulation;

   function Compute_Maximum_Buffer_Size_From_Simulation
     (Buff : in Buffer_Size_Table_Ptr)
      return Natural
   is
      Max : Natural := 0;
   begin

      for I in 0 .. Buff.nb_entries - 1 loop
         if Buff.entries (I).size > Max then
            Max := Buff.entries (I).size;
         end if;
      end loop;

      return Max;

   end Compute_Maximum_Buffer_Size_From_Simulation;

   procedure Compute_Buffer_Size_From_Simulation
     (Sched   : in Scheduling_Sequence_Ptr;
      My_Buff : in Buffer_Ptr;
      Buff    : in out Buffer_Size_Table_Ptr)
   is

      Current_Size : Natural := 0;
      A_Buff_Size  : Buffer_Size_Item;

   begin

      for I in 0 .. Sched.nb_entries - 1 loop

         if (Sched.entries (I).data.type_of_event = Read_From_Buffer) then
            if (Sched.entries (I).data.read_buffer.name = My_Buff.name) then
               A_Buff_Size.time := Sched.entries (I).item;
               Current_Size     := Current_Size -
                                   Sched.entries (I).data.read_size;
               A_Buff_Size.size := Current_Size;
               add (Buff.all, A_Buff_Size);
            end if;
         end if;

         if (Sched.entries (I).data.type_of_event = Write_To_Buffer) then
            if (Sched.entries (I).data.write_buffer.name = My_Buff.name) then
               A_Buff_Size.time := Sched.entries (I).item;
               Current_Size     := Current_Size +
                                   Sched.entries (I).data.write_size;
               A_Buff_Size.size := Current_Size;
               add (Buff.all, A_Buff_Size);
            end if;
         end if;

      end loop;
   end Compute_Buffer_Size_From_Simulation;

   function Compute_Maximum_Waiting_Time_From_Simulation
     (Buff                    : in Buffer_Size_Table_Ptr;
      Average_Consumer_Period : in Double)
      return                    Natural
   is
      Max : Natural := 0;
   begin

      for I in 0 .. Buff.nb_entries - 1 loop
         if Buff.entries (I).size > Max then
            Max := Buff.entries (I).size;
         end if;
      end loop;

      return (Max * Natural (Average_Consumer_Period));

   end Compute_Maximum_Waiting_Time_From_Simulation;

   function Compute_Average_Waiting_Time_From_Simulation
     (Buff                    : in Buffer_Size_Table_Ptr;
      Average_Consumer_Period : in Double)
      return                    Double
   is
      Sum : Double := 0.0;

   begin

      for I in 0 .. Buff.nb_entries - 1 loop
         Sum := Sum + Double (Buff.entries (I).size);
      end loop;

      return ((Sum / Double (Buff.nb_entries)) * Average_Consumer_Period);

   end Compute_Average_Waiting_Time_From_Simulation;

                          
                     





end Scheduling_Analysis.extended.buffer_analysis;



