------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 317 $
--    $Date: 2010-12-06 13:14:59 +0100 (lun., 06 déc. 2010) $
--    $Author: gaudel $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Buffers;                use Buffers;
use Buffers.Buffer_Roles_package;
with Queueing_System;        use Queueing_System;
use Queueing_System.A_Resp_Time_Consumer;


package Scheduling_Analysis.extended.buffer_analysis is


   No_Performance_Criteria : exception;


   -- Compute from a scheduling sequence buffer size for each
   -- time unit
   --
   procedure Compute_Buffer_Size_From_Simulation
     (Sched   : in Scheduling_Sequence_Ptr;
      My_Buff : in Buffer_Ptr;
      Buff    : in out Buffer_Size_Table_Ptr);

   -- Compute, from a buffer_Size_Table, maximum and average
   -- performance criterion
   --
   function Compute_Average_Buffer_Size_From_Simulation
     (Buff : in Buffer_Size_Table_Ptr)
      return Double;

   function Compute_Maximum_Buffer_Size_From_Simulation
     (Buff : in Buffer_Size_Table_Ptr)
      return Natural;

   function Compute_Maximum_Waiting_Time_From_Simulation
     (Buff                    : in Buffer_Size_Table_Ptr;
      Average_Consumer_Period : in Double)
      return                    Natural;

   function Compute_Average_Waiting_Time_From_Simulation
     (Buff                    : in Buffer_Size_Table_Ptr;
      Average_Consumer_Period : in Double)
      return                    Double;


end Scheduling_Analysis.extended.buffer_analysis;
