------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 317 $
--    $Date: 2010-12-06 13:14:59 +0100 (lun., 06 déc. 2010) $
--    $Author: gaudel $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Task_Set;               use Task_Set;
use Task_Set.Generic_Task_Set;
with Tasks.Extended;         use Tasks.Extended;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;

package Scheduling_Analysis.extended.resource_analysis is

   

   -- Compute maximum/minimum/average shared resource blocking time from scheduling simulaion
   --
   procedure Blocking_Time_From_Simulation
     (My_Task      : in Generic_Task_Ptr;
      My_Resources : in Resources_Set;
      Sched        : in Scheduling_Sequence_Ptr;
      Max          : in out Natural;
      Min          : in out Natural;
      Average      : in out Double);

   -- Check if there are deadlocks on shared resources in the scheduling simulation
   --
   function Deadlock_From_Simulation
     (My_Tasks       : in Tasks_Set;
      My_Resources   : in Resources_Set;
      Processor_Name : in Unbounded_String;
      Sched          : in Scheduling_Sequence_Ptr)
      return           Deadlock_List;

   -- Check if priority inversions exist in a scheduling simulation
   --
   function Priority_Inversion_From_Simulation
     (My_Tasks       : in Tasks_Set;
      My_Resources   : in Resources_Set;
      Processor_Name : in Unbounded_String;
      Sched          : in Scheduling_Sequence_Ptr)
      return           Priority_Inversion_List;

end Scheduling_Analysis.extended.resource_analysis;
