------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 317 $
--    $Date: 2010-12-06 13:14:59 +0100 (lun., 06 déc. 2010) $
--    $Author: gaudel $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Task_Set;               use Task_Set;
use Task_Set.Generic_Task_Set;
with Time_Unit_Events;       use Time_Unit_Events;
use Time_Unit_Events.Time_Unit_Lists_Package;
use Time_Unit_Events.Time_Unit_Package;
with Tasks.Extended;         use Tasks.Extended;
with Buffers;                use Buffers;
use Buffers.Buffer_Roles_package;
with Event_Analyzer_Set;     use Event_Analyzer_Set;
use Event_Analyzer_Set.Generic_Event_Analyzer_Set;
with Queueing_System;        use Queueing_System;
use Queueing_System.A_Resp_Time_Consumer;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with tables;
with indexed_tables;
with access_lists;
with Unchecked_Deallocation;
with Queueing_System;        use Queueing_System;
use Queueing_System.A_Resp_Time_Consumer;



package Scheduling_Analysis.extended.task_analysis  is


   -- Compute number of task preemption from simulation
   --
   function Number_Of_Preemption_From_Simulation
     (Sched    : in Scheduling_Sequence_Ptr;
      My_Tasks : in Tasks_Set)
      return     Natural;

   -- Compute number of switch context from simulation
   --
   function Number_Of_Switch_Context_From_Simulation
     (Sched : in Scheduling_Sequence_Ptr)
      return  Natural;


   -- Compute the worst/best/average response time of a task from simulation
   --
   procedure Response_Time_By_Simulation
     (My_Task : Generic_Task_Ptr;
      Sched   : in Scheduling_Sequence_Ptr;
      Msg     : in out Unbounded_String;
      Max     : in out Natural;
      Min     : in out Natural;
      Average : in out Double);


-- Compute ALL response times of a task from simulation
--
   procedure All_Response_Times_By_Simulation
     (My_Task : in Generic_Task_Ptr;
      Sched   : in Scheduling_Sequence_Ptr;
      Result  : in out Task_Occurence_Table_Ptr);


   -- Compute probability distribution of response time from simulation
   --
   procedure Compute_Response_Time_Distribution
     (My_Tasks       : in Tasks_Set;
      Sched          : in Scheduling_Sequence_Ptr;
      Processor_Name : Unbounded_String;
      Result         : in out Densities_Table);


   -- Consumer of a buffer response from simulation 
   -- (required for MP1 criteria computation)
   --
   procedure Consumer_Response_Time
     (Sched              : in Scheduling_Sequence_Ptr;
      Consumer_Resp_Time : in out Resp_Time_Consumer_Table_Ptr;
      My_Consumer_Task   : in Generic_Task_Ptr);




end Scheduling_Analysis.extended.task_analysis;
