------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 317 $
--    $Date: 2010-12-06 13:14:59 +0100 (lun., 06 déc. 2010) $
--    $Author: gaudel $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Task_Set;               use Task_Set;
use Task_Set.Generic_Task_Set;
with Time_Unit_Events;       use Time_Unit_Events;
use Time_Unit_Events.Time_Unit_Lists_Package;
use Time_Unit_Events.Time_Unit_Package;
with Tasks.Extended;         use Tasks.Extended;
with Event_Analyzer_Set;     use Event_Analyzer_Set;
use Event_Analyzer_Set.Generic_Event_Analyzer_Set;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with natural_util; use  natural_util;
with tables;
with indexed_tables;
with Unchecked_Deallocation;


package Scheduling_Analysis.extended is

      
 package natural_list is new access_lists (
      Natural,
      natural_ptr,
      put,
      free,
      copy, xml_string);
   use natural_list;


   package Generic_Index_Lst is new access_lists (
      put         => put,
      element     => Double,
      element_ptr => double_ptr,
      free        => double_util.free,
      copy        => double_util.copy,
      xml_string  => xml_string);

   use Generic_Index_Lst;

   
   package Task_Occurence_Table_Package is new tables (
      Task_Occurence,
      Framework_Config.Max_Scheduling_Period,
      Put,
      XML_String, xml_ref_string);
   use Task_Occurence_Table_Package;

   subtype Task_Occurence_Table is Task_Occurence_Table_Package.table;
   subtype Task_Occurence_Range is Task_Occurence_Table_Package.table_range;
   type Task_Occurence_Table_Ptr is access Task_Occurence_Table;

   procedure Free is new Unchecked_Deallocation (
      Task_Occurence_Table,
      Task_Occurence_Table_Ptr);



end Scheduling_Analysis.extended;
