---------------------------------------------------------------
-----------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 534 $
--    $Date: 2012-10-05 19:46:28 +0200 (Fri, 05 Oct 2012) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Framework_Config;      use Framework_Config;
with Sections;              use Sections;
with sets;

package section_Set is

   ----------------------------------------------------------
   -- Definition of a set
   ----------------------------------------------------------

   package package_Generic_section_Set is new Sets (
      Max_Element => Framework_Config.Max_Sections,
      Element => Generic_Section_Ptr,
      Copy => Copy,
      Free => Free,
      Put => Put,
      xml_string => XML_String,
      xml_ref_string => XML_ref_String);
   use package_Generic_section_Set;

   type sections_Set is new package_Generic_section_Set.set with private;

   subtype sections_Range is package_Generic_section_Set.element_range;
   subtype sections_Iterator is package_Generic_section_Set.iterator;

   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------

   -- Raised when a given task is not in a section set
   --
   section_Not_Found : exception;

   -- Raised when parameters provided to Add_Section are wrong
   --
   Invalid_Parameter : exception;

   ----------------------------------------------------------
   -- I/O operations
   ----------------------------------------------------------

   function Export_Aadl_Implementations
     (My_sections : in sections_Set)
      return        Unbounded_String;

   function Export_Aadl_Declarations
     (My_sections  : in sections_Set;
      Number_Of_Ht : in Natural)
      return         Unbounded_String;

   ----------------------------------------------------------
   -- Procedure to proceed modification on a set
   ----------------------------------------------------------

   procedure Check_section
     (My_sections  : in sections_Set;
      Name         : in Unbounded_String;
      section_Type : in Sections_Type);

   procedure Add_section
     (My_sections  : in out sections_Set;
      Name         : in Unbounded_String;
      section_Type : in Sections_Type);

   procedure Add_section
     (My_sections  : in out sections_Set;
      A_section    : in out Generic_Section_Ptr;
      Name         : in Unbounded_String;
      section_Type : in Sections_Type);

   -- Search a section with its name
   --
   function Search_section
     (My_sections : in sections_Set;
      Name        : in Unbounded_String)
      return        Generic_Section_Ptr;

   -- Search the first section of a given type
   --
   function Search_section
     (My_sections : in sections_Set;
      type_name   : in Sections_Type)
      return        Generic_Section_Ptr;

private

   type sections_Set is new package_Generic_section_Set.set with null record;

end section_Set;
