------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 534 $
--    $Date: 2012-10-05 19:46:28 +0200 (Fri, 05 Oct 2012) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Framework_Config;      use Framework_Config;
with Resource_Set;          use Resource_Set;
with Tasks;                 use Tasks;
with Task_Groups;           use Task_Groups;
with Task_Set;              use Task_Set;
with Parameters;            use Parameters;
with Parameters.extended;   use Parameters.extended;
use Parameters.User_Defined_Parameters_Table_Package;
with Offsets;               use Offsets;
with Offsets.extended;      use Offsets.extended;
use Offsets.Offsets_Table_Package;
with Ada.Numerics.Aux;      use Ada.Numerics.Aux;
with Systems;               use Systems;
with sets;


package Task_Group_Transformation is
   -- Exceptions
   Deadline_Violated : exception;

   ----------------------------------------------------------
   -- Transformation procedures
   ----------------------------------------------------------

   procedure Multiframe_to_Transaction
     (My_Multiframe : in Multiframe_Task_Group_Ptr;
      A_System      : in out System);

   procedure Merge_Transactions
     (A_System : in out System);

   procedure Modify_Offsets
     (A_System : in out System);

   procedure Multiframe_To_Transaction_Sys
     (My_System : in out System;
      A_System  : in out System);

   procedure Multiframe_To_Transaction_Sys_Crossref
     (My_System : in out System;
      A_System  : in out System;
      Merge     : in Boolean := false);

   procedure Add_Delta_Tasks
     (My_System : in out System);

   procedure Delete_Delta_Tasks
     (My_System : in out System);

   procedure Add_Root_Tasks
     (My_System : in out System);

end Task_Group_Transformation;
