------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
------------------------------------------------------------------------------
-- Last update :
--    $Rev: 97 $
--    $Date: 2007-07-20 15:17:15 +0200 (Fri, 20 Jul 2007) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_IO;                  use Text_IO;
with Ada.Strings.Unbounded;    use Ada.Strings.Unbounded;
with Systems;                  use Systems;
with Xml_Tag;                  use Xml_Tag;
with Call_Framework;           use Call_Framework;
with Call_Framework_Interface; use Call_Framework_Interface;
use Call_Framework_Interface.Framework_Response_Package;
use Call_Framework_Interface.Framework_Request_Package;
with Call_Memory_Framework;    use Call_Memory_Framework;
use Call_Memory_Framework.Buffer_Result_Package;
with Parameters;               use Parameters;
with Parameters.extended;
use  Parameters.Framework_Parameters_Table_Package;
with unbounded_strings;        use unbounded_strings;
use unbounded_strings.unbounded_string_list_package;
with Framework_Config;         use Framework_Config;

procedure Xml_basics is

   Sys : System;

   Response_List : Framework_Response_Table;
   Request_List  : Framework_Request_Table;
   A_Request     : Framework_Request;
   dir           : unbounded_string_list;

   result_file : File_Type;

begin

   -- Open file to save analysis result
   --
   Create (result_file, Mode => Out_File, Name => "basics_result.xml");

   Call_Framework.initialize (False);
   Put_Line (result_file, "<results>");

   -- Test on periodic tasks
   --
   Read_From_Xml_File (Sys, dir, To_Unbounded_String ("basics.xml"));

   Initialize (Response_List);
   Initialize (Request_List);
   Initialize (A_Request);
   A_Request.statement := Scheduling_Feasibility_Basics;
   Add (Request_List, A_Request);
   Sequential_Framework_Request
     (Sys,
      Request_List,
      Response_List,
      Total_Order,
      Xml_Output);
   Put (result_file, To_String (XML_root_String (Response_List)));

   Put_Line (result_file, "</results>");
   Close (result_file);

end Xml_basics;
