------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog; use Gtk.About_Dialog;
with Gtk.Label;        use Gtk.Label;
with Glib.Object; use Glib.Object;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Gtk.Combo_Box; use Gtk.Combo_Box;
with Gtk.Text_Iter;    use Gtk.Text_Iter;
with Gtk.List_Store;   use Gtk.List_Store;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Gtk.Tree_View;  use Gtk.Tree_View;
with Gtk.Tree_View_Column;  use Gtk.Tree_View_Column;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with Glib.Values; use Glib.Values;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Exceptions;                                    use Ada.Exceptions;
with unbounded_strings;     use unbounded_strings;
with Processors;                                           use Processors;
with address_spaces; 	use address_spaces;
with unbounded_strings;                                    use unbounded_strings;
use unbounded_strings.strings_table_package;
with Framework_Config;                                     use Framework_Config;
with Buffer_Set;                                           use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;                                         use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with graphical_editor.user_message;    use graphical_editor.user_message;
with GNAT.Current_Exception;                               use GNAT.Current_Exception;
with Translate;                                            use Translate;
with Networks;                                             use Networks;
with Task_Set;                                             use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduler;                                            use Scheduler;
with Scheduler.user_defined.interpreted.pipeline;  use  Scheduler.user_defined.interpreted.pipeline;
with Scheduler.user_defined.interpreted.Automata;  use  Scheduler.user_defined.interpreted.Automata;
with Scheduler_Interface;                                  use Scheduler_Interface;
with Processor_Set; 					use Processor_Set;
with Address_Space_Set; 			use Address_Space_Set;
with graphical_editor.convert_text; use graphical_editor.convert_text;


package body graphical_editor.address_spaces is


Address_Space_table_length : constant natural :=18;


procedure Initialize_Address_Space(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is
	ob1: GObject:= Get_Object(Object, "textbuffer1");
	ob2: GObject:= Get_Object(Object, "combobox1");
	ob3: GObject:= Get_Object(Object, "textbuffer3");
	ob4: GObject:= Get_Object(Object, "textbuffer4");
	ob5: GObject:= Get_Object(Object, "textbuffer5");
	ob6: GObject:= Get_Object(Object, "textbuffer6");
	ob7: GObject:= Get_Object(Object, "combobox2");
	ob8: GObject:= Get_Object(Object, "textbuffer7");
	ob9: GObject:= Get_Object(Object, "combobox3");
	ob10: GObject:= Get_Object(Object, "textbuffer8");
	ob11: GObject:= Get_Object(Object, "textbuffer9");
	ob12: GObject:= Get_Object(Object, "textbuffer10");
	ob13: GObject:= Get_Object(Object, "textbuffer11");
	ob14: GObject:= Get_Object(Object, "textbuffer12");
	ob15: GObject:= Get_Object(Object, "textbuffer13");
	ob16: GObject:= Get_Object(Object, "liststore1");
	ob17: GObject:= Get_Object(Object, "treeview1");

	wid1: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob1));
	wid2: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob2));
	wid3: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob3));
	wid4: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob4));
	wid5: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob5));
	wid6: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob6));
	wid7: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob7));
	wid8: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob8));
	wid9: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob9));
	wid10: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob10));
	wid11: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob11));
	wid12: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob12));
	wid13: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob13));
	wid14: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob14));
	wid15: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob15));
	wid16: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob16));
	wid17: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob17));
	wid18: t_widget := (ptype => my_window, window_value => address_space_window);

begin

	my_tab(1..address_space_table_length) := (new t_widget'(wid1), new t_widget'(wid2), new t_widget'(wid3), new t_widget'(wid4), new t_widget'(wid5), new t_widget'(wid6), new t_widget'(wid7), new t_widget'(wid8), new t_widget'(wid9), new t_widget'(wid10), new t_widget'(wid11), new t_widget'(wid12), new t_widget'(wid13), new t_widget'(wid14), new t_widget'(wid15), new t_widget'(wid16), new t_widget'(wid17), new t_widget'(wid18));

end initialize_Address_Space;



-- Data retrieved from the widgets
--
address_space_Text_Memory_Size, address_space_Stack_Memory_Size, address_space_Data_Memory_Size, address_space_Heap_Memory_Size: Integer;
Address_Space_Name, address_space_Cpu_Name : unbounded_string;
address_space_preemptive : Preemptives_Type;
address_space_quantum : Integer;
address_space_capacity : integer;
address_space_period : integer;
address_space_priority : integer;
address_space_file_name : unbounded_string;
address_space_scheduler : Schedulers_Type;
address_space_automaton : unbounded_string;
address_space_start_time : integer;


procedure Check_Address_Space(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is

	text_Memory_Size_String, Stack_Memory_Size_String, Data_Memory_Size_String, quantum_string, capacity_string, period_string, start_time_string, priority_string, Heap_Memory_Size_String : Unbounded_String;
	Integer_Ok  : Boolean;


begin

	Address_Space_Name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));

	address_space_Cpu_Name := to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(2).combo_value)));

	text_Memory_Size_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(3).buffer_value)));
	if (text_Memory_Size_String = "") then
		address_space_text_Memory_Size := 0;
	else
		to_integer (text_Memory_Size_String, address_space_text_Memory_Size, Integer_Ok);
		if not Integer_Ok then
    Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String(Lb_Text_Memory_Size (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Stack_Memory_Size_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(4).buffer_value)));
	if (Stack_Memory_Size_String = "") then
		address_space_Stack_Memory_Size := 0;
	else
         to_integer (Stack_Memory_Size_String, address_space_Stack_Memory_Size, Integer_Ok);
         if not Integer_Ok then
    Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String(Lb_Stack_Memory_Size (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

	Data_Memory_Size_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(5).buffer_value)));
	if (Data_Memory_Size_String = "") then
         address_space_Data_Memory_Size := 0;
      else
         to_integer (Data_Memory_Size_String, address_space_Data_Memory_Size, Integer_Ok);
         if not Integer_Ok then
    Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String(Lb_Data_Memory_Size (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

	Heap_Memory_Size_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(6).buffer_value)));
	if (Heap_Memory_Size_String = "") then
         address_space_Heap_Memory_Size := 0;
      else
         to_integer (Heap_Memory_Size_String, address_space_Heap_Memory_Size, Integer_Ok);
         if not Integer_Ok then
    Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String ( Lb_Heap_Memory_Size (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

address_space_scheduler := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(7).combo_value)))).schedulers_value;

       quantum_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(8).buffer_value)));
        if (Quantum_String = "") then
         address_space_Quantum := 0;
      else
         to_integer (Quantum_String, address_space_Quantum, Integer_Ok);
         if not Integer_Ok then
        Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String
               ( "Quantum" & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

        address_space_preemptive := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(9).combo_value)))).preemptives_value;

        address_space_automaton := get_value_buffer_string(Gtk_Text_Buffer(my_tab(10).buffer_value));

        capacity_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(11).buffer_value)));
        if (capacity_String = "") then
         address_space_capacity := 0;
      else
         to_integer (capacity_String, address_space_capacity, Integer_Ok);
         if not Integer_Ok then
        Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_Capacity (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

        period_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(12).buffer_value)));
        if (period_String = "") then
                address_space_period := 0;
        else
                to_integer (period_String, address_space_period, Integer_Ok);
                if not Integer_Ok then
        Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_Period (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

   priority_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(13).buffer_value)));
        if (priority_String = "") then
                address_space_priority := 0;
        else
                to_integer (priority_String, address_space_priority, Integer_Ok);
                if not Integer_Ok then
        Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_priority (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

    address_space_file_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(14).buffer_value));

        start_time_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(15).buffer_value)));
        if (start_time_String = "") then
                address_space_start_time := 0;
        else
                to_integer (start_time_String, address_space_start_time, Integer_Ok);
                if not Integer_Ok then
        Raise_Exception
           (address_space_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_start_time (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

	Check_Address_Space (Sys.Address_Spaces, Address_Space_Name, address_space_Cpu_Name, address_space_Text_Memory_Size, address_space_Stack_Memory_Size, address_space_Data_Memory_Size, address_space_Heap_Memory_Size, address_space_preemptive, address_space_quantum, address_space_file_name, address_space_scheduler, address_space_automaton);

end Check_Address_Space;


procedure Add_In_Sys_Address_Space(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is

begin

	address_space_set.Add_Address_Space (Sys.Address_Spaces, Address_Space_Name, address_space_Cpu_Name, address_space_Text_Memory_Size, address_space_Stack_Memory_Size, address_space_Data_Memory_Size, address_space_Heap_Memory_Size, address_space_preemptive, address_space_quantum, address_space_file_name, address_space_scheduler, address_space_automaton);

end Add_In_Sys_Address_Space;


procedure Deleted_In_Sys_Address_Space(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is
	An_Address_Space : Address_Space_Ptr;
	name : unbounded_string;
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	An_Address_Space := Search_Address_Space (Sys.Address_Spaces, name);

	check_entity_referencing_an_address_space(sys, An_Address_Space);

	delete (Sys.Address_Spaces, An_Address_Space);

end Deleted_In_Sys_Address_Space;


procedure Modify_In_Sys_Address_Space(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

	An_Address_Space_Buffer : Address_Space_Ptr;
	An_Address_Space : Address_Space_Ptr;
        id : unbounded_string;

begin
	An_Address_Space := Search_Address_Space (Sys.Address_Spaces, address_space_name);
        id:=an_address_space.cheddar_private_id;
        delete(sys.address_spaces, an_address_space);

	address_space_set.Add_Address_Space (Sys.Address_Spaces, an_address_space_buffer, Address_Space_Name, address_space_Cpu_Name, address_space_Text_Memory_Size, address_space_Stack_Memory_Size, address_space_Data_Memory_Size, address_space_Heap_Memory_Size, address_space_preemptive, address_space_quantum, address_space_file_name, address_space_scheduler, address_space_automaton);

        an_address_space_buffer.cheddar_private_id:=id;

end Modify_In_Sys_Address_Space;


procedure Show_In_Sys_Address_Space(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is

begin
	null;
end Show_In_Sys_Address_Space;

package Address_Space_callbacks is new generic_callbacks(Number_of_widget => Address_Space_table_length, Initialize => initialize_Address_Space, Check => Check_Address_Space, Add_In_Sys => Add_In_Sys_Address_Space, Deleted_In_Sys => Deleted_In_Sys_Address_Space, Modify_In_Sys => Modify_In_Sys_Address_Space, Show_In_Sys => Show_In_Sys_Address_Space);  use Address_Space_callbacks;


procedure Show_Address_space_Widget is

	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;
	My_Iterator : Address_Spaces_Iterator;
	An_Address_Space: Address_Space_Ptr;
	tree_iter: Gtk_Tree_Iter;
	liststore: GObject;

	A_Processor : Generic_Processor_Ptr;
	My_Processor_Iterator : Processors_Iterator;
	my_combo: GObject;

begin
	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "address_space.glade");
	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;

        initialize(Sys_buffer);
        duplicate(sys, Sys_buffer);

	my_combo := Get_Object(Builder, "combobox1");
	reset_iterator (sys.Processors, My_Processor_Iterator);
	if not is_empty (sys.Processors) then
	loop
		current_element (sys.Processors, A_Processor, My_Processor_Iterator);
		Append_Text (Gtk_Combo_Box(my_combo), To_String(A_Processor.name));
		exit when is_last_element (sys.Processors, My_Processor_Iterator);

		next_element (sys.Processors, My_Processor_Iterator);
	end loop;
	else
		Show_Message_Box (Lb_Define_processors_Before (Current_Language));
	end if;

	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox1")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox2")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox3")), 0);

	liststore:= Get_Object(Builder, "liststore1");
	reset_iterator (sys.Address_Spaces, My_Iterator);
	if not is_empty (sys.Address_Spaces) then
		loop
		current_element (sys.Address_Spaces, An_Address_Space, My_Iterator);
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (An_Address_Space.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, To_String (An_Address_Space.cpu_name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, Gint(An_Address_Space.text_memory_size));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 3, Gint(An_Address_Space.stack_memory_size));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 4, Gint(An_Address_Space.data_memory_size));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 5, Gint(An_Address_Space.heap_memory_size));


          Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 6, To_String(to_label(new t_all_enumeration_type'(Schedulers, An_Address_Space.scheduling.Scheduler_Type))));
                Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 7, Gint(An_Address_Space.scheduling.quantum));
                Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 8, To_String(to_label(new t_all_enumeration_type'(Preemptives, An_Address_Space.scheduling.preemptive_type))));
                Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 9, To_String(An_Address_Space.scheduling.automaton_name));
                Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 10, Gint(An_Address_Space.scheduling.capacity));
                Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 11, Gint(An_Address_Space.scheduling.period));
                Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 12, Gint(An_Address_Space.scheduling.priority));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 13, To_String (An_Address_Space.scheduling.user_defined_scheduler_source_file_name));
                Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 14, Gint(An_Address_Space.scheduling.start_time));

		exit when is_last_element (sys.Address_Spaces, My_Iterator);
		next_element (sys.Address_Spaces, My_Iterator);
		end loop;
	end if;

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview1_cursor_changed",
	Handler      => Address_Space_callbacks.Show_Lign_Selected'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button6_clicked",
	Handler      => Address_Space_callbacks.Add_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button5_clicked",
	Handler      => Address_Space_callbacks.Modify_Lign'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button4_clicked",
	Handler      => Address_Space_callbacks.Delected_Lign_Selected'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked",
	Handler      => Address_Space_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked",
	Handler      => Address_Space_callbacks.Close_View'Access);

	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
 	Unref (Builder);

end Show_Address_space_Widget;

end graphical_editor.address_spaces;
