------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog; use Gtk.About_Dialog;
with Gtk.Label;        use Gtk.Label;
with Glib.Object; use Glib.Object;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Gtk.Combo_Box; use Gtk.Combo_Box;
with Gtk.Text_Iter;    use Gtk.Text_Iter;
with Gtk.List_Store;   use Gtk.List_Store;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Gtk.Tree_View;  use Gtk.Tree_View;
with Gtk.Tree_View_Column;  use Gtk.Tree_View_Column;
with Glib.Values; use Glib.Values;
with Glib; use Glib;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Ada.Strings.Unbounded;    use Ada.Strings.Unbounded;
with Gtkada.Builder; use Gtkada.Builder;
with graphical_editor.manual_callbacks; use graphical_editor.manual_callbacks;
with convert_unbounded_strings; 
with task_dependencies; use task_dependencies;
use task_dependencies.Half_Dep_Set;
with dependencies; use dependencies;
with task_set; use task_set;
with tasks; use tasks;
with graphical_editor.convert_text; use graphical_editor.convert_text; 
with buffer_set; use buffer_set;
with buffers; use buffers;


package body graphical_editor.black_board_buffer_dependency is

black_board_buffer_table_length : constant natural :=6;

procedure Initialize_black_board_buffer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
ob1: GObject:= Get_Object(Object, "combobox1");
ob2: GObject:= Get_Object(Object, "combobox2");
ob3: GObject:= Get_Object(Object, "combobox3");
ob4: GObject:= Get_Object(Object, "liststore1");
ob5: GObject:= Get_Object(Object, "treeview1");
wid1: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob1));
wid2: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob2));
wid3: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob3));
wid4: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob4));
wid5: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob5));
wid6: t_widget := (ptype => my_window, window_value => black_board_buffer_dependency_window);

begin
	my_tab(1..black_board_buffer_table_length) := (new t_widget'(wid1), new t_widget'(wid2), new t_widget'(wid3), new t_widget'(wid4), new t_widget'(wid5), new t_widget'(wid6) );
end initialize_black_board_buffer;


-- Data retrieved from widget
--
black_board_buffer_task_name: unbounded_string;
black_board_buffer_buffer_name: unbounded_string;
black_board_buffer_orientation : Orientation_Dependency_Type;



procedure Check_black_board_buffer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
black_board_buffer_task_name:=to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(1).combo_value)));
black_board_buffer_buffer_name:=to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(2).combo_value)));
black_board_buffer_orientation:=
           to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(3).combo_value)))).orientations_value;
end check_black_board_buffer;



procedure Add_In_Sys_black_board_buffer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
	 add_one_task_dependency_black_board_buffer(sys.dependencies,
                                    search_task(sys.tasks, black_board_buffer_task_name),
                                    search_buffer(sys.buffers, black_board_buffer_buffer_name),
					black_board_buffer_orientation);
end Add_In_Sys_black_board_buffer;



procedure Deleted_In_Sys_black_board_buffer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
black_board_buffer_task_name:=to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(1).combo_value)));
black_board_buffer_buffer_name:=to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(2).combo_value)));
black_board_buffer_orientation:=
           to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(3).combo_value)))).orientations_value;
      
	 delete_one_task_dependency_black_board_buffer(sys.dependencies,
                                    search_task(sys.tasks, black_board_buffer_task_name),
                                    search_buffer(sys.buffers, black_board_buffer_buffer_name),
					black_board_buffer_orientation);
end Deleted_In_Sys_black_board_buffer;

procedure Show_In_Sys_black_board_buffer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
	null;
end Show_In_Sys_black_board_buffer;

procedure Modify_In_Sys_black_board_buffer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
	null;
end Modify_In_Sys_black_board_buffer;

package black_board_buffer_callbacks is new generic_callbacks(Number_of_widget => black_board_buffer_table_length, Initialize => Initialize_black_board_buffer, 
	Check => Check_black_board_buffer, Add_In_Sys => Add_In_Sys_black_board_buffer, Deleted_In_Sys => Deleted_In_Sys_black_board_buffer, 
	Modify_In_Sys => Modify_In_Sys_black_board_buffer, Show_In_Sys => Show_In_Sys_black_board_buffer);  use black_board_buffer_callbacks;


procedure Show_black_board_buffer_Widget is
 
   Builder : Gtkada_Builder;
   Error   : Glib.Error.GError;
        liststore: GObject;
        tree_iter: Gtk_Tree_Iter;

My_Iterator : Tasks_Dependencies_Iterator;
      A_Half_Dep   : Dependency_Ptr;

  combo_orientation, combo_task, combo_buffer :  GObject; 
        My_Task_Iterator : Tasks_Iterator;
        A_Task : Generic_Task_Ptr;
        My_buffer_Iterator : buffers_Iterator;
        A_buffer : buffer_Ptr;


begin
   Gtk.Main.Init;
                         
   Gtk_New (Builder);
   Error := Add_From_File (Builder, to_string(Glade_path) & "black_board_buffer_dependency.glade");
   if Error /= null then
      Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
      Error_Free (Error);
      return;
   end if;


        initialize(Sys_buffer);
        duplicate(sys, Sys_buffer);


        combo_task := Get_Object(Builder, "combobox1");
        combo_buffer := Get_Object(Builder, "combobox2");
        combo_orientation := Get_Object(Builder, "combobox3");


        reset_iterator (sys.Tasks, My_Task_Iterator);
        if not is_empty (sys.Tasks) then
        loop
        	current_element (sys.Tasks, A_Task, My_Task_Iterator);
        	Append_Text (Gtk_Combo_Box(combo_task), To_String(A_Task.name));
        	exit when is_last_element (sys.Tasks, My_Task_Iterator);
                next_element (sys.Tasks, My_Task_Iterator);
        end loop;
        end if;
        reset_iterator (sys.buffers, My_buffer_Iterator);
        if not is_empty (sys.buffers) then
        loop
        	current_element (sys.buffers, A_buffer, My_buffer_Iterator);
        	Append_Text (Gtk_Combo_Box(combo_buffer), To_String(A_buffer.name));
        	exit when is_last_element (sys.buffers, My_buffer_Iterator);
                next_element (sys.buffers, My_buffer_Iterator);
        end loop;
        end if;

        Set_Active (Gtk_Combo_Box(combo_task), 0);
	Set_Active (Gtk_Combo_Box(combo_buffer), 0);
	Set_Active (Gtk_Combo_Box(combo_orientation), 0);

      if not is_empty (sys.Dependencies.Depends) then

	liststore:= Get_Object(Builder, "liststore1");
         reset_iterator (sys.Dependencies.Depends, My_Iterator);

         loop
            current_element
              (sys.Dependencies.Depends,
               A_Half_Dep,
               My_Iterator);

            if A_Half_Dep.type_of_dependency = dependencies.black_board_buffer_Dependency then
 Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (A_Half_Dep.black_board_dependent_task.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, To_String (A_Half_Dep.black_board_dependency_object.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, to_string(to_label(new t_all_enumeration_type'(orientations, A_Half_Dep.black_board_orientation))));
            end if;

  	exit when is_last_element (sys.Dependencies.Depends, My_Iterator);

            next_element (sys.Dependencies.Depends, My_Iterator);
         end loop;
      end if;



	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => black_board_buffer_callbacks.Add_View'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button4_clicked", 
	Handler      => black_board_buffer_callbacks.Delected_Lign_Selected'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button3_clicked", 
	Handler      => black_board_buffer_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview1_cursor_changed", 
	Handler      => black_board_buffer_callbacks.Show_Lign_Selected'Access);  

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => black_board_buffer_callbacks.Close_View'Access);


    Do_Connect (Builder);

   
  Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
  
   Gtk.Main.Main;
 
   Unref (Builder);
 
end Show_black_board_buffer_Widget;

end graphical_editor.black_board_buffer_dependency;

