------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtkada.Builder; use Gtkada.Builder;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Gtk.Arguments;
 
package graphical_editor.cheddar_callbacks is


   -- File menu
   --
   function new_quit (Object : access Gtkada_Builder_Record'Class) return   Boolean;
   procedure Quit (Object : access Gtkada_Builder_Record'Class);
   procedure new_Quit_proc (Object : access Gtkada_Builder_Record'Class);
   procedure initialyse_system (Object : access Gtkada_Builder_Record'Class);
   procedure open_xml (Object : access Gtkada_Builder_Record'Class);
   procedure import_aadl (Object : access Gtkada_Builder_Record'Class);
   procedure export_aadl (Object : access Gtkada_Builder_Record'Class);
   procedure aadl_option (Object : access Gtkada_Builder_Record'Class); 
   procedure save_xml (Object : access Gtkada_Builder_Record'Class);
   procedure save_as_xml (Object : access Gtkada_Builder_Record'Class);
   procedure export_cheddar_properties_set (Object : access Gtkada_Builder_Record'Class);
   procedure export_aadl_standard_properties_set (Object : access Gtkada_Builder_Record'Class);
   procedure clear_workspace (Object : access Gtkada_Builder_Record'Class);
   procedure clear_workspace;



   -- Help menu
   --
   procedure about (Object : access Gtkada_Builder_Record'Class);
   procedure manual (Object : access Gtkada_Builder_Record'Class);
   procedure scheduling_ref (Object : access Gtkada_Builder_Record'Class);



   -- Edit menu
   --
   procedure cache (Object : access Gtkada_Builder_Record'Class);
   procedure deployment (Object : access Gtkada_Builder_Record'Class);
   procedure core (Object : access Gtkada_Builder_Record'Class);
   procedure processor (Object : access Gtkada_Builder_Record'Class);
   procedure event_analyzer (Object : access Gtkada_Builder_Record'Class);
   procedure network (Object : access Gtkada_Builder_Record'Class);
   procedure address_space (Object : access Gtkada_Builder_Record'Class);
   procedure taskc (Object : access Gtkada_Builder_Record'Class);
   procedure task_group (Object : access Gtkada_Builder_Record'Class);
   procedure message (Object : access Gtkada_Builder_Record'Class);
   procedure buffer (Object : access Gtkada_Builder_Record'Class);
   procedure resource (Object : access Gtkada_Builder_Record'Class);
   procedure time_triggered_dependency(Object : access Gtkada_Builder_Record'Class);
   procedure precedence_dependency (Object : access Gtkada_Builder_Record'Class);
   procedure black_board_buffer_dependency (Object : access Gtkada_Builder_Record'Class);
   procedure resource_dependency (Object : access Gtkada_Builder_Record'Class);
   procedure queueing_buffer_dependency (Object : access Gtkada_Builder_Record'Class);
   procedure asynchronous_communication_dependency (Object : access Gtkada_Builder_Record'Class);


   -- Tool menu
   --
   procedure scheduling_simulation (Object : access Gtkada_Builder_Record'Class);
   procedure export_event_table (Object : access Gtkada_Builder_Record'Class);
   procedure import_event_table (Object : access Gtkada_Builder_Record'Class);
   procedure set_priorities_rm (Object : access Gtkada_Builder_Record'Class);
   procedure set_priorities_dm (Object : access Gtkada_Builder_Record'Class);
   procedure set_priorities_audsley_opa (Object : access Gtkada_Builder_Record'Class);
   procedure set_priorities_opa_crpd_pt (Object : access Gtkada_Builder_Record'Class);
   procedure set_priorities_opa_crpd_pt_simplified (Object : access Gtkada_Builder_Record'Class);
   procedure set_priorities_opa_crpd_tree (Object : access Gtkada_Builder_Record'Class);
   procedure scheduling_feasibility (Object : access Gtkada_Builder_Record'Class);
   procedure scheduling_feasibility_widget (Object : access Gtkada_Builder_Record'Class);
   procedure analysis_buffer_feasibility (Object : access Gtkada_Builder_Record'Class);
   procedure analysis_buffer_simulation (Object : access Gtkada_Builder_Record'Class);
   procedure scheduling_partition_best_fit (Object : access Gtkada_Builder_Record'Class); 
   procedure scheduling_partition_first_fit (Object : access Gtkada_Builder_Record'Class); 
   procedure scheduling_partition_next_fit (Object : access Gtkada_Builder_Record'Class); 
   procedure scheduling_partition_small_task (Object : access Gtkada_Builder_Record'Class); 
   procedure scheduling_partition_general (Object : access Gtkada_Builder_Record'Class); 
   procedure compute_response_time_density (Object : access Gtkada_Builder_Record'Class);
   procedure priorities_display_chetto (Object : access Gtkada_Builder_Record'Class);
   procedure priorities_update_chetto (Object : access Gtkada_Builder_Record'Class);
   procedure deadline_display_chetto (Object : access Gtkada_Builder_Record'Class);
   procedure deadline_update_chetto (Object : access Gtkada_Builder_Record'Class);
   procedure end_to_end_display_one_step (Object : access Gtkada_Builder_Record'Class);
   procedure end_to_end_update_one_step (Object : access Gtkada_Builder_Record'Class);
   procedure end_to_end_display_all_steps (Object : access Gtkada_Builder_Record'Class); 
   procedure end_to_end_update_all_steps (Object : access Gtkada_Builder_Record'Class);
   procedure resource_deadlock (Object : access Gtkada_Builder_Record'Class); 
   procedure resource_priority_inversion (Object : access Gtkada_Builder_Record'Class);
   procedure bound_blocking_display (Object : access Gtkada_Builder_Record'Class);
   procedure bound_blocking_update (Object : access Gtkada_Builder_Record'Class);
   procedure ceiling_priority_display (Object : access Gtkada_Builder_Record'Class);
   procedure ceiling_priority_update (Object : access Gtkada_Builder_Record'Class);
   procedure check_address_space (Object : access Gtkada_Builder_Record'Class);  
   procedure check_address_space_update (Object : access Gtkada_Builder_Record'Class);
   procedure draw_time_line_from_event_table (Object : access Gtkada_Builder_Record'Class);
   procedure run_analysis_on_event_table (Object : access Gtkada_Builder_Record'Class);
   procedure scheduling_option (Object : access Gtkada_Builder_Record'Class);
   procedure tindell_transaction_wcrt (Object : access Gtkada_Builder_Record'Class);
   procedure audsley_transaction_wcrt (Object : access Gtkada_Builder_Record'Class);
   procedure palencia_transaction_wcrt (Object : access Gtkada_Builder_Record'Class);
   procedure wcdops_plus_transaction_wcrt (Object : access Gtkada_Builder_Record'Class);
   procedure wcdops_plus_nim_transaction_wcrt (Object : access Gtkada_Builder_Record'Class);
   procedure compute_task_cache_access_profile  (Object : access Gtkada_Builder_Record'Class);
   procedure export_arinc653_tsp (Object : access Gtkada_Builder_Record'Class);
   procedure design_pattern_analysis(Object : access Gtkada_Builder_Record'Class);
   

end graphical_editor.cheddar_callbacks;
