------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Ada.Text_IO; use Ada.Text_IO;
with Gtk.Combo_Box; 		use Gtk.Combo_Box;
with Gtk.List_Store;   use Gtk.List_Store;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Gtk.Tree_View;  use Gtk.Tree_View;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with Glib.Values; use Glib.Values;
with gtk.tree_view_column; use gtk.tree_view_column;
with deployments; use deployments;
with deployment_set; use deployment_set;
with graphical_editor.cores; use graphical_editor.cores;
with graphical_editor.user_message;    use graphical_editor.user_message;
with ada.Exceptions;                               use ada.Exceptions;
with GNAT.Current_Exception;                               use GNAT.Current_Exception;
with Framework_Config;                                     use Framework_Config;
with scheduler_interface; use scheduler_interface;
with Translate;                                            use Translate;
with objects; use objects;
use objects.Generic_Object_Set_Package;
with objects.extended; use objects.extended;
with sets;
with Tasks;                             use Tasks;
with Task_Set;                          use Task_Set;   
with Task_Groups;                             use Task_Groups;
with Task_Group_Set;                          use Task_Group_Set;
with Resources; use Resources;
with Resource_set; use Resource_set;
with Messages;              use Messages;
with Message_Set;           use Message_Set;
with core_units;   use core_units;
with Processors;                                           use Processors;
with Processor_set;                                           use Processor_set;
use Processor_Set.Generic_Processor_Set;
use Processor_Set.Generic_core_unit_Set;
with buffers;			use buffers;
with buffer_set;			use buffer_set;
with Networks;                                         use Networks;
with Network_Set;                                         use Network_Set;
with Event_Analyzers;    use Event_Analyzers;
with Event_Analyzer_set;    use Event_Analyzer_set;
with address_spaces; 	use address_spaces;
with address_space_set; 	use address_space_set;
with caches; 	use caches;
with cache_set; 	use cache_set;
with deployments; use deployments;
with deployment_set; use deployment_set;
with graphical_editor.convert_text; use graphical_editor.convert_text;
with systems; use systems;



package body graphical_editor.deployments is


deployment_table_length : constant natural := 19;


procedure Initialize_deployment(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	ob1: GObject:= Get_Object(Object, "textbuffer1");
	ob2: GObject:= Get_Object(Object, "combobox3");
	ob3: GObject:= Get_Object(Object, "combobox5");
	ob4: GObject:= Get_Object(Object, "combobox4");
	ob5: GObject:= Get_Object(Object, "textbuffer2");
	ob6: GObject:= Get_Object(Object, "textbuffer4");
	ob7: GObject:= Get_Object(Object, "textbuffer5");
	ob8: GObject:= Get_Object(Object, "textbuffer6");
	ob9: GObject:= Get_Object(Object, "textbuffer7");
	ob10: GObject:= Get_Object(Object, "textbuffer8");
	ob11: GObject:= Get_Object(Object, "textbuffer9");
	ob12: GObject:= Get_Object(Object, "textbuffer10");
	ob13: GObject:= Get_Object(Object, "liststore4");
	ob14: GObject:= Get_Object(Object, "treeview2");
	ob15: GObject:= Get_Object(Object, "liststore5");
	ob16: GObject:= Get_Object(Object, "treeview3");
	ob17: GObject:= Get_Object(Object, "liststore1");
	ob18: GObject:= Get_Object(Object, "treeview1");
	
	wid1: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob1));
	wid2: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob2));
	wid3: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob3));
	wid4: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob4));
	wid5: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob5));
	wid6: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob6));
	wid7: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob7));
	wid8: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob8));
	wid9: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob9));
	wid10: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob10));
	wid11: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob11));
	wid12: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob12));
	wid13: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob13));
	wid14: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob14));
	wid15: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob15));
	wid16: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob16));
	wid17: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob17));
	wid18: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob18));
	wid19: t_widget := (ptype => my_window, window_value => deployment_window);
begin
	my_tab(1..deployment_table_length) := (new t_widget'(wid1), new t_widget'(wid2), new t_widget'(wid3), new t_widget'(wid4), new t_widget'(wid5), new t_widget'(wid6), new t_widget'(wid7), new t_widget'(wid8), new t_widget'(wid9), new t_widget'(wid10), new t_widget'(wid11), new t_widget'(wid12), new t_widget'(wid13), new t_widget'(wid14), new t_widget'(wid15), new t_widget'(wid16), new t_widget'(wid17), new t_widget'(wid18), new t_widget'(wid19)   );
end initialize_deployment;


-- Data retrieved from the widgets
--
deployment_scheduler: Schedulers_Type;
deployment_preemptive: Preemptives_Type;
deployment_type : Deployments_Type;
deployment_name, deployment_automaton_name : unbounded_string;
deployment_file_name, deployment_allocation_description : Unbounded_String;
deployment_start_time, deployment_priority, number_of_resource, number_of_consumer, deployment_Quantum, deployment_capacity, deployment_period : integer; 
deployment_resource_entities, deployment_consumer_entities : Generic_Objects_Set;


procedure Check_deployment(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

	start_time_string, quantum_string, capacity_string, period_string, priority_string : unbounded_string; 
	tree_iter_resource, tree_iter_consumer : Gtk_Tree_Iter;
	object_name_value : GValue;

	Integer_Ok      : Boolean;


begin

	deployment_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));

	deployment_type := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(2).combo_value)))).deployment_value;

	deployment_scheduler := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(3).combo_value)))).schedulers_value;

	deployment_preemptive := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(4).combo_value)))).preemptives_value;

	deployment_automaton_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(5).buffer_value));


	capacity_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(6).buffer_value)));
	if (capacity_String = "") then
         deployment_capacity := 0;
      else
         to_integer (capacity_String, deployment_capacity, Integer_Ok);
         if not Integer_Ok then
          Raise_Exception
           (deployment_set.Invalid_Parameter'Identity,
            To_String
                ( Lb_capacity (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
          end if;
      end if;

	
	period_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(7).buffer_value)));
	if (period_String = "") then
		deployment_period := 0;
	else
		to_integer (period_String, deployment_period, Integer_Ok);
		if not Integer_Ok then
          Raise_Exception
           (deployment_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Period (Current_Language) & Lb_Must_Be_Numeric (Current_Language)));
		end if;
	end if;

	priority_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(8).buffer_value)));
	if (priority_String = "") then
		deployment_priority := 0;
	else
		to_integer (priority_String, deployment_priority, Integer_Ok);
		if not Integer_Ok then
          Raise_Exception
           (deployment_set.Invalid_Parameter'Identity,
            To_String(Lb_priority (Current_Language) & Lb_Must_Be_Numeric (Current_Language)));
		end if;
	end if;

	deployment_file_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(9).buffer_value));

	start_time_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(10).buffer_value)));
	if (start_time_String = "") then
		deployment_start_time := 0;
	else
		to_integer (start_time_String, deployment_start_time, Integer_Ok);
		if not Integer_Ok then
          Raise_Exception
           (deployment_set.Invalid_Parameter'Identity,
            To_String (Lb_start_time (Current_Language) & Lb_Must_Be_Numeric (Current_Language)));
		end if;
	end if;

	quantum_string := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(11).buffer_value)));
	if (Quantum_String = "") then
         deployment_Quantum := 0;
      else
         to_integer (Quantum_String, deployment_Quantum, Integer_Ok);
         if not Integer_Ok then
          Raise_Exception
           (deployment_set.Invalid_Parameter'Identity,
            To_String
            ( "Quantum" & Lb_Must_Be_Numeric (Current_Language)));
            return;
         end if;
      end if;

	deployment_allocation_description := get_value_buffer_string(Gtk_Text_Buffer(my_tab(12).buffer_value));


	number_of_consumer:= get_number_elements(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(14).view_value)));
	tree_iter_consumer := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(14).view_value)));
	for i in 1 .. number_of_consumer loop
		Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(14).view_value)), 
		tree_iter_consumer, 0, object_name_value);
		add (deployment_consumer_entities, search_by_name(sys,to_unbounded_string(Get_String(object_name_value))));
		next(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(14).view_value)), tree_iter_consumer); 
	end loop;

	number_of_resource:= get_number_elements(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(16).view_value)));
	tree_iter_resource := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(16).view_value)));
	for i in 1 .. number_of_resource loop
		Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(16).view_value)), tree_iter_resource, 0, object_name_value);
		add (deployment_resource_entities, search_by_name(sys,to_unbounded_string(Get_String(object_name_value))));
		unset(object_name_value);
		next(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(16).view_value)), tree_iter_resource); 
	end loop;

	Check_deployment (Sys.deployments, deployment_name, deployment_resource_entities, deployment_consumer_entities, empty_string, deployment_capacity, deployment_period, deployment_priority, deployment_preemptive, deployment_Quantum, deployment_file_name, 	deployment_scheduler, deployment_automaton_name);


end Check_deployment;


procedure Add_In_Sys_deployment(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

begin
	add_deployment (Sys.deployments, deployment_name, deployment_resource_entities, deployment_consumer_entities, deployment_allocation_description, deployment_capacity, deployment_period, deployment_priority, deployment_preemptive, deployment_Quantum, deployment_file_name, 	deployment_scheduler, deployment_automaton_name);

end Add_In_Sys_deployment;



procedure Deleted_In_Sys_deployment(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	A_deployment: generic_deployment_ptr; 
	name: unbounded_string;
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	A_deployment := Search_deployment (sys.deployments, Name);

        Check_Entity_Referencing_a_deployment(sys, a_deployment);

	Delete (Sys.deployments, A_deployment);
end Deleted_In_Sys_deployment;



procedure Modify_In_Sys_deployment(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

	A_Deployment, A_Deployment_buffer     : generic_deployment_ptr;
        id : unbounded_string;
begin
        A_deployment := Search_deployment (Sys.deployments, deployment_name);
	id:= A_Deployment.cheddar_private_id;
	Delete (Sys.Deployments, A_Deployment);

	add_deployment (Sys.deployments, a_deployment_buffer, deployment_name, deployment_resource_entities, deployment_consumer_entities, deployment_allocation_description, deployment_capacity, deployment_period, deployment_priority, deployment_preemptive, deployment_Quantum, deployment_file_name, 	deployment_scheduler, deployment_automaton_name);
        
a_deployment_buffer.cheddar_private_id:=id;

end Modify_In_Sys_deployment;



procedure Show_In_Sys_deployment(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	name : unbounded_string;
	A_deployment: generic_deployment_ptr;
	liststore1, liststore2 : GObject;
	tree_iter1, tree_iter2 : Gtk_Tree_Iter;
	consumer_entities_set, resource_entities_set : Generic_Objects_Set;
	My_consumer_entities_set_Iterator, My_resource_entities_set_Iterator : Generic_Object_Set_Package.iterator;
	A_consumer_entities, A_resource_entities : Generic_Object_Ptr;
begin

	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	A_deployment := Search_deployment (sys.deployments, Name);

	liststore1:= Get_Object(Object, "liststore4");
	liststore2:= Get_Object(Object, "liststore5");

	Gtk.List_Store.Clear(Gtk_List_Store(liststore1));
	Gtk.List_Store.Clear(Gtk_List_Store(liststore2));

	consumer_entities_set := A_deployment.consumer_entities;
	resource_entities_set := A_deployment.resource_entities;

	reset_iterator (consumer_entities_set, My_consumer_entities_set_Iterator);
	if not is_empty (consumer_entities_set) then 
	loop
		current_element (consumer_entities_set, A_consumer_entities, My_consumer_entities_set_Iterator);
		Append(Gtk_List_Store(liststore1), tree_iter1);
		Gtk.List_Store.Set(Gtk_List_Store(liststore1), tree_iter1, 0, 
		To_String (get_name_of_generic_object(A_consumer_entities)));
		
	exit when is_last_element (consumer_entities_set, My_consumer_entities_set_Iterator);
		next_element (consumer_entities_set, My_consumer_entities_set_Iterator);
	end loop;
	end if;

	reset_iterator (resource_entities_set, My_resource_entities_set_Iterator);
	if not is_empty (resource_entities_set) then 
	loop
		current_element (resource_entities_set, A_resource_entities, My_resource_entities_set_Iterator);
		Append(Gtk_List_Store(liststore2), tree_iter2);
		Gtk.List_Store.Set(Gtk_List_Store(liststore2), tree_iter2, 0, 
		To_String (get_name_of_generic_object(A_resource_entities)));
		
	exit when is_last_element (resource_entities_set, My_resource_entities_set_Iterator);
		next_element (resource_entities_set, My_resource_entities_set_Iterator);
	end loop;
	end if;

end Show_In_Sys_deployment;



procedure Delected_lign_consumer (Object : access Gtkada_Builder_Record'Class) is 
	liststore : Gobject;
	tree_iter: Gtk_Tree_Iter;
	path1: Gtk_Tree_Path;
	focus1: Gtk_Tree_View_Column;
begin
	liststore := get_object(object, "liststore4");
	Get_Cursor(Gtk_Tree_View(get_object(object, "treeview2")), path1, focus1);
	if path1 /= null and focus1 /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(liststore), path1);
		Gtk.List_Store.Remove(Gtk_List_Store(liststore), tree_iter);
	end if;
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Delected_lign_consumer;



procedure Delected_lign_resource (Object : access Gtkada_Builder_Record'Class) is 
	liststore : Gobject;
	tree_iter: Gtk_Tree_Iter;
	path1: Gtk_Tree_Path;
	focus1: Gtk_Tree_View_Column;
begin
	liststore := get_object(object, "liststore5");
	Get_Cursor(Gtk_Tree_View(get_object(object, "treeview3")), path1, focus1);
	if path1 /= null and focus1 /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(liststore), path1);
		Gtk.List_Store.Remove(Gtk_List_Store(liststore), tree_iter);
	end if;
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Delected_lign_resource;




procedure Add_View_resource (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
begin
	Append(Gtk_List_Store(Get_Object(Object, "liststore5")), tree_iter);
	Gtk.List_Store.Set(Gtk_List_Store(Get_Object(Object, "liststore5")), tree_iter, 0, 
	Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox2"))));
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Add_View_resource;



procedure Add_View_consumer (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
begin
	Append(Gtk_List_Store(Get_Object(Object, "liststore4")), tree_iter);
	Gtk.List_Store.Set(Gtk_List_Store(Get_Object(Object, "liststore4")), tree_iter, 0, 
	Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox1"))));
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Add_View_consumer;


package deployment_callbacks is new generic_callbacks(Number_of_widget => deployment_table_length, 
	Initialize => initialize_deployment, Check => Check_deployment, Add_In_Sys => Add_In_Sys_deployment, 
	Deleted_In_Sys => Deleted_In_Sys_deployment, Modify_In_Sys => Modify_In_Sys_deployment, 
	Show_In_Sys => Show_In_Sys_deployment);  use deployment_callbacks;



procedure Show_Deployment_Widget is 

	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;

	My_Iterator : deployments_Iterator;
	A_deployment: generic_deployment_ptr; 
	tree_iter: Gtk_Tree_Iter;
	liststore: GObject;

	A_Task : Generic_Task_Ptr;
	A_Task_group : Generic_Task_Group_Ptr;
	A_Resource : Generic_Resource_Ptr;
	A_Message : Generic_Message_Ptr;
	A_Core_units : Core_Unit_Ptr;
	A_Processor : Generic_Processor_Ptr;
	A_Buffer : Buffer_Ptr;
	A_Network : Network_Ptr;
	An_Address_Space: Address_Space_Ptr;
	A_Cache : Generic_Cache_Ptr;
	
	My_Task_Iterator : Tasks_Iterator;
	My_Task_group_Iterator : Task_Groups_Iterator;
	My_Resource_Iterator : Resources_Iterator;
	My_Message_Iterator : Messages_Iterator;
	My_Core_units_Iterator : core_units_Iterator;
	My_Processor_Iterator : Processors_Iterator;
	My_Buffer_Iterator : buffers_Iterator;
	My_Network_Iterator : Network_Iterator;
	My_Address_Space_Iterator : Address_Spaces_Iterator;
	My_Cache_Iterator : Caches_Iterator;

	my_combo_consumer, my_combo_resource : GObject;

begin
	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "deployment.glade");
	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;


	initialize(Sys_buffer);
        duplicate(sys, Sys_buffer);



	my_combo_consumer := Get_Object(Builder, "combobox1");
	my_combo_resource := Get_Object(Builder, "combobox2");

	reset_iterator (sys.Tasks, My_Task_Iterator);
	if not is_empty (sys.Tasks) then 
	loop
	current_element (sys.Tasks, A_Task, My_Task_Iterator);
		
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Task.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Task.name));

	exit when is_last_element (sys.Tasks, My_Task_Iterator);
	next_element (sys.Tasks, My_Task_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Task_Groups, My_Task_group_Iterator);
	if not is_empty (sys.Task_Groups) then 
	loop
	current_element (sys.Task_Groups, A_Task_group, My_Task_group_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Task_group.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Task_group.name));

	exit when is_last_element (sys.Task_Groups, My_Task_group_Iterator);
	next_element (sys.Task_Groups, My_Task_group_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Resources, My_Resource_Iterator);
	if not is_empty (sys.Resources) then 
	loop
	current_element (sys.Resources, A_Resource, My_Resource_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Resource.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Resource.name));

	exit when is_last_element (sys.Resources, My_Resource_Iterator);
	next_element (sys.Resources, My_Resource_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Messages, My_Message_Iterator);
	if not is_empty (sys.Messages) then 
	loop
	current_element (sys.Messages, A_Message, My_Message_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Message.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Message.name));

	exit when is_last_element (sys.Messages, My_Message_Iterator);
	next_element (sys.Messages, My_Message_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Core_units, My_Core_units_Iterator);
	if not is_empty (sys.Core_units) then 
	loop
	current_element (sys.Core_units, A_Core_units, My_Core_units_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Core_units.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Core_units.name));

	exit when is_last_element (sys.Core_units, My_Core_units_Iterator);
	next_element (sys.Core_units, My_Core_units_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Processors, My_Processor_Iterator);
	if not is_empty (sys.Processors) then 
	loop
	current_element (sys.Processors, A_Processor, My_Processor_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Processor.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Processor.name));

	exit when is_last_element (sys.Processors, My_Processor_Iterator);
	next_element (sys.Processors, My_Processor_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Buffers, My_Buffer_Iterator);
	if not is_empty (sys.Buffers) then 
	loop
	current_element (sys.Buffers, A_Buffer, My_Buffer_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Buffer.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Buffer.name));

	exit when is_last_element (sys.Buffers, My_Buffer_Iterator);
	next_element (sys.Buffers, My_Buffer_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Networks, My_Network_Iterator);
	if not is_empty (sys.Networks) then 
	loop
	current_element (sys.Networks, A_Network, My_Network_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Network.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Network.name));

	exit when is_last_element (sys.Networks, My_Network_Iterator);
	next_element (sys.Networks, My_Network_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Address_Spaces, My_Address_Space_Iterator);
	if not is_empty (sys.Address_Spaces) then 
	loop
	current_element (sys.Address_Spaces, An_Address_Space, My_Address_Space_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(An_Address_Space.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(An_Address_Space.name));

	exit when is_last_element (sys.Address_Spaces, My_Address_Space_Iterator);
	next_element (sys.Address_Spaces, My_Address_Space_Iterator);
	end loop;
	end if;

	reset_iterator (sys.Caches, My_Cache_Iterator);
	if not is_empty (sys.Caches) then 
	loop
	current_element (sys.Caches, A_Cache, My_Cache_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_consumer), To_String(A_Cache.name));
	Append_Text (Gtk_Combo_Box(my_combo_resource), To_String(A_Cache.name));

	exit when is_last_element (sys.Caches, My_Cache_Iterator);
	next_element (sys.Caches, My_Cache_Iterator);
	end loop;
	end if;

	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox1")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox2")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox3")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox4")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox5")), 0);


	liststore:= Get_Object(Builder, "liststore1");
	reset_iterator (sys.Deployments, My_Iterator);
	if not is_empty (sys.Deployments) then
		loop
		current_element (sys.Deployments, A_deployment, My_Iterator);

		Append(Gtk_List_Store(liststore), tree_iter);

		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (A_deployment.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, To_String(to_label(new t_all_enumeration_type'(deployment_enum, A_deployment.deployment_type))));


if A_deployment.deployment_type = dynamic_deployment_type
then
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, To_String(to_label(new t_all_enumeration_type'(Schedulers, dynamic_deployment_ptr(A_deployment).allocation_parameters.scheduler_type))));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 3, To_String(to_label(new t_all_enumeration_type'(Preemptives, dynamic_deployment_ptr(A_deployment).allocation_parameters.preemptive_type))));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 4, To_String(dynamic_deployment_ptr(A_deployment).allocation_parameters.automaton_name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 5, Gint(dynamic_deployment_ptr(A_deployment).allocation_parameters.capacity));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 6, Gint(dynamic_deployment_ptr(A_deployment).allocation_parameters.period));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 7, Gint(dynamic_deployment_ptr(A_deployment).allocation_parameters.priority));
 		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 8, To_String(dynamic_deployment_ptr(A_deployment).allocation_parameters.user_defined_scheduler_source_file_name));
 		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 9, Gint(dynamic_deployment_ptr(A_deployment).allocation_parameters.start_time));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 10, Gint(dynamic_deployment_ptr(A_deployment).allocation_parameters.quantum));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 11, "");

else

		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, "Rate Monotonic Protocol");
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 3, "Preemptive");
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 4, "");
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 5, 0);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 6, 0);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 7, 0);
 		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 8, "");
 		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 9, 0);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 10, 0);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 11, "");
end if;
		

		exit when is_last_element (sys.Deployments, My_Iterator);
		next_element (sys.Deployments, My_Iterator);
		end loop;
	end if;

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_treeview1_cursor_changed", 
		Handler      => deployment_callbacks.Show_Lign_Selected'Access);  

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button1_clicked", 
		Handler      => deployment_callbacks.Close_View'Access);

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button2_clicked", 
		Handler      => deployment_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button5_clicked", 
		Handler      => deployment_callbacks.Add_View'Access);

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button9_clicked", 
		Handler      => Add_View_resource'Access);

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button7_clicked", 
		Handler      => Add_View_consumer'Access);

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button4_clicked", 
		Handler      => deployment_callbacks.Modify_Lign'Access); 

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button3_clicked", 
		Handler      => deployment_callbacks.Delected_Lign_Selected'Access); 

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button8_clicked", 
		Handler      => Delected_lign_consumer'Access); 

	Gtkada.Builder.Register_Handler
		(Builder      => Builder,
		Handler_Name => "on_button10_clicked", 
		Handler      => Delected_lign_resource'Access); 


	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
	Unref (Builder);

end Show_Deployment_Widget;

end graphical_editor.deployments;



