------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 15:09:39 +0200 (mer. 26 sept. 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Gdk;
with Gdk.Event;           use Gdk.Event;
with Gtk;                 use Gtk;
with graphical_editor.Double_Buffer;       use graphical_editor.Double_Buffer;
with Gtk.Widget;          use Gtk.Widget;
with Gdk.Color;           use Gdk.Color;
with Gdk.Font;            use Gdk.Font;
with Glib;                use Glib;
with Gdk.GC;              use Gdk.GC;
with Buffers;             use Buffers;
with Scheduling_Analysis; use Scheduling_Analysis;
with Time_Unit_Events;    use Time_Unit_Events;
use Time_Unit_Events.Time_Unit_Package;
use Time_Unit_Events.Time_Unit_Lists_Package;

with gtkada.builder; use gtkada.builder;

package graphical_editor.draw_buffer is

   type Draw_Buffer_Record is new Gtk_Double_Buffer_Record with record
      Font : Gdk_Font;

      Buff             : Buffer_Size_Table_Ptr;
      Last_Time_Unit   : Natural;
      Time_Unit_Number : Time_Unit_Range;
      Max_Buff         : Natural;

      White_Gc  : Gdk.GC.Gdk_GC;
      Black_Gc  : Gdk.GC.Gdk_GC;
      Red_Color : Gdk_Color;

      Left_Margin     : Gint := 60;
      High_Margin     : Gint := 20;
      Buffer_Margin   : Gint := 20;
      Timeunit_Margin : Gint := 20;

      Current_Width  : Gint;
      Current_Height : Gint;

      Have_Colors : Boolean := False;
      Initialized : Boolean := False;

      Buff_Ptr : Buffer_Ptr;
   end record;

   type Draw_Buffer_Access is access all Draw_Buffer_Record'Class;

   procedure Gtk_New (Draw_Buffer : out Draw_Buffer_Access; Object : access Gtkada_Builder_Record'Class);
   procedure Initialize (Draw_Buffer : access Draw_Buffer_Record'Class; Object : access Gtkada_Builder_Record'Class);

   procedure Redraw (Draw_Buffer : access Draw_Buffer_Record'Class);

procedure Redraw_mod (Object : access Gtkada_Builder_Record'Class; Draw_Buffer       : access Draw_Buffer_Record'Class);

   procedure Draw_mod
     (Object : access Gtkada_Builder_Record'Class;
	Draw_Buffer       : access Draw_Buffer_Record'Class;
	Buff              : Buffer_Ptr;
      Buff1             : Buffer_Size_Table;
      Last_Time_Unit1   : Natural;
      Time_Unit_Number1 : Time_Unit_Range);

   procedure Draw
     (Draw_Buffer       : access Draw_Buffer_Record'Class;
      Buff              : Buffer_Ptr;
      Buff1             : Buffer_Size_Table;
      Last_Time_Unit1   : Natural;
      Time_Unit_Number1 : Time_Unit_Range);

   procedure Clear (Draw_Buffer : access Draw_Buffer_Record'Class);

   function Configure
     (Draw_Buffer : access Draw_Buffer_Record'Class;
      Event       : Gdk.Event.Gdk_Event)
      return        Boolean;

   procedure Color_Allocation
     (Draw_Buffer : access Draw_Buffer_Record'Class);

   procedure Size_Allocate
     (Draw_Buffer : access Draw_Buffer_Record'Class;
      Allocation  : Gtk_Allocation_Access);

   Draw_Buffer : Draw_Buffer_Access := new draw_buffer_record;

   pragma Inline (Redraw);
   pragma Inline (Color_Allocation);
   pragma Inline (Clear);

end graphical_editor.draw_buffer;
