------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog; use Gtk.About_Dialog;
with Gtk.Label;        use Gtk.Label;
with Ada.Text_IO; use Ada.Text_IO;
with Gtkada.Builder; use Gtkada.Builder;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Glib;      use Glib;
with Event_Analyzers;    use Event_Analyzers;
with Event_Analyzer_set;    use Event_Analyzer_set;
with Gtk.Text_Iter;    use Gtk.Text_Iter;
with Gtk.List_Store;   use Gtk.List_Store;
with Gtk.Tree_View;  use Gtk.Tree_View;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with Glib.Values; use Glib.Values;
with Framework_Config;                                     use Framework_Config;
with Translate;                                            use Translate;
with graphical_editor.user_message;    use graphical_editor.user_message;
with GNAT.Current_Exception;                               use GNAT.Current_Exception;

package body graphical_editor.event_analyzers is


Event_Analyzer_table_length : constant natural :=5;

procedure Initialize_Event_Analyzer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

	ob1: GObject:= Get_Object(Object, "textbuffer2");
	ob2: GObject:= Get_Object(Object, "textbuffer1");
	ob3: GObject:= Get_Object(Object, "liststore1");
	ob4: GObject:= Get_Object(Object, "treeview1");
	wid1: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob1));
	wid2: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob2));
	wid3: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob3));
	wid4: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob4));
	wid5: t_widget := (ptype => my_window, window_value => event_analyser_Window);

begin
	my_tab(1..Event_Analyzer_table_length) := (new t_widget'(wid1), new t_widget'(wid2), new t_widget'(wid3), new t_widget'(wid4), 
	new t_widget'(wid5));
end Initialize_Event_Analyzer;



-- Data retrieved from widgets
--
event_file_name, event_name : unbounded_string;



procedure Check_Event_Analyzer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
	event_file_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(2).buffer_value));
	event_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));

	Check_Event_Analyzer (sys.Event_Analyzers, event_name, event_File_Name);

end Check_Event_Analyzer;


procedure Add_In_Sys_Event_Analyzer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
	Add_Event_Analyzer (sys.Event_Analyzers, event_Name, event_File_Name); 

end Add_In_Sys_Event_Analyzer;



procedure Deleted_In_Sys_Event_Analyzer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	An_Event_Analyzer     : Event_Analyzer_Ptr;
	name: unbounded_string;
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	An_Event_Analyzer := Search_Event_Analyzer (Sys.Event_Analyzers, name);
	delete (Sys.Event_Analyzers, An_Event_Analyzer);
end Deleted_In_Sys_Event_Analyzer;



procedure Show_In_Sys_Event_Analyzer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

begin
	null;
end Show_In_Sys_Event_Analyzer;



procedure Modify_In_Sys_Event_Analyzer(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	An_Event_Analyzer_Buffer     : Event_Analyzer_Ptr;

begin
	An_Event_Analyzer_Buffer := Search_Event_Analyzer (Sys.Event_Analyzers, event_name);
	
	An_Event_Analyzer_Buffer.event_analyzer_source_file_name := event_file_name;

end Modify_In_Sys_Event_Analyzer;


package Event_Analyzer_callbacks is new generic_callbacks(Number_of_widget => Event_Analyzer_table_length, 
	Initialize => initialize_Event_Analyzer, Check => Check_Event_Analyzer, Add_In_Sys => Add_In_Sys_Event_Analyzer, 
	Deleted_In_Sys => Deleted_In_Sys_Event_Analyzer, Modify_In_Sys => Modify_In_Sys_Event_Analyzer, 
	Show_In_Sys => Show_In_Sys_Event_Analyzer);  use Event_Analyzer_callbacks;



procedure Show_Event_Analyzer_Widget is
	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;

	An_Event_Analyzer : Event_Analyzer_Ptr;
	My_Iterator       : Event_Analyzers_Iterator;
	tree_iter: Gtk_Tree_Iter;
	liststore: GObject;

begin

	Gtk.Main.Init;
        Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "event_analyzer.glade");
	if Error /= null then 
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;


	initialize(Sys_buffer);
        duplicate(sys, Sys_buffer);

	
	liststore:= Get_Object(Builder, "liststore1");
	reset_iterator (Sys.Event_Analyzers, My_Iterator);
	if not is_empty (Sys.Event_Analyzers) then
		loop
		current_element (sys.Event_Analyzers, An_Event_Analyzer, My_Iterator);
		Append(Gtk_List_Store(liststore), tree_iter);

		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, 
		To_String (An_Event_Analyzer.event_analyzer_source_file_name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, 
		To_String (An_Event_Analyzer.name));

		exit when is_last_element (sys.Event_Analyzers, My_Iterator);
		next_element (sys.Event_Analyzers, My_Iterator);
	         end loop;
      end if;

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview1_cursor_changed", 
	Handler      => Event_Analyzer_callbacks.Show_Lign_Selected'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button6_clicked", 
	Handler      => Event_Analyzer_callbacks.Add_View'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button5_clicked", 
	Handler      => Event_Analyzer_callbacks.Modify_Lign'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button4_clicked", 
	Handler      => Event_Analyzer_callbacks.Delected_Lign_Selected'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => Event_Analyzer_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => Event_Analyzer_callbacks.Close_View'Access);

	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
	Unref (Builder);
 
end Show_Event_Analyzer_Widget;

end graphical_editor.event_analyzers;
