------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtkada.Builder; use Gtkada.Builder;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with Glib.Error;     use Glib.Error;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Ada.Text_IO;      use Ada.Text_IO;
with graphical_editor.event_tables_callbacks; use graphical_editor.event_tables_callbacks;

package body graphical_editor.event_tables is


procedure show_draw_time_line_from_event_table_widget is
 
	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;
	buf1, buf2 : GObject;

begin

	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "draw_time_line_from_event_table.glade");
	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;

	buf1:=Get_Object(Builder, "textbuffer1");
	buf2:=Get_Object(Builder, "textbuffer2");

	Set_Text(Gtk_Text_Buffer(buf1), "0");
	Set_Text(Gtk_Text_Buffer(buf2), "0");

	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => graphical_editor.event_tables_callbacks.draw_time_line_from_event_table'Access);

	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => graphical_editor.event_tables_callbacks.close_widget'Access);

	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
	Unref (Builder);

end show_draw_time_line_from_event_table_widget;

end graphical_editor.event_tables;
