
with Ada.IO_Exceptions;                use Ada.IO_Exceptions;
with GNAT.Current_Exception;           use GNAT.Current_Exception;
with graphical_editor.user_message;    use graphical_editor.user_message;
with translate; use translate;
with framework_config; use framework_config;


with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Object;     use Gtk.Object;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.Label;        use Gtk.Label;
with Gtk.Text_Tag; use Gtk.Text_Tag;
with Ada.Text_IO;
with Gtkada.Builder; use Gtkada.Builder;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with graphical_editor.manual_callbacks; use graphical_editor.manual_callbacks;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with io_tools; use io_tools;


package body graphical_editor.manuals is

procedure Show_Manual_Widget is

	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;
	Buffer_text : GObject;
	file_content : unbounded_string;

begin

	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "manual.glade");
	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;

	Buffer_text:=Get_Object(Builder, "textbuffer1");


      begin
	file_content:=read_sequential_file(to_string(Glade_path) & "manual_fr.txt");
      exception
         when Ada.IO_Exceptions.Name_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", manual text file, Name_Error"));
         when Ada.IO_Exceptions.Status_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", manual text file, Status_Error"));
         when Ada.IO_Exceptions.Mode_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", manual text file, Mode_Error"));
         when Ada.IO_Exceptions.Use_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", manual text file, Use_Error"));
         when Ada.IO_Exceptions.Device_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", manual text file, Device_Error"));
         when Ada.IO_Exceptions.End_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", manual text file, End_Error"));
         when Ada.IO_Exceptions.Data_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (" manual text file, Data_Error"));
         when Ada.IO_Exceptions.Layout_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", manual text file, Layout_Error"));
         when others =>
            Show_Message_Box
              (Lb_Can_Not_Read_File (Current_Language) &
               ", manual text file ; " &
               Exception_Name &
               " ; " &
               Exception_Message);
      end;


	Gtk.Text_Buffer.Set_Text(Gtk_Text_Buffer(Buffer_text), to_string(file_content));


	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => graphical_editor.manual_callbacks.Close_Manual_Widget'Access);

	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
	Unref (Builder);
 
end Show_Manual_Widget;

end graphical_editor.manuals;
