------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog; use Gtk.About_Dialog;
with Gtk.Label;        use Gtk.Label;
with Gtk.tree_view_column; use Gtk.tree_view_column;
with Ada.Text_IO;
with Ada.Exceptions;                                    use Ada.Exceptions;
with Gtkada.Builder; use Gtkada.Builder;
with Glib.Values; use Glib.Values;
with Glib; use Glib;
with Glib.Object; use Glib.Object;
with Messages;      use Messages; 
with Message_Set;    use Message_Set;  
use Message_Set.Generic_Message_Set;
with Gtk.List_Store;   use Gtk.List_Store;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Gtk.Combo_Box; use Gtk.Combo_Box;
with Gtk.Tree_View;  use Gtk.Tree_View;  
with Parameters;   use Parameters;
use parameters.User_Defined_Parameters_Table_Package;
with parameters.extended; use parameters.extended;
with GNAT.Current_Exception;                               use GNAT.Current_Exception;
with Framework_Config;                                     use Framework_Config;     
with Translate;                                            use Translate; 
with graphical_editor.user_message;    use graphical_editor.user_message;  
with graphical_editor.convert_text; use graphical_editor.convert_text;
with Ada.Numerics.Aux;      use Ada.Numerics.Aux;
with Ada.Text_IO;	use Ada.Text_IO;
with Ada.Strings.Unbounded;	use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;

package body graphical_editor.messages is


message_table_length : constant natural :=13;


procedure Initialize_Message(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	ob1: GObject:= Get_Object(Object, "textbuffer1");
	ob2: GObject:= Get_Object(Object, "combobox1");
	ob3: GObject:= Get_Object(Object, "textbuffer2");
	ob4: GObject:= Get_Object(Object, "textbuffer3");
	ob5: GObject:= Get_Object(Object, "textbuffer4");
	ob6: GObject:= Get_Object(Object, "textbuffer5");
	ob7: GObject:= Get_Object(Object, "textbuffer6");
	ob8: GObject:= Get_Object(Object, "textbuffer7");
	ob9: GObject:= Get_Object(Object, "liststore4");
	ob10: GObject:= Get_Object(Object, "treeview2");
	ob11: GObject:= Get_Object(Object, "liststore2");
	ob12: GObject:= Get_Object(Object, "treeview1");
	wid1: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob1));
	wid2: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob2));
	wid3: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob3));
	wid4: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob4));
	wid5: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob5));
	wid6: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob6));
	wid7: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob7));
	wid8: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob8));
	wid9: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob9));
	wid10: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob10));
	wid11: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob11));
	wid12: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob12));
	wid13: t_widget := (ptype => my_window, window_value => Message_Window);

begin

	my_tab(1..message_table_length) := (new t_widget'(wid1), new t_widget'(wid2), 
	new t_widget'(wid3), new t_widget'(wid4), new t_widget'(wid5), new t_widget'(wid6), 
	new t_widget'(wid7), new t_widget'(wid8), new t_widget'(wid9), new t_widget'(wid10), 
	new t_widget'(wid11), new t_widget'(wid12), new t_widget'(wid13));

end initialize_Message;



-- Data retrieved from the widgets
--
message_Name : Unbounded_String;
message_type : Messages_Type;
message_Size, message_Period, message_Deadline, message_Jitter, message_Response_Time, message_Communication_Time : Natural;
message_Param : User_Defined_Parameters_Table;
number_of_param : integer;



procedure Check_message(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	Size_String, Period_String, Deadline_String, Jitter_String, Response_Time_String, 
	Communication_Time_String : Unbounded_String;
	a_param : Parameter_Ptr;
	tree_iter_inter_param: Gtk_Tree_Iter;
	param_name, param_type, param_value : Gvalue;
	integer_ok : boolean;
begin

	message_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));

  message_type := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(2).combo_value)))).messages_value;

     Deadline_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(3).buffer_value)));
      if Deadline_String = "" then
         message_Deadline := 0;
      else
         to_integer (Deadline_String, message_Deadline, integer_Ok);
         if not integer_Ok then
       Raise_Exception
           (message_set.Invalid_Parameter'Identity,
            To_String
            ( Lb_Deadline (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

     Size_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(4).buffer_value)));
      if Size_String = "" then
         message_Size := 0;
      else
         to_integer (Size_String, message_Size, integer_Ok);
         if not integer_Ok then
       Raise_Exception
           (message_set.Invalid_Parameter'Identity,
            To_String
                ( Lb_Size (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

     Response_Time_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(5).buffer_value)));
      if Response_Time_String = "" then
         message_Response_Time := 0;
      else
         to_integer (Response_Time_String, message_Response_Time, integer_Ok);
         if not integer_Ok then
       Raise_Exception
           (message_set.Invalid_Parameter'Identity,
            To_String
            ( Lb_Response_Time (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

     Communication_Time_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(6).buffer_value)));
      if Communication_Time_String = "" then
         message_Communication_Time := 0;
      else
         to_integer (Communication_Time_String, message_Communication_Time, integer_Ok);
         if not integer_Ok then
       Raise_Exception
           (message_set.Invalid_Parameter'Identity,
            To_String
            ( Lb_Communication_Time_Message (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

     Period_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(7).buffer_value)));
      if Period_String = "" then
         message_Period := 0;
      else
         to_integer (Period_String, message_Period, integer_Ok);
         if not integer_Ok then
       Raise_Exception
           (message_set.Invalid_Parameter'Identity,
            To_String
                ( Lb_Period (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;

     Jitter_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(8).buffer_value)));
      if Jitter_String = "" then
         message_Jitter := 0;
      else
         to_integer (Jitter_String, message_Jitter, integer_Ok);
         if not integer_Ok then
       Raise_Exception
           (message_set.Invalid_Parameter'Identity,
            To_String
            ( Lb_Jitter (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
         end if;
      end if;


initialize(message_param);
number_of_param:= get_number_elements(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)));
if number_of_param = 0 then 
	message_Param := No_User_Defined_Parameter;
else
	tree_iter_inter_param := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)));
	for i in 1..number_of_param
	loop
		Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)), 
		tree_iter_inter_param, 0, param_name);
		Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)), 
		tree_iter_inter_param, 1, param_type);
		Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)), 
		tree_iter_inter_param, 2, param_value);
				case to_type(param_type).params_value is 
					when boolean_parameter => 
					a_param := new parameter'(boolean_parameter, to_unbounded_string(Get_string(param_name)), 
					boolean'value(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(my_tab(9).list_value), 
					tree_iter_inter_param, 2)));
					when integer_parameter => 
					a_param := new parameter'(integer_parameter, to_unbounded_string(Get_string(param_name)),  
					integer'value(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(my_tab(9).list_value), 
					tree_iter_inter_param, 2)));	
					when double_parameter => 
					a_param := new parameter'(double_parameter, to_unbounded_string(Get_string(param_name)), 
					Ada.Numerics.Aux.Double'value(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(my_tab(9).list_value), 
					tree_iter_inter_param, 2)));
					when string_parameter => 
					a_param := new parameter'(string_parameter, to_unbounded_string(Get_string(param_name)), 
					to_unbounded_string(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(my_tab(9).list_value), 
					tree_iter_inter_param, 2)));
				end case;

				User_Defined_Parameters_Table_Package.add(message_param, a_param);
				next(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)), tree_iter_inter_param);
			end loop;			
		end if;

	Check_Message (sys.messages, message_Name, message_Size, message_Period, message_Deadline, message_Jitter, message_param, message_Response_Time, message_Communication_Time);

end Check_message;



procedure Add_In_Sys_message(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

begin

	Add_Message (sys.Messages, message_Name, message_Size, message_Period, message_Deadline, message_Jitter, message_Param, message_Response_Time, message_Communication_Time);

end Add_In_Sys_message;




procedure Deleted_In_Sys_message(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	a_message:Generic_Message_Ptr;
	name: unbounded_string;
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	a_message := Search_message (Sys.messages, name);

	Check_Entity_Referencing_a_message(sys, a_message);

	Delete (Sys.messages, a_message);
end Deleted_In_Sys_message;



procedure Show_In_Sys_message(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	name : unbounded_string;
	A_Message      : Generic_Message_Ptr;
	tree_iter: Gtk_Tree_Iter;
	liststore: GObject; 
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	A_Message := Search_Message (Sys.Messages, name);

	liststore:= Get_Object(Object, "liststore4");
	Gtk.List_Store.Clear(Gtk_List_Store(liststore));

	for i in 0..A_Message.parameters.Nb_Entries -1
	loop
	
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (A_Message.parameters.Entries(i).parameter_name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, 
		To_String(to_label(new t_all_enumeration_type'(Parameters_enum, A_Message.parameters.Entries(i).type_of_parameter))));

		case A_Message.parameters.Entries(i).type_of_parameter is 
		when boolean_parameter =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, boolean'image(A_Message.parameters.Entries(i).boolean_value));
		when integer_parameter =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, integer'image(A_Message.parameters.Entries(i).integer_value));
		when double_parameter =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, double'image(A_Message.parameters.Entries(i).double_value));
		when string_parameter =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, to_string(A_Message.parameters.Entries(i).string_value));
		end case;
		
	end loop;
end Show_In_Sys_message;


procedure Modify_In_Sys_message(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

	a_message, a_message_buffer :Generic_Message_Ptr;
 id : unbounded_string;

begin
	a_message := Search_message (Sys.messages, message_name);
        id:=a_message.cheddar_private_id;

	Delete (Sys.messages, a_message);

	Add_Message (sys.Messages, a_message_buffer, message_Name, message_Size, message_Period, message_Deadline, message_Jitter, message_Param, message_Response_Time, message_Communication_Time);
	a_message_buffer.cheddar_private_id:=id;
			
end Modify_In_Sys_message;



procedure Add_view_parameters (Object : access Gtkada_Builder_Record'Class) is 

	tree_iter: Gtk_Tree_Iter;
	liststore : GObject;
	param_name, param_value: unbounded_string;
	param_type : Parameters.Parameter_type;
	ok : boolean;
	param_value_integer : integer;
	param_value_double : double;
	param_value_boolean: boolean;

begin
	liststore:= Get_Object(Object, "liststore4");

	param_name := get_value_buffer_string(Gtk_Text_Buffer(Get_Object(Object, "textbuffer8")));
	param_type := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox2"))))).params_value;
	param_value := suppress_space(to_string(get_value_buffer_string(Gtk_Text_Buffer(Get_Object(Object, "textbuffer9")))));


     case param_type is
       when  Boolean_Parameter =>
		if param_value = "" then
			param_value_boolean:=true;
		else
			to_boolean (param_value, param_value_boolean, Ok);
			if not ok then
		 	Raise_Exception
           			(message_set.Invalid_Parameter'Identity,
            			To_String ( lb_parameter(Current_Language) & Lb_Must_be(Current_Language) & " Boolean"  ));
			end if;
		end if;
       when  Integer_Parameter =>
		if param_value = "" then
			param_value_integer:=0;
		else
			to_integer (param_value, param_value_integer, Ok);
			if not ok then
		 	Raise_Exception
           			(message_set.Invalid_Parameter'Identity,
            			To_String ( lb_parameter(Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
			end if;
		end if;
       when  Double_Parameter =>
		if param_value = "" then
			param_value_double:=0.0;
		else
			to_double (param_value, param_value_double, Ok);
			if not ok then
		 	Raise_Exception
           			(message_set.Invalid_Parameter'Identity,
            			To_String ( lb_parameter(Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
			end if;
		end if;
	when  String_Parameter =>
				null;
		end case;

	Append(Gtk_List_Store(liststore), tree_iter);
	Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, to_string(param_name));
	Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, 
		to_string(to_label(new t_all_enumeration_type'(Parameters_enum, param_type))));
	case param_type is
	  when  Boolean_Parameter =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, param_value_boolean'img);
          when  Integer_Parameter =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, param_value_integer'img);
          when  Double_Parameter =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, param_value_double'img);
	  when  String_Parameter =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, to_string(param_value));
	end case;



exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  

end Add_view_parameters;


procedure Delete_view_parameters (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	path: Gtk_Tree_Path;
	focus: Gtk_Tree_View_Column;
begin
	Get_Cursor(Gtk_Tree_View(Get_Object(Object, "treeview2")), path, focus);
	if path /= null and focus /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(Get_Object(Object, "liststore4")), path);
		Gtk.List_Store.Remove(Gtk_List_Store(Get_Object(Object, "liststore4")), tree_iter);
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer8")), "");
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer9")), "");
	end if;

exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  

end Delete_view_parameters;



package Message_callbacks is new generic_callbacks(Number_of_widget => Message_table_length, Initialize => initialize_Message, 
	Check => Check_Message, Add_In_Sys => Add_In_Sys_Message, Deleted_In_Sys => Deleted_In_Sys_Message, 
	Modify_In_Sys => Modify_In_Sys_Message, Show_In_Sys => Show_In_Sys_Message);  use Message_callbacks;



procedure Show_Message_Widget is
 
   Builder : Gtkada_Builder;
   Error   : Glib.Error.GError;

	tree_iter: Gtk_Tree_Iter;
	liststore: GObject; 
	A_Message : Generic_Message_Ptr;
	My_Iterator : Messages_Iterator;

begin
   Gtk.Main.Init;
                         
   Gtk_New (Builder);
   Error := Add_From_File (Builder, to_string(Glade_path) & "message.glade");
   if Error /= null then
      Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
      Error_Free (Error);
      return;
   end if;


        initialize(sys_buffer);
        duplicate(sys, Sys_buffer);


	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox1")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox2")), 0);

	liststore:= Get_Object(Builder, "liststore2");
	reset_iterator (sys.Messages, My_Iterator);
	if not is_empty (sys.Messages) then
		loop
		current_element (sys.Messages, A_Message, My_Iterator);
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (A_Message.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, 
		To_String(to_label(new t_all_enumeration_type'(Messages_enum, A_Message.Message_Type))));

            case A_Message.message_type is
               when Aperiodic_Type =>
                  Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 7, Gint(0));
                  Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 6, Gint(0));
               when Periodic_Type =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 7, Gint(Periodic_Message_Ptr (A_Message).jitter));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 6, Gint(Periodic_Message_Ptr (A_Message).period));

               when Generic_Type =>
                  Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 7, Gint(0));
                  Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 6, Gint(0));
            end case;

		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, Gint(A_Message.deadline));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 3, Gint(A_Message.size));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 4, Gint(A_Message.response_time));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 5, Gint(A_Message.communication_time));

		exit when is_last_element (sys.Messages, My_Iterator);
	next_element (sys.Messages, My_Iterator);
	         end loop;
      end if;


	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview1_cursor_changed", 
	Handler      => Message_callbacks.Show_Lign_Selected'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button6_clicked", 
	Handler      => Message_callbacks.Add_View'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button5_clicked", 
	Handler      => Message_callbacks.Modify_Lign'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button4_clicked", 
	Handler      => Message_callbacks.Delected_Lign_Selected'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => Message_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => Message_callbacks.Close_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button8_clicked", 
	Handler      => Add_View_parameters'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button7_clicked", 
	Handler      => delete_View_parameters'Access);

	Do_Connect (Builder);

	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));

	Gtk.Main.Main;

	Unref (Builder);
 
end Show_Message_Widget;

end graphical_editor.messages;
