------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Exceptions;                                    use Ada.Exceptions;
with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog; use Gtk.About_Dialog;
with Gtk.Label;        use Gtk.Label;
with Glib.Object; use Glib.Object;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Gtk.Combo_Box; use Gtk.Combo_Box;
with Gtk.Text_Iter;    use Gtk.Text_Iter;
with Gtk.List_Store;   use Gtk.List_Store;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Gtk.Tree_View;  use Gtk.Tree_View;
with Gtk.Tree_View_Column;  use Gtk.Tree_View_Column;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with Glib.Values; use Glib.Values;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Processors;                                           use Processors;
with Processor_set;                                           use Processor_set;
with unbounded_strings;                                    use unbounded_strings;
with core_units; use core_units;
use core_units.Core_Units_Table_Package;
with Framework_Config;                                     use Framework_Config;
with Buffer_Set;                                           use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;                                         use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with graphical_editor.user_message;    use graphical_editor.user_message;
with GNAT.Current_Exception;                               use GNAT.Current_Exception;
with Translate;                                            use Translate;
with Networks;                                             use Networks;
with Task_Set;                                             use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduler;                                            use Scheduler;
with Scheduler.user_defined.interpreted.pipeline;  use  Scheduler.user_defined.interpreted.pipeline;
with Scheduler.user_defined.interpreted.Automata;  use  Scheduler.user_defined.interpreted.Automata;
with Scheduler_Interface;                                  use Scheduler_Interface;
with Network_Set;                                         use Network_Set;
with graphical_editor.convert_text; use graphical_editor.convert_text;

package body graphical_editor.processors is

PACKAGE Es_Entiers IS NEW Ada.Text_IO.Integer_IO(Integer); use Es_Entiers;   


processor_table_length : constant natural :=9;


procedure Initialize_Processor(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	ob1: GObject:= Get_Object(Object, "textbuffer1");
	ob2: GObject:= Get_Object(Object, "textbuffer3");
	ob3: GObject:= Get_Object(Object, "combobox1");
	ob4: GObject:= Get_Object(Object, "combobox2");
	ob5: GObject:= Get_Object(Object, "liststore4");
	ob6: GObject:= Get_Object(Object, "treeview2");
	ob7: GObject:= Get_Object(Object, "liststore3");
	ob8: GObject:= Get_Object(Object, "treeview1");
	wid1: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob1));
	wid2: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob2));
	wid3: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob3));
	wid4: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob4));
	wid5: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob5));
	wid6: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob6));
	wid7: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob7));
	wid8: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob8));
	wid9: t_widget := (ptype => my_window, window_value => processor_window);

begin

	my_tab(1..processor_table_length) := (new t_widget'(wid1), new t_widget'(wid2), new t_widget'(wid3), new t_widget'(wid4), 
	new t_widget'(wid5), new t_widget'(wid6), new t_widget'(wid7), new t_widget'(wid8), new t_widget'(wid9));

end initialize_Processor;


-- Data retrieved from the widgets
--
processor_name : Unbounded_String;
processor_type : processors_type;
processor_migration: migrations_Type;
processor_network_name: Unbounded_String;
processor_Core : Core_Unit_Ptr;
processor_Cores: Core_Units_Table; 
processor_number_cores : integer;


procedure Check_Processor(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

	core_name_value: GValue;
	migration_string, processor_string: Unbounded_String;
	a_Core : Core_Unit_Ptr;
	tree_iter_inter: Gtk_Tree_Iter;

begin
	processor_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));

	processor_network_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(2).buffer_value));

	processor_type := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(3).combo_value)))).processors_value;

	processor_migration := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(4).combo_value)))).migrations_value;

	processor_number_cores := get_number_elements(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(6).view_value)));

	if processor_number_cores = 0 then 
		Check_Processor (Sys.Processors, processor_name, processor_network_name, processor_Core );
	else
	   if processor_number_cores = 1 then 
		tree_iter_inter := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(6).view_value)));
		Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(6).view_value)), 
		tree_iter_inter, 0, core_name_value);
		processor_Core := Search_core_unit(sys.Core_units, to_Unbounded_String(Get_String(core_name_value)));

		if to_label(new t_all_enumeration_type'(Processors_enum, processor_type)) /= to_unbounded_string("Monocore Type") then 
			  Raise_Exception
           			(processor_set.Invalid_Parameter'Identity,
            			To_String (lb_create_only_one_core_unit (Current_Language) ));
		end if;

		if to_label(new t_all_enumeration_type'(Migrations, processor_migration)) /= to_unbounded_string("No Migration Type") then 
			  Raise_Exception
           			(processor_set.Invalid_Parameter'Identity,
            			To_String (lb_create_only_one_core_unit(Current_Language) ));
		end if;

		Check_Processor (Sys.Processors, processor_name, processor_network_name, processor_Core );

	else
		tree_iter_inter := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(6).view_value)));
		initialize(processor_cores);
		for i in 1..processor_number_cores
		loop
			Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(6).view_value)), 
			tree_iter_inter, 0, core_name_value);
			A_Core := Search_core_unit(sys.Core_units, to_Unbounded_String(Get_String(core_name_value)));
			Core_Units_Table_Package.Add(processor_Cores, A_Core);

			unset(core_name_value);	

			next(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(6).view_value)), tree_iter_inter);
		end loop;

		if to_label(new t_all_enumeration_type'(Migrations, processor_migration)) = to_unbounded_string("No Migration Type") then 
			  Raise_Exception
           			(processor_set.Invalid_Parameter'Identity,
            			To_String (lb_Migration_Type_Not_Allow_With_Several_Cores(Current_Language) ));
		end if;

		if to_label(new t_all_enumeration_type'(Processors_enum, processor_type)) = to_unbounded_string("Monocore Type") then 
			  Raise_Exception
           			(processor_set.Invalid_Parameter'Identity,
            			To_String (lb_Processor_Type_Not_Allow_With_Several_Cores(Current_Language) ));
		end if;

		Check_processor (Sys.Processors, processor_name, processor_network_name, processor_Cores, processor_migration, processor_type);
	   end if;
	end if;

end Check_Processor;


procedure Add_In_Sys_Processor(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

begin

	if processor_number_cores = 1 then 
		Add_Processor (Sys.Processors, processor_name, processor_network_name, processor_Core );
	else
		Add_processor (Sys.Processors, processor_name, processor_network_name, processor_Cores, processor_migration, processor_type);
	end if;

end Add_In_Sys_Processor;


procedure Deleted_In_Sys_Processor(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

	A_processor : Generic_Processor_Ptr;
	name : unbounded_string;
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	A_processor := Search_processor (Sys.processors, name);

	Check_Entity_Referencing_a_processor(sys, a_processor);

	delete (Sys.processors, A_processor);

end Deleted_In_Sys_Processor;


procedure Show_In_Sys_Processor(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	A_processor : Generic_Processor_Ptr;
	name : unbounded_string;
	The_Cores : Core_Units_Table;
	a_Core   : Core_Unit_Ptr;
	tree_iter: Gtk_Tree_Iter;
	liststore: GObject; 

begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	A_processor := Search_processor (Sys.processors, name);
	The_Cores := build_core_table (A_processor);
	
	liststore:= Get_Object(Object, "liststore4");

	Gtk.List_Store.Clear(Gtk_List_Store(liststore));
	
	for i in 0..The_Cores.Nb_Entries -1
	loop
		a_core := The_cores.Entries(i);
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (a_core.name));
	end loop;
	
end Show_In_Sys_Processor;


procedure Modify_In_Sys_Processor(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

	A_processor : Generic_Processor_Ptr;
	A_processor_Buffer : Generic_Processor_Ptr;
	id : unbounded_string;	
begin
	A_processor := Search_processor (Sys.processors, processor_name);
	id:=a_processor.cheddar_private_id;
	delete(sys.processors, a_processor);

	if processor_number_cores = 1 then 
		Add_Processor (Sys.Processors, a_processor_buffer, processor_name, processor_network_name, processor_Core );
	else
		Add_processor (Sys.Processors, a_processor_buffer, processor_name, processor_network_name, processor_Cores, processor_migration, processor_type);
	end if;
	a_processor_buffer.cheddar_private_id:=id;


end Modify_In_Sys_Processor;


package processor_callbacks is new generic_callbacks(Number_of_widget => processor_table_length, 
	Initialize => initialize_Processor, Check => Check_Processor, Add_In_Sys => Add_In_Sys_Processor, 
	Deleted_In_Sys => Deleted_In_Sys_Processor, Modify_In_Sys => Modify_In_Sys_Processor, 
	Show_In_Sys => Show_In_Sys_Processor);  use processor_callbacks;

procedure Add_View_Core (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	core_name: Unbounded_String;
	core_name_value : GValue;
	number_rows : integer;
	ok: boolean := false;
	i: integer:= 0;
	tree_iter_inter: Gtk_Tree_Iter;
	liststore : GObject;
begin

	liststore:= Get_Object(Object, "liststore4");
	core_name := to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox3"))));
	if (core_name = "") then
		Show_Message_Box (Lb_Define_core_units_Before (Current_Language));
		return;
        end if;
	number_rows:= get_number_elements(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(Gtk_Tree_view(Get_Object(Object, "treeview2")))));
	if number_rows = 0 then 
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, 
		Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox3"))));

	else
	
	tree_iter_inter := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(Gtk_Tree_view(Get_Object(Object, "treeview2")))));
	while i <= number_rows and ok = false and tree_iter_inter /= Null_Iter loop
		
		Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(Gtk_Tree_view(Get_Object(Object, "treeview2")))), 
		tree_iter_inter, 0, core_name_value);

		if core_name = to_Unbounded_String(Get_String(core_name_value)) then 
			ok := true;
			exit;
		end if;

		Glib.Values.unset(core_name_value);	

		next(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(Gtk_Tree_view(Get_Object(Object, "treeview2")))), tree_iter_inter);
		i := i+1;
	end loop;
	
	if ok = false then 
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, 
		Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox3"))));
	end if;
	end if;

exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Add_View_Core;

procedure Delete_View_Core (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	path: Gtk_Tree_Path;
	focus: Gtk_Tree_View_Column;
begin
	Get_Cursor(Gtk_Tree_View(Get_Object(Object, "treeview2")), path, focus);
	if path /= null and focus /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(Get_Object(Object, "liststore4")), path);
		Gtk.List_Store.Remove(Gtk_List_Store(Get_Object(Object, "liststore4")), tree_iter);
	end if;
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Delete_View_Core;

procedure Show_Lign_Core (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	path1: Gtk_Tree_Path;
	focus1: Gtk_Tree_View_Column;
	combo_text: unbounded_string;
begin
	Get_Cursor(Gtk_Tree_View(get_object(object, "treeview2")), path1, focus1);
	if path1 /= null and focus1 /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(get_object(object, "liststore4")), path1);
		combo_text := to_unbounded_string(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(get_object(object, "liststore4")), 
		tree_iter, 0));
		Set_Active(Gtk_Combo_Box(get_object(object, "combobox3")), 
		Gint(get_int_combo(Gtk_Combo_Box(get_object(object, "combobox3")), combo_text)));
	end if;
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Show_Lign_Core;

procedure Show_Processor_Widget is

	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;
	tree_iter: Gtk_Tree_Iter;
	liststore: GObject; 
	A_Processor : Generic_Processor_Ptr;
	My_Iterator : Processors_Iterator;

	A_Core : Core_Unit_Ptr;
	My_Core_Iterator : core_units_Iterator;
	my_combo: GObject;

	My_Network_Iterator : Network_Iterator;
	
begin
	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "processor.glade");
	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;


        initialize(Sys_buffer);
        duplicate(sys, Sys_buffer);


	my_combo := Get_Object(Builder, "combobox3");
	reset_iterator (sys.Core_units, My_Core_Iterator);
	if not is_empty (sys.Core_units) then 
	loop
	current_element (sys.Core_units, A_Core, My_Core_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo), To_String(A_Core.name));
	exit when is_last_element (sys.Core_units, My_Core_Iterator);
		next_element (sys.Core_units, My_Core_Iterator);
	end loop;
	else
	Show_Message_Box
	(Lb_Define_core_units_Before (Current_Language));
	end if;

	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox1")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox2")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox3")), 0);

	liststore:= Get_Object(Builder, "liststore3");
	reset_iterator (sys.Processors, My_Iterator);
	if not is_empty (sys.Processors) then
		loop
		current_element (sys.Processors, A_Processor, My_Iterator);
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (A_Processor.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, To_String (A_Processor.network_name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, 
		To_String(to_label(new t_all_enumeration_type'(Processors_enum, A_Processor.processor_type))));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 3, 
		To_String(to_label(new t_all_enumeration_type'(Migrations, A_Processor.migration_type))));
		exit when is_last_element (sys.Processors, My_Iterator);
		next_element (sys.Processors, My_Iterator);
		end loop;
	end if;

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button6_clicked", 
	Handler      => processor_callbacks.Add_View'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button4_clicked", 
	Handler      => processor_callbacks.Delected_Lign_Selected'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => processor_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview1_cursor_changed", 
	Handler      => processor_callbacks.Show_Lign_Selected'Access);  

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview2_cursor_changed", 
	Handler      => Show_Lign_Core'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button5_clicked", 
	Handler      => processor_callbacks.Modify_Lign'Access);  

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => processor_callbacks.Close_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button8_clicked", 
	Handler      => Add_View_Core'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button7_clicked", 
	Handler      => Delete_View_Core'Access);

	Do_Connect (Builder);
 
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));

	Gtk.Main.Main;

	Unref (Builder);
 
end Show_Processor_Widget;

end graphical_editor.processors;
