------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with Gtk.Text_Buffer; 	use Gtk.Text_Buffer;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Combo_Box; use Gtk.Combo_Box;
with Call_Framework;           use Call_Framework;
with Call_Framework_Interface; use Call_Framework_Interface;
use Call_Framework_Interface.Framework_Request_Package;
use Call_Framework_Interface.Framework_Response_Package;
with Processors;                     use Processors;
with Processor_Set;                  use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Gtk.Text_Iter; use Gtk.Text_Iter;
with Ada.Text_IO;	use Ada.Text_IO;
with unbounded_strings;                 use unbounded_strings;
with Gtk.Check_Button; use Gtk.Check_Button;
with graphical_editor.user_message;    use graphical_editor.user_message;
with Translate;                      use Translate;
with Framework_Config;               use Framework_Config;
with graphical_editor.message_text; use graphical_editor.message_text;
with Gtk.Main;       use Gtk.Main;

package body graphical_editor.scheduling_feasibility_widget_callbacks is

	line_Number : integer := 0;

procedure Close_Widget(Object : access Gtkada_Builder_Record'Class) is

	my_widget : GObject;
 
begin
	my_widget:=Get_Object(Object, "window1");
	Destroy_cb(Gtk_Widget(my_widget));
	gtk.main.main_quit;

end Close_Widget;

procedure show_scheduling_feasibility(Object : access Gtkada_Builder_Record'Class) is

	Test1       : Boolean := False;
	Test2       : Boolean := False;
	A_Processor : Generic_Processor_Ptr;
	My_Iterator : Processors_Iterator;

	procedure Call_Feasibility (Processor_Name : in Unbounded_String) is

		Response_List : Framework_Response_Table;
		Request_List  : Framework_Request_Table;
		A_Request     : Framework_Request;

	begin
		initialize (Response_List);
		initialize (Request_List);

		if Test1 then
			Initialize (A_Request);
			A_Request.target    := Processor_Name;
			A_Request.statement := Scheduling_Feasibility_Cpu_Utilization;
			add (Request_List, A_Request);
		end if;

		if Test2 then
			Initialize (A_Request);
			A_Request.target    := Processor_Name;
			A_Request.statement := Scheduling_Feasibility_periodic_task_worst_case_Response_Time;
			add (Request_List, A_Request);
		end if;

		Sequential_Framework_Request (Sys, Request_List, Response_List);
		Write_Title(Response_List);
		Write_Text(Response_List);

	end Call_Feasibility;

begin

	if Get_Active (Gtk_Check_Button(Get_Object(Object, "checkbutton2"))) then
        	Test1 := True;
	end if;

	if Get_Active (Gtk_Check_Button(Get_Object(Object, "checkbutton3"))) then
		Test2 := True;
	end if;

	if (Test1 = False) and (Test2 = False) then
		Show_Message_Box (Lb_Select_Almost_One_Feasilibity_Test (Current_Language));
		return;
	end if;

	A_Processor := Search_Processor (Sys.Processors, 
	to_Unbounded_String(Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox1")))));

	if not Get_Active (Gtk_Check_Button(Get_Object(Object, "checkbutton1"))) then
		Call_Feasibility (A_Processor.name);
	else
		reset_iterator (Sys.Processors, My_Iterator);
		loop
			current_element (Sys.Processors, A_Processor, My_Iterator);
			Call_Feasibility (A_Processor.name);
			exit when is_last_element (Sys.Processors, My_Iterator);
			next_element (Sys.Processors, My_Iterator);
		end loop;
	end if;

	Close_Widget(Object);


end show_scheduling_feasibility;

end graphical_editor.scheduling_feasibility_widget_callbacks;
