------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtkada.Builder; use Gtkada.Builder;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Ada.Text_IO; use Ada.Text_IO;
with Gtk.Widget;     use Gtk.Widget;
with Processors;   use Processors;
with Processor_Set; 	use Processor_Set;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with Gtk.Combo_Box; use Gtk.Combo_Box;
with graphical_editor.user_message;    use graphical_editor.user_message;
with Framework_Config;                                     use Framework_Config;
with Translate;                                            use Translate;
with graphical_editor.scheduling_feasibility_widget_callbacks; use graphical_editor.scheduling_feasibility_widget_callbacks;

package body graphical_editor.scheduling_feasibility_widgets is

procedure Show_scheduling_feasibility_Widget is
 
	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;

	A_Processor : Generic_Processor_Ptr;
	My_Processor_Iterator : Processors_Iterator;
	my_combo_cpu: GObject;

begin
	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "scheduling_feasibility.glade");
	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;

	my_combo_cpu := Get_Object(Builder, "combobox1");
	reset_iterator (sys.Processors, My_Processor_Iterator);
	if not is_empty (sys.Processors) then 
	loop
	current_element (sys.Processors, A_Processor, My_Processor_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_cpu), To_String(A_Processor.name));
	exit when is_last_element (sys.Processors, My_Processor_Iterator);
		next_element (sys.Processors, My_Processor_Iterator);
	end loop;
	Set_active(Gtk_Combo_Box(my_combo_cpu), 0);

	else
	Show_Message_Box
	(Lb_Define_processors_Before (Current_Language));
	end if;

	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => graphical_editor.scheduling_feasibility_widget_callbacks.show_scheduling_feasibility'Access);

	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => graphical_editor.scheduling_feasibility_widget_callbacks.close_widget'Access);

	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
	Unref (Builder);
 
end Show_scheduling_feasibility_Widget;

end graphical_editor.scheduling_feasibility_widgets;
