with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog; use Gtk.About_Dialog;
with Gtk.List_Store;   use Gtk.List_Store;
with Gtk.Combo_Box; use Gtk.Combo_Box;
with Gtk.Label;        use Gtk.Label;
with Ada.Text_IO;      use Ada.Text_IO;
with Gtk.Check_Button; use Gtk.Check_Button;
with Gtkada.Builder; use Gtkada.Builder;
with graphical_editor.scheduling_option_callbacks; use graphical_editor.scheduling_option_callbacks;
with graphical_editor.generic_callbacks;
with graphical_editor.generic_package_widget; use graphical_editor.generic_package_widget;
with graphical_editor.processors; use graphical_editor.processors;
with Processors;   use Processors;
with Processor_Set; 	use Processor_Set;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with graphical_editor.user_message;    use graphical_editor.user_message;
with Framework_Config;                                     use Framework_Config;
with Translate;                                            use Translate;
with time_unit_events; use time_unit_events;
with Editor_Config;             use Editor_Config;

package body graphical_editor.scheduling_options is

PACKAGE Es_Entiers IS NEW Ada.Text_IO.Integer_IO(Integer); use Es_Entiers;




procedure show_scheduling_options_widget is
 
	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;


begin
	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "scheduling_options.glade");
	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;



	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => graphical_editor.scheduling_option_callbacks.ok'Access);

	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => graphical_editor.scheduling_option_callbacks.cancel'Access);

	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button3_clicked", 
	Handler      => graphical_editor.scheduling_option_callbacks.seed'Access);

	Do_Connect (Builder);


	Event_Check_Button(end_of_task_capacity) := Gtk_Check_Button(Get_Object(Builder, "checkbutton5"));
	Event_Check_Button(start_of_task_capacity) := Gtk_Check_Button(Get_Object(Builder, "checkbutton4"));
	Event_Check_Button(read_from_buffer) := Gtk_Check_Button(Get_Object(Builder, "checkbutton6"));
	Event_Check_Button(write_to_buffer) := Gtk_Check_Button(Get_Object(Builder, "checkbutton10"));
	Event_Check_Button(receive_message) := Gtk_Check_Button(Get_Object(Builder, "checkbutton7"));
	Event_Check_Button(send_message) := Gtk_Check_Button(Get_Object(Builder, "checkbutton11"));
	Event_Check_Button(release_resource) := Gtk_Check_Button(Get_Object(Builder, "checkbutton8"));
	Event_Check_Button(allocate_resource) := Gtk_Check_Button(Get_Object(Builder, "checkbutton12"));
	Event_Check_Button(context_switch_overhead) := Gtk_Check_Button(Get_Object(Builder, "checkbutton9"));
	Event_Check_Button(wait_for_resource) := Gtk_Check_Button(Get_Object(Builder, "checkbutton13"));
	Event_Check_Button(preemption) := Gtk_Check_Button(Get_Object(Builder, "checkbutton28"));

      for I in Time_Unit_Event_Type'Range loop

         if (I /= Task_Activation) and (I /= Running_Task) and (I /= Address_Space_Activation) and (I /= Wait_For_Memory) then
            if Generate_Events (I) then
               Set_Active (Event_Check_Button (I), True);
            else
               Set_Active (Event_Check_Button (I), False);
            end if;
	end if;
	end loop;


if Schedule_With_Offsets 
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton1")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton1")), false);
end if;

if Schedule_With_resources
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton3")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton3")), false);
end if;

if Schedule_With_Precedencies
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton2")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton2")), false);
end if;

if Has_Global_Seed
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton15")), false);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton15")), true);
end if;

if Is_Global_Predictable_Seed
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton16")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton16")), false);
end if;

if  draw_core_unit_time_line 
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton14")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton14")), false);
end if;

if draw_address_space_time_line 
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton17")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton17")), false);
end if;

      Set_Text (Gtk_Text_buffer(Get_Object(Builder, "textbuffer1")), Global_Seed_Value'Img);
      Set_Text (Gtk_Text_buffer(Get_Object(Builder, "textbuffer3")), Max_Time_Line_To_Display'Img);
      Set_Text (Gtk_Text_buffer(Get_Object(Builder, "textbuffer2")), Time_Line_Between_Time_Axis'Img);


if several_colors_for_time_lines
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton18")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton18")), false);
end if;

if display_wait_for_resource
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton19")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton19")), false);
end if;

if display_task_activation 
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton20")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton20")), false);
end if;

if display_allocate_resource
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton21")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton21")), false);
end if;

if display_release_resource
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton24")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton24")), false);
end if;

if display_address_space_completion
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton23")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton23")), false);
end if;

if display_address_space_activation
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton22")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton22")), false);
end if;

if schedule_with_jitters
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton25")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton25")), false);
end if;

if minimize_preemption
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton26")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton26")), false);
end if;

if schedule_with_crpd 
	then Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton27")), true);
	else Set_Active (Gtk_Check_Button(Get_Object(Builder, "checkbutton27")), false);
end if;



	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
	Unref (Builder);
 
end show_scheduling_options_widget;

end graphical_editor.scheduling_options;
