------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Object;     use Gtk.Object;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.Label;        use Gtk.Label;
with Gtkada.Builder; use Gtkada.Builder;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with Glib.Convert; use Glib.Convert;

with translate; use translate;
with framework_config; use framework_config;

with graphical_editor.scheduling_ref_callbacks; use graphical_editor.scheduling_ref_callbacks;
with graphical_editor.user_message;    use graphical_editor.user_message;

with unbounded_strings;     use unbounded_strings;
with Ada.Strings.Unbounded;         use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Text_IO; use Ada.Strings.Unbounded.Text_IO;
with io_tools; use io_tools;

with ada.io_exceptions; use ada.io_exceptions;
with GNAT.Current_Exception;           use GNAT.Current_Exception;
with Ada.Text_IO;



package body graphical_editor.scheduling_refs is

procedure Show_Sched_Reference_Widget is

	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;
	Buffer_text : GObject;
      file_content : unbounded_string;

begin

	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "scheduling_ref.glade");
	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;

	Buffer_text:=Get_Object(Builder, "textbuffer1");



      begin
	file_content:=read_sequential_file(to_string(Glade_path) & "ref_bib.txt");
      exception
         when Ada.IO_Exceptions.Name_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", ref bib text file, Name_Error"));
         when Ada.IO_Exceptions.Status_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", ref bib text file, Status_Error"));
         when Ada.IO_Exceptions.Mode_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", ref bib text file, Mode_Error"));
         when Ada.IO_Exceptions.Use_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", ref bib text file, Use_Error"));
         when Ada.IO_Exceptions.Device_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", ref bib text file, Device_Error"));
         when Ada.IO_Exceptions.End_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", ref bib text file, End_Error"));
         when Ada.IO_Exceptions.Data_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (" ref bib text file, Data_Error"));
         when Ada.IO_Exceptions.Layout_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", ref bib text file, Layout_Error"));
         when others =>
            Show_Message_Box
              (Lb_Can_Not_Read_File (Current_Language) &
               ", ref bib text file ; " &
               Exception_Name &
               " ; " &
               Exception_Message);
      end;


	Gtk.Text_Buffer.Set_Text(Gtk_Text_Buffer(Buffer_text), to_string(file_content));


	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => graphical_editor.scheduling_ref_callbacks.Close_Sched_Reference_Widget'Access);

	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;

	Unref (Builder);
 
end Show_Sched_Reference_Widget;

end graphical_editor.scheduling_refs;
