------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtkada.Builder; use Gtkada.Builder;
with Glib;                              use Glib;
with Gdk;                               use Gdk;
with Gdk.Event;                         use Gdk.Event;
with Gdk.GC;                            use Gdk.GC;
with Gdk.Color;                         use Gdk.Color;
with Gtk;                               use Gtk;
with Gtk.Widget;                        use Gtk.Widget;
with Buffer_Set;                        use Buffer_Set;
with Task_Set;                          use Task_Set;
with Resource_Set;                      use Resource_Set;
with Message_Set;                       use Message_Set;
with processor_set;			use processor_set;
with Multiprocessor_Services_Interface; use Multiprocessor_Services_Interface;
use Multiprocessor_Services_Interface.Scheduling_Result_Per_Processor_Package;
with Gtk.Drawing_Area; use Gtk.Drawing_Area;

package graphical_editor.scheduling_simulation_draw_callbacks is

      package Cb is new
                Gtk.Handlers.Return_Callback(Gtk.Drawing_Area.Gtk_Drawing_Area_Record, boolean);  use Cb;


   type Draw_Scheduling_Record is tagged record
      Font   : Gdk_Font;

      Tasks      : Tasks_Set;
      Resources  : Resources_Set;
      Messages   : Messages_Set;
      Buffers    : Buffers_Set;
      processors    : processors_Set;
      Period     : Natural;
      Start_Draw : Natural;
      Sched      : Scheduling_Table_Ptr;

      White_Gc   : Gdk.GC.Gdk_GC;
      Black_Gc   : Gdk.GC.Gdk_GC;
      Red_Color  : Gdk_Color;
      Blue_Color : Gdk_Color;

      Left_Margin     : Gint := 20;
      Right_Margin    : Gint := 500;
      High_Margin     : Gint := 20;
      Interligne      : Gint := 50;
      Timeunit_Margin : Gint := 20;

      Current_Width  : Gint;
      Current_Height : Gint;

      Have_Colors : Boolean := False;
   end record;

	type Draw_Scheduling_Access is access all Draw_Scheduling_Record'Class;

	Draw_Scheduling : Draw_Scheduling_Access := new Draw_Scheduling_Record;

	function Expose (W : access Gtk.Drawing_Area.Gtk_Drawing_Area_Record'Class) return boolean;

	procedure Read_Scheduling_Duration_And_Draw_Scheduling(Object : access Gtkada_Builder_Record'Class);

	procedure Close_Widget(Object : access Gtkada_Builder_Record'Class);

	procedure Call_Customized_Simulation;

	procedure Call_UnCustomized_Simulation;

end graphical_editor.scheduling_simulation_draw_callbacks;
