------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item;	use Gtk.Menu_Item;
with Gtk;	use Gtk;
with Gtk.Main;	use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog;	use Gtk.About_Dialog;
with Gtk.Label;        use Gtk.Label;
with Glib.Object;	use Glib.Object;
with Gtk.Text_Buffer;	use Gtk.Text_Buffer;
with Gtk.Combo_Box;	use Gtk.Combo_Box;
with Gtk.Text_Iter;	use Gtk.Text_Iter;
with Gtk.List_Store;	use Gtk.List_Store;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Gtk.Tree_View;  use Gtk.Tree_View;
with Gtk.Tree_View_Column;  use Gtk.Tree_View_Column;
with gdk.event; use gdk.event;
with Gtkada.Builder; use Gtkada.Builder;
with Glib;      use Glib;
with Glib.Object;	use Glib.Object;
with Glib.Values;	use Glib.Values;
with graphical_editor.user_message;    use graphical_editor.user_message;
with graphical_editor.convert_text; use graphical_editor.convert_text;
with Ada.Text_IO;	use Ada.Text_IO;
with Ada.Exceptions;                                    use Ada.Exceptions;
with Ada.Strings.Unbounded;	use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
use unbounded_strings.strings_table_package;
with Framework_Config;                                     use Framework_Config;
with task_groups;	use task_groups;
with task_group_Set;                                           use task_group_Set;
use task_group_Set.Generic_task_group_Set;
with GNAT.Current_Exception;                               use GNAT.Current_Exception;
with Translate;                                            use Translate;
with Task_Set; 			use Task_Set;                                       
use Task_Set.Generic_Task_Set;
with Tasks;                                             use Tasks;
use tasks.Generic_Task_List_Package;
with Ada.Numerics.Float_Random; use Ada.Numerics.Float_Random;
with Ada.Numerics.Aux;          use Ada.Numerics.Aux;


package body graphical_editor.task_groups is


Task_group_table_length : constant natural := 13;


procedure Initialize_Task_group(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	ob1: GObject:= Get_Object(Object, "textbuffer1");
	ob2: GObject:= Get_Object(Object, "combobox3");
	ob3: GObject:= Get_Object(Object, "textbuffer2");
	ob4: GObject:= Get_Object(Object, "textbuffer3");
	ob5: GObject:= Get_Object(Object, "textbuffer4");
	ob6: GObject:= Get_Object(Object, "textbuffer5");
	ob7: GObject:= Get_Object(Object, "textbuffer6");
	ob8: GObject:= Get_Object(Object, "textbuffer7");
	ob9: GObject:= Get_Object(Object, "liststore2");
	ob10: GObject:= Get_Object(Object, "treeview2");
	ob11: GObject:= Get_Object(Object, "liststore1");
	ob12: GObject:= Get_Object(Object, "treeview1");

	wid1: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob1));
	wid2: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob2));
	wid3: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob3));
	wid4: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob4));
	wid5: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob5));
	wid6: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob6));
	wid7: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob7));
	wid8: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob8));
	wid9: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob9));
	wid10: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob10));
	wid11: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob11));
	wid12: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob12));
	wid13: t_widget := (ptype => my_window, window_value => Task_groups_Window);

begin
	my_tab(1..task_group_table_length) := (new t_widget'(wid1), new t_widget'(wid2), new t_widget'(wid3), new t_widget'(wid4), new t_widget'(wid5), new t_widget'(wid6), new t_widget'(wid7), new t_widget'(wid8), new t_widget'(wid9), new t_widget'(wid10), new t_widget'(wid11), new t_widget'(wid12), new t_widget'(wid13) );
end initialize_Task_group;


-- Data retrieved from the widgets
--
task_group_name : Unbounded_String;
task_group_Start_Time, task_group_Period, task_group_Deadline, task_group_Jitter, task_group_Priority, task_group_Criticality : Integer;
task_group_type : task_groups_type;
task_group_task_list : Generic_Task_List;
task_group_number_of_task : integer;



procedure Check_Task_group(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is  

	deadline_string, start_time_string, priority_string, period_string, jitter_string, criticality_string : unbounded_string;
	integer_ok : boolean;
       tree_iter_task: Gtk_Tree_Iter;
      object_name_value : GValue;


begin
	task_group_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
        task_group_type := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(2).combo_value)))).task_group_types_value;

        deadline_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(3).buffer_value)));
        if (deadline_String = "") then
                task_group_deadline := 0;
        else
                to_integer (deadline_String, task_group_deadline, Integer_Ok);
                if not Integer_Ok then
          Raise_Exception
           (task_group_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_deadline (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

        start_time_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(4).buffer_value)));
        if (start_time_String = "") then
                task_group_Start_Time := 0;
        else
                to_integer (Start_Time_String, task_group_Start_Time, Integer_Ok);
                if not Integer_Ok then
          Raise_Exception
           (task_group_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_Start_Time (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

        priority_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(5).buffer_value)));
        if (priority_String = "") then
                task_group_priority:= 0;
        else
                to_integer (priority_String, task_group_priority, Integer_Ok);
                if not Integer_Ok then
          Raise_Exception
           (task_group_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_priority(Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

        period_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(6).buffer_value)));
        if (period_String = "") then
                task_group_period:= 0;
        else
                to_integer (period_String, task_group_period, Integer_Ok);
                if not Integer_Ok then
          Raise_Exception
           (task_group_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_period(Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

        jitter_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(7).buffer_value)));
        if (jitter_String = "") then
                task_group_jitter := 0;
        else
                to_integer (jitter_string, task_group_jitter, Integer_Ok);
                if not Integer_Ok then
          Raise_Exception
           (task_group_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_jitter (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

        criticality_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(8).buffer_value)));
        if (criticality_String = "") then
                task_group_criticality:= 0;
        else
                to_integer (criticality_String, task_group_criticality, Integer_Ok);
                if not Integer_Ok then
          Raise_Exception
           (task_group_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_criticality (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
                end if;
        end if;

	initialize(task_group_task_list);
	task_group_number_of_task:=get_number_elements(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)));
        tree_iter_task := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)));
        for i in 1 .. task_group_number_of_task loop
                Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)),
                tree_iter_task, 0, object_name_value);
                add (task_group_task_list, search_task(sys.tasks,to_unbounded_string(Get_String(object_name_value))));
                next(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(10).view_value)), tree_iter_task);
        end loop;


end Check_Task_group;



procedure Add_In_Sys_Task_group(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

a_task_group : generic_task_group_ptr;
Task_Iterator : Generic_Task_Iterator;
A_Task        : Generic_Task_Ptr;

begin
        Add_Task_Group(sys.task_groups, a_task_group, task_group_name, task_group_type, task_group_Start_Time, task_group_period, task_group_Deadline, task_group_jitter, task_group_priority, task_group_criticality);

         if not is_empty (task_group_task_list) then
            reset_head_iterator (task_group_task_list, Task_Iterator);

            loop
               current_element (task_group_task_list, A_Task, Task_Iterator);

               Add_Task_To_Group(a_task_group, A_Task);

               exit when is_tail_element (task_group_task_list, Task_Iterator);
               next_element (task_group_task_list, Task_Iterator);
            end loop;
         end if;


end Add_In_Sys_Task_group;



procedure Deleted_In_Sys_Task_group(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	A_Task_group      : Generic_Task_group_Ptr;
	name: unbounded_string;
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	A_Task_group := Search_Task_group (Sys.Task_groups, name);

	Check_Entity_Referencing_a_task_group(sys, a_task_group);

	Delete (Sys.task_groups, A_Task_group);
end Deleted_In_Sys_Task_group;



procedure Show_In_Sys_Task_group(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

        A_task_group : Generic_task_group_Ptr;
        name : unbounded_string;

        Iterator_Task : Generic_Task_Iterator;
        a_task : generic_task_Ptr;

        tree_iter: Gtk_Tree_Iter;
        liststore: GObject;

begin
        name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
        A_task_group := Search_task_group (Sys.task_groups, name);

        liststore:= Get_Object(Object, "liststore2");

        Gtk.List_Store.Clear(Gtk_List_Store(liststore));

        if not is_empty(a_task_Group.task_list) then
        reset_head_iterator(a_task_Group.task_list, Iterator_Task);
         loop
            current_element (a_task_Group.task_list, a_Task, Iterator_Task);
                Append(Gtk_List_Store(liststore), tree_iter);
                Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (a_task.name));
            exit when is_tail_element(a_task_Group.task_list, Iterator_Task);
            next_element (a_task_Group.task_list, Iterator_Task);
        end loop;
	end if;
end Show_In_Sys_Task_group;



procedure Modify_In_Sys_Task_group(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

        A_task_group : Generic_task_group_Ptr;
        A_task_group_Buffer : Generic_task_group_Ptr;
        id : unbounded_string;
begin
        A_task_group:= Search_task_group (Sys.task_groups, task_group_name);
        id:=a_task_group.cheddar_private_id;
        delete(sys.task_groups, a_task_group);

  	Add_Task_Group(sys.task_groups, a_task_group_buffer, task_group_name, task_group_type, task_group_Start_Time, task_group_period, task_group_Deadline, task_group_jitter, task_group_priority, task_group_criticality);
        a_task_group_buffer.cheddar_private_id:=id;

end Modify_In_Sys_Task_group;



procedure Add_task_list(Object : access Gtkada_Builder_Record'Class) is
       tree_iter: Gtk_Tree_Iter;
begin
        Append(Gtk_List_Store(Get_Object(Object, "liststore2")), tree_iter);
        Gtk.List_Store.Set(Gtk_List_Store(Get_Object(Object, "liststore2")), tree_iter, 0,
        Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox1"))));


exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;
end Add_task_list;


procedure Show_Lign_task_list(Object : access Gtkada_Builder_Record'Class) is
        tree_iter: Gtk_Tree_Iter;
        path1: Gtk_Tree_Path;
        focus1: Gtk_Tree_View_Column;
        combo_text: unbounded_string;
begin
        Get_Cursor(Gtk_Tree_View(get_object(object, "treeview2")), path1, focus1);
        if path1 /= null and focus1 /= null then
                tree_iter:= get_iter(Gtk_Tree_Model(get_object(object, "liststore2")), path1);
                combo_text := to_unbounded_string(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(get_object(object, "liststore2")), tree_iter, 0));
                Set_Active(Gtk_Combo_Box(get_object(object, "combobox1")),
                Gint(get_int_combo(Gtk_Combo_Box(get_object(object, "combobox1")), combo_text)));
        end if;

exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;
end Show_Lign_task_list;

        

procedure delete_task_list(Object : access Gtkada_Builder_Record'Class) is

 tree_iter: Gtk_Tree_Iter;
        path: Gtk_Tree_Path;
        focus: Gtk_Tree_View_Column;
begin
        Get_Cursor(Gtk_Tree_View(Get_Object(Object, "treeview2")), path, focus);
        if path /= null and focus /= null then
                tree_iter:= get_iter(Gtk_Tree_Model(Get_Object(Object, "liststore2")), path);
                Gtk.List_Store.Remove(Gtk_List_Store(Get_Object(Object, "liststore2")), tree_iter);
        end if;

exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;
end delete_task_list;





package Task_group_callbacks is new generic_callbacks(Number_of_widget => Task_group_table_length, Initialize => initialize_Task_group, Check => Check_Task_group, Add_In_Sys => Add_In_Sys_Task_group, Deleted_In_Sys => Deleted_In_Sys_Task_group, 
	Modify_In_Sys => Modify_In_Sys_Task_group, Show_In_Sys => Show_In_Sys_Task_group);  use Task_group_callbacks;



procedure Show_Task_group_Widget is
 
	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;

	tree_iter: Gtk_Tree_Iter;
	liststore: GObject;

	A_Task : Generic_Task_Ptr;
	A_Task_group : Generic_Task_Group_Ptr;
	
	My_Task_Iterator : Tasks_Iterator;
	My_Task_group_Iterator : Task_Groups_Iterator;

	my_combo_task_list : GObject;



begin
	Gtk.Main.Init;
        Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "task_group.glade");

	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;

	initialize(sys_buffer);
        duplicate(sys, Sys_buffer);


	my_combo_task_list := Get_Object(Builder, "combobox1");

	reset_iterator (sys.Tasks, My_Task_Iterator);
	if not is_empty (sys.Tasks) then 
	loop
	current_element (sys.Tasks, A_Task, My_Task_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_task_list), To_String(A_Task.name));

	exit when is_last_element (sys.Tasks, My_Task_Iterator);
	next_element (sys.Tasks, My_Task_Iterator);
	end loop;
	end if;


	liststore:= Get_Object(Builder, "liststore1");
	reset_iterator (sys.task_groups, My_task_group_Iterator);
	if not is_empty (sys.task_groups) then
		loop
		current_element (sys.task_groups, A_task_group, My_task_group_Iterator);

		Append(Gtk_List_Store(liststore), tree_iter);

		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (A_task_group.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, To_String(to_label(new t_all_enumeration_type'(task_group_types_enum, A_task_group.task_group_type))));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, Gint(A_task_group.deadline));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 3, Gint(A_task_group.start_time));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 4, Gint(A_task_group.priority));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 5, Gint(A_task_group.period));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 6, Gint(A_task_group.jitter));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 7, Gint(A_task_group.criticality));

		exit when is_last_element (sys.task_groups, My_task_group_Iterator);
		next_element (sys.task_groups, My_task_group_Iterator);
		end loop;
	end if;


	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox1")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox3")), 0);

      

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview1_cursor_changed", 
	Handler      => Task_group_callbacks.Show_Lign_Selected'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => Task_group_callbacks.Close_View'Access);  

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => Task_group_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button3_clicked", 
	Handler      => Task_group_callbacks.Delected_Lign_Selected'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button4_clicked", 
	Handler      => Task_group_callbacks.Modify_Lign'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button5_clicked", 
	Handler      => Task_group_callbacks.Add_View'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button7_clicked", 
	Handler      => Add_task_list'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button8_clicked", 
	Handler      => Delete_task_list'Access); 

  Gtkada.Builder.Register_Handler
        (Builder      => Builder,
        Handler_Name => "on_treeview2_cursor_changed",
        Handler      => show_lign_task_list'Access);

	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
	Unref (Builder);

end Show_Task_group_Widget;




end graphical_editor.task_groups;


