------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item;	use Gtk.Menu_Item;
with Gtk;	use Gtk;
with Gtk.Main;	use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog;	use Gtk.About_Dialog;
with Gtk.Label;        use Gtk.Label;
with Glib.Object;	use Glib.Object;
with Gtk.Text_Buffer;	use Gtk.Text_Buffer;
with Gtk.Combo_Box;	use Gtk.Combo_Box;
with Gtk.Text_Iter;	use Gtk.Text_Iter;
with Gtk.List_Store;	use Gtk.List_Store;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Gtk.Tree_View;  use Gtk.Tree_View;
with Gtk.Tree_View_Column;  use Gtk.Tree_View_Column;
with Glib;      use Glib;
with Glib.Object;	use Glib.Object;
with Glib.Values;	use Glib.Values;
with Ada.Text_IO;	use Ada.Text_IO;
with Ada.Exceptions;                                    use Ada.Exceptions;
with Ada.Strings.Unbounded;	use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Processors;	use Processors;
with address_spaces;	use address_spaces;
with unbounded_strings;		use unbounded_strings;
use unbounded_strings.strings_table_package;
with Framework_Config;                                     use Framework_Config;
with Buffer_Set;                                           use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;                                         use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with graphical_editor.user_message;    use graphical_editor.user_message;
with GNAT.Current_Exception;                               use GNAT.Current_Exception;
with Translate;                                            use Translate;
with Networks;  			use Networks;
with Task_Set; 			use Task_Set;                                       
use Task_Set.Generic_Task_Set;
with Scheduler;                                            use Scheduler;
with Scheduler.user_defined.interpreted.pipeline;  use  Scheduler.user_defined.interpreted.pipeline;
with Scheduler.user_defined.interpreted.Automata;  use  Scheduler.user_defined.interpreted.Automata;
with Scheduler_Interface;                                  use Scheduler_Interface;
with Processor_Set; 					use Processor_Set;
with Address_Space_Set; 			use Address_Space_Set;
with Tasks;                                             use Tasks;
with Gtkada.Builder; use Gtkada.Builder;
with Glib;      use Glib;
with gdk.event; use gdk.event;
with offsets; use offsets;
use offsets.Offsets_Table_Package;
with offsets.extended; use offsets.extended;
with parameters; use parameters;
use parameters.User_Defined_Parameters_Table_Package;
with parameters.extended; use parameters.extended;
with Ada.Numerics.Float_Random; use Ada.Numerics.Float_Random;
with Ada.Numerics.Aux;          use Ada.Numerics.Aux;
with graphical_editor.convert_text; use graphical_editor.convert_text;
with task_dependencies; use task_dependencies;

package body graphical_editor.tasks is


Task_table_length : constant natural := 27;


procedure Initialize_Task(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	ob1: GObject:= Get_Object(Object, "textbuffer1");
	ob2: GObject:= Get_Object(Object, "combobox1");
	ob3: GObject:= Get_Object(Object, "combobox3");
	ob4: GObject:= Get_Object(Object, "combobox2");
	ob5: GObject:= Get_Object(Object, "textbuffer2");
	ob6: GObject:= Get_Object(Object, "textbuffer3");
	ob7: GObject:= Get_Object(Object, "textbuffer4");
	ob8: GObject:= Get_Object(Object, "textbuffer12");
	ob9: GObject:= Get_Object(Object, "textbuffer5");
	ob10: GObject:= Get_Object(Object, "combobox5");
	ob11: GObject:= Get_Object(Object, "textbuffer6");
	ob12: GObject:= Get_Object(Object, "textbuffer7");
	ob13: GObject:= Get_Object(Object, "textbuffer8");
	ob14: GObject:= Get_Object(Object, "textbuffer9");
	ob15: GObject:= Get_Object(Object, "textbuffer10");
	ob16: GObject:= Get_Object(Object, "textbuffer11");
	ob17: GObject:= Get_Object(Object, "combobox7");
	ob18: GObject:= Get_Object(Object, "textbuffer15");
	ob19: GObject:= Get_Object(Object, "textbuffer16");
	ob20: GObject:= Get_Object(Object, "textbuffer19");
	ob21: GObject:= Get_Object(Object, "liststore7");
	ob22: GObject:= Get_Object(Object, "treeview2");
	ob23: GObject:= Get_Object(Object, "liststore8");
	ob24: GObject:= Get_Object(Object, "treeview3");
	ob25: GObject:= Get_Object(Object, "liststore4");
	ob26: GObject:= Get_Object(Object, "treeview1");
	wid1: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob1));
	wid2: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob2));
	wid3: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob3));
	wid4: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob4));
	wid5: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob5));
	wid6: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob6));
	wid7: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob7));
	wid8: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob8));
	wid9: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob9));
	wid10: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob10));
	wid11: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob11));
	wid12: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob12));
	wid13: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob13));
	wid14: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob14));
	wid15: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob15));
	wid16: t_widget := (ptype => buffer_string, buffer_value => Gtk_Text_Buffer(ob16));
	wid17: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob17));
	wid18: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob18));
	wid19: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob19));
	wid20: t_widget := (ptype => buffer_integer, buffer_value => Gtk_Text_Buffer(ob20));
	wid21: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob21));
	wid22: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob22));
	wid23: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob23));
	wid24: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob24));
	wid25: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob25));
	wid26: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob26));
	wid27: t_widget := (ptype => my_window, window_value => Tasks_Window);

begin


	my_tab(1..task_table_length) := (new t_widget'(wid1), new t_widget'(wid2), new t_widget'(wid3), new t_widget'(wid4), 
	new t_widget'(wid5), new t_widget'(wid6), new t_widget'(wid7), new t_widget'(wid8), new t_widget'(wid9), new t_widget'(wid10), 
	new t_widget'(wid11), new t_widget'(wid12), new t_widget'(wid13), new t_widget'(wid14), new t_widget'(wid15), 
	new t_widget'(wid16), new t_widget'(wid17), new t_widget'(wid18), new t_widget'(wid19), new t_widget'(wid20), 
	new t_widget'(wid21), new t_widget'(wid22), new t_widget'(wid23), new t_widget'(wid24), new t_widget'(wid25), 
	new t_widget'(wid26), new t_widget'(wid27));
end initialize_Task;


-- Data retrieved from the widgets
--
task_name, task_Cpu_Name, task_Address_Space_Name, task_Parametric_Rule_Name : Unbounded_String;
Task_Type : Tasks_Type;
task_Start_Time, task_Capacity, task_Period, task_Deadline, task_Jitter, task_Blocking_Time, task_Priority, task_Criticality, task_Stack_Memory_Size, 
task_Text_Memory_Size, task_context_switch_overhead, task_every, task_seed : Integer;
task_Policy : Policies;
task_Predictable  : Boolean;
task_offsets: Offsets_Table;
task_number_offsets : integer;
task_Params : User_Defined_Parameters_Table;
task_number_params : integer;



procedure Check_Task(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is  

	Start_Time_string, task_type_string, Capacity_string, Period_string, 
	Deadline_string, Jitter_string, Blocking_Time_string, Priority_task_string, Criticality_string, 
	Policy_string, Stack_Memory_Size_string, Text_Memory_Size_string, Seed_string, 
	context_switch_overhead_string, every_string: Unbounded_String;
	Integer_Ok : Boolean;

	tree_iter_inter: Gtk_Tree_Iter;
	tree_iter_inter_param: Gtk_Tree_Iter;
	param_name, param_type, param_value : GValue;
	activation_value, offset_value_value : GValue;

	a_param : parameter_ptr;
	an_offset : offset_type;


begin

	task_name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	task_Cpu_Name := to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(3).combo_value)));
	task_Address_Space_Name := to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(4).combo_value)));
	task_Parametric_Rule_Name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(16).buffer_value));
	task_type := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(2).combo_value)))).tasks_value;
	
	
	Start_Time_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(7).buffer_value)));
	if (Start_Time_String = "") then
		task_Start_Time := 0;
	else
		to_integer (Start_Time_String, task_Start_Time, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
               ( Lb_Start_Time (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	seed_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(18).buffer_value)));
	if (seed_String = "") then
		task_seed := 0;
	else
		to_integer (seed_String, task_seed, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_seed (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Capacity_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(5).buffer_value)));
	if (Capacity_String = "") then
		task_Capacity := 0;
	else
		to_integer (Capacity_String, task_Capacity, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Capacity (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Period_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(15).buffer_value)));
	if (Period_String = "") then
		task_Period := 0;
	else
		to_integer (Period_String, task_Period, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Period (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Deadline_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(6).buffer_value)));
	if (Deadline_String = "") then
		task_Deadline := 0;
	else
		to_integer (Deadline_String, task_Deadline, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Deadline (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Jitter_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(14).buffer_value)));
	if (Jitter_String = "" or Jitter_String = " ") then
		task_Jitter := 0;
	else
		to_integer (Jitter_String, task_Jitter, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Jitter (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Blocking_Time_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(9).buffer_value)));
	if (Blocking_Time_String = "") then
		task_Blocking_Time := 0;
	else
		to_integer (Blocking_Time_String, task_Blocking_Time, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Blocking_Time (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Criticality_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(13).buffer_value)));
	if (Criticality_String = "") then
		task_Criticality := 0;
	else
		to_integer (Criticality_String, task_Criticality, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Criticality (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Priority_task_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(8).buffer_value)));
	if (Priority_task_String = "") then
		task_Priority := 0;
	else
		to_integer (Priority_task_String, task_Priority, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_priority (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	task_Policy := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(10).combo_value)))).policies_value;

	task_Predictable := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(17).combo_value)))).predictables_value;

	Stack_Memory_Size_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(12).buffer_value)));
	if (Stack_Memory_Size_String = "") then
		task_Stack_Memory_Size := 0;
	else
		to_integer (Stack_Memory_Size_String, task_Stack_Memory_Size, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Stack_Memory_Size (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;  

	Text_Memory_Size_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(11).buffer_value)));
	if (Text_Memory_Size_String = "") then
		task_Text_Memory_Size := 0;
	else
		to_integer (Text_Memory_Size_String, task_Text_Memory_Size, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_Text_Memory_Size (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	context_switch_overhead_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(19).buffer_value)));
	if (context_switch_overhead_String = "") then
		task_context_switch_overhead := 0;
	else
		to_integer (context_switch_overhead_String, task_context_switch_overhead, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_context_switch_overhead (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	every_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(my_tab(20).buffer_value)));
	if (every_String = "") then
		task_every := 0;
	else
		to_integer (every_String, task_every, Integer_Ok);
		if not Integer_Ok then
	  Raise_Exception
           (task_set.Invalid_Parameter'Identity,
            To_String
		( Lb_every (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;



-- Retrieve offsets and parameters
--
initialize(task_params);
	task_number_params:= get_number_elements(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(24).view_value)));
	if task_number_params = 0 then 
			task_Params := No_User_Defined_Parameter;
		else
			tree_iter_inter_param := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(24).view_value)));
			for i in 1..task_number_params
			loop
				Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(24).view_value)), 
				tree_iter_inter_param, 0, param_name);
				Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(24).view_value)), 
				tree_iter_inter_param, 1, param_type);
				Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(24).view_value)), 
				tree_iter_inter_param, 2, param_value);

				case to_type(param_type).params_value is 
					when boolean_parameter => 
					a_param := new parameter'(boolean_parameter, to_unbounded_string(Get_string(param_name)), 
					boolean'value(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(my_tab(23).list_value), 
					tree_iter_inter_param, 2)));

					when integer_parameter => 
					a_param := new parameter'(integer_parameter, to_unbounded_string(Get_string(param_name)),  
					integer'value(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(my_tab(23).list_value), 
					tree_iter_inter_param, 2)));	

					when double_parameter => 
					a_param := new parameter'(double_parameter, to_unbounded_string(Get_string(param_name)), 
					Ada.Numerics.Aux.Double'value(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(my_tab(23).list_value), 
					tree_iter_inter_param, 2)));

					when string_parameter => 
					a_param := new parameter'(string_parameter, to_unbounded_string(Get_string(param_name)), 
					to_unbounded_string(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(my_tab(23).list_value), 
					tree_iter_inter_param, 2)));
				end case;
				a_param.all.parameter_name := to_unbounded_string(Get_string(param_name));

				User_Defined_Parameters_Table_Package.add(task_params, a_param);
					
				unset(param_name);
				unset(param_type);
				unset(param_value);

				next(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(24).view_value)), tree_iter_inter_param);
			end loop;			
		end if;


initialize(task_offsets);
	task_number_offsets:= get_number_elements(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(22).view_value)));
	if task_number_offsets = 0 then 
		task_Offsets := No_Offset;
	else 
		tree_iter_inter := Get_Iter_First (Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(22).view_value)));
		for i in 1..task_number_offsets
		loop
			Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(22).view_value)), tree_iter_inter, 0, activation_value);
			Gtk.Tree_Model.Get_Value(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(22).view_value)), tree_iter_inter, 1, offset_value_value);
			an_offset.activation := integer(Get_int(activation_value));
			an_offset.offset_value := integer(get_int(offset_value_value));
			Offsets_Table_Package.Add(task_Offsets, An_offset);

			unset(activation_value);
			unset(offset_value_value);

			next(Gtk_Tree_Model(Gtk.Tree_View.Get_Model(my_tab(22).view_value)), tree_iter_inter);
		end loop;
	end if;

   Check_entity_location(sys, task_name, task_cpu_name, task_Address_Space_Name);

	Check_Task (sys.tasks, task_name, task_Cpu_Name, task_Address_Space_Name, Task_Type, task_Start_Time , task_Capacity, task_Period, task_Deadline, task_Jitter, task_Blocking_Time, task_Priority, task_Criticality , task_Policy, task_Offsets, task_Stack_Memory_Size, task_Text_Memory_Size, task_Params, task_Parametric_Rule_Name, task_Seed, task_Predictable, task_context_switch_overhead, task_every);


end Check_Task;



procedure Add_In_Sys_Task(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin

	Add_Task (sys.tasks, task_name, task_Cpu_Name, task_Address_Space_Name, Task_Type, task_Start_Time , task_Capacity, task_Period, task_Deadline, task_Jitter, task_Blocking_Time, task_Priority, task_Criticality , task_Policy, task_Offsets, task_Stack_Memory_Size, task_Text_Memory_Size, task_Params, task_Parametric_Rule_Name, task_Seed, task_Predictable, task_context_switch_overhead, task_every);

end Add_In_Sys_Task;



procedure Deleted_In_Sys_Task(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	A_Task      : Generic_Task_Ptr;
	name: unbounded_string;
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	A_Task := Search_Task (Sys.Tasks, name);

	Check_Entity_Referencing_a_task(sys, a_task);

	Delete (Sys.tasks, A_Task);
end Deleted_In_Sys_Task;



procedure Show_In_Sys_Task(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
	A_Task      : Generic_Task_Ptr;
	name : unbounded_string;
	tree_iter, tree_iter_param: Gtk_Tree_Iter;
	liststore, liststore_param: GObject; 
begin
	name := get_value_buffer_string(Gtk_Text_Buffer(my_tab(1).buffer_value));
	A_Task := Search_task (Sys.tasks, name);

	liststore:= Get_Object(Object, "liststore7");
	Gtk.List_Store.Clear(Gtk_List_Store(liststore));

	for i in 0..A_Task.offsets.Nb_Entries -1
	loop
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, gint(A_Task.offsets.Entries(i).activation));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, gint(A_Task.offsets.Entries(i).offset_value));
	end loop;

	liststore_param:= Get_Object(Object, "liststore8");
	Gtk.List_Store.Clear(Gtk_List_Store(liststore_param));

	for i in 0..A_Task.parameters.Nb_Entries-1
	loop
		Append(Gtk_List_Store(liststore_param), tree_iter_param);
		Gtk.List_Store.Set(Gtk_List_Store(liststore_param), tree_iter_param, 0, 
			to_string(A_Task.parameters.Entries(i).parameter_name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore_param), tree_iter_param, 1, 
			to_string(to_label(new t_all_enumeration_type'(Parameters_enum, 
			A_Task.parameters.Entries(i).type_of_parameter))));
		case A_Task.parameters.Entries(i).type_of_parameter is
		when boolean_parameter =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore_param), tree_iter_param, 2, 
			boolean'image(A_Task.parameters.Entries(i).boolean_value));
		when integer_parameter =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore_param), tree_iter_param, 2, 
			integer'image(A_Task.parameters.Entries(i).integer_value));
		when double_parameter =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore_param), tree_iter_param, 2, 
			double'image(A_Task.parameters.Entries(i).double_value));
		when string_parameter =>
		Gtk.List_Store.Set(Gtk_List_Store(liststore_param), tree_iter_param, 2, 
			to_string(A_Task.parameters.Entries(i).string_value));
		end case;
	end loop;
	
end Show_In_Sys_Task;



procedure Modify_In_Sys_Task(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

        A_task : Generic_task_Ptr;
        A_task_Buffer : Generic_task_Ptr;
        id : unbounded_string;
begin
        A_task := Search_task (Sys.tasks, task_name);
        id:=a_task.cheddar_private_id;
        delete(sys.tasks, a_task);

	Add_Task (sys.tasks, a_task_buffer, task_name, task_Cpu_Name, task_Address_Space_Name, Task_Type, task_Start_Time , task_Capacity, task_Period, task_Deadline, task_Jitter, task_Blocking_Time, task_Priority, task_Criticality , task_Policy, task_Offsets, task_Stack_Memory_Size, task_Text_Memory_Size, task_Params, task_Parametric_Rule_Name, task_Seed, task_Predictable, task_context_switch_overhead, task_every);
        a_task_buffer.cheddar_private_id:=id;

	Update_All_Task_Dependencies (Sys.Dependencies, a_task_buffer);

end Modify_In_Sys_Task;



procedure Add_View_parameters (Object : access Gtkada_Builder_Record'Class) is 

	tree_iter: Gtk_Tree_Iter;
	liststore : GObject;
	param_name, param_value : unbounded_string;
	param_type : Parameters.Parameter_type;
	ok : boolean;
	param_value_integer : integer;
	param_value_double : double;
	param_value_boolean: boolean;


begin
	liststore:= Get_Object(Object, "liststore8");

	param_name := get_value_buffer_string(Gtk_Text_Buffer(Get_Object(Object, "textbuffer17")));
	param_type := to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox6"))))).params_value;
	param_value := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(Get_Object(Object, "textbuffer18"))));

     case param_type is
       when  Boolean_Parameter =>
		if param_value = "" then
			param_value_boolean:=true;
		else
			to_boolean (param_value, param_value_boolean, Ok);
			if not ok then
		 	Raise_Exception
           			(task_set.Invalid_Parameter'Identity,
            			To_String ( lb_parameter(Current_Language) & Lb_Must_be(Current_Language) & " Boolean"  ));
			end if;
		end if;
       when  Integer_Parameter =>
		if param_value = "" then
			param_value_integer:=0;
		else
			to_integer (param_value, param_value_integer, Ok);
			if not ok then
		 	Raise_Exception
           			(task_set.Invalid_Parameter'Identity,
            			To_String ( lb_parameter(Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
			end if;
		end if;
       when  Double_Parameter =>
		if param_value = "" then
			param_value_double:=0.0;
		else
			to_double (param_value, param_value_double, Ok);
			if not ok then
		 	Raise_Exception
           			(task_set.Invalid_Parameter'Identity,
            			To_String ( lb_parameter(Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
			end if;
		end if;
	when  String_Parameter =>
				null;
		end case;

	Append(Gtk_List_Store(liststore), tree_iter);
	Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, to_string(param_name));
	Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, 
		to_string(to_label(new t_all_enumeration_type'(Parameters_enum, param_type))));
	case param_type is
	  when  Boolean_Parameter =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, param_value_boolean'img);
          when  Integer_Parameter =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, param_value_integer'img);
          when  Double_Parameter =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, param_value_double'img);
	  when  String_Parameter =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, to_string(param_value));
	end case;



exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;
end Add_View_parameters;


procedure Add_View_offsets (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	value, activation: integer;
	value_string, activation_string: unbounded_string;
	ok: boolean := false;
	liststore : GObject;
begin

	liststore:= Get_Object(Object, "liststore7");

	activation_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(Get_Object(Object, "textbuffer13"))));
	if (activation_String = "") then
		activation:=0;
	else
		to_integer (activation_String, activation, Ok);
		if not ok then
		 Raise_Exception
           		(task_set.Invalid_Parameter'Identity,
            		To_String ( Lb_offset_act (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	value_String := suppress_space(get_value_buffer_string(Gtk_Text_Buffer(Get_Object(Object, "textbuffer14"))));
	if (value_String = "") then
		value:=0;
	else
		to_integer (value_String, value, Ok);
		if not ok then
		 Raise_Exception
           		(task_set.Invalid_Parameter'Identity,
            		To_String ( Lb_offset_val (Current_Language) & Lb_Must_Be_Numeric (Current_Language) ));
		end if;
	end if;

	Append(Gtk_List_Store(liststore), tree_iter);
	Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, Gint(activation));
	Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, Gint(value));

exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Add_View_offsets;


procedure Delete_View_offsets (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	path: Gtk_Tree_Path;
	focus: Gtk_Tree_View_Column;
begin
	Get_Cursor(Gtk_Tree_View(Get_Object(Object, "treeview2")), path, focus);
	if path /= null and focus /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(Get_Object(Object, "liststore7")), path);
		Gtk.List_Store.Remove(Gtk_List_Store(Get_Object(Object, "liststore7")), tree_iter);
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer13")), "");
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer14")), "");
	end if;

exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Delete_View_Offsets;


procedure Delete_view_parameters (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	path: Gtk_Tree_Path;
	focus: Gtk_Tree_View_Column;
begin
	Get_Cursor(Gtk_Tree_View(Get_Object(Object, "treeview3")), path, focus);
	if path /= null and focus /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(Get_Object(Object, "liststore8")), path);
		Gtk.List_Store.Remove(Gtk_List_Store(Get_Object(Object, "liststore8")), tree_iter);
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer17")), "");
		Set_Active(Gtk_Combo_Box(Get_Object(Object, "combobox6")), Gint(0));
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer18")), "");
	end if;

exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Delete_view_parameters;


procedure Show_Lign_offsets (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	path1: Gtk_Tree_Path;
	focus1: Gtk_Tree_View_Column;
	combo_text: unbounded_string;
begin
	Get_Cursor(Gtk_Tree_View(get_object(object, "treeview2")), path1, focus1);
	if path1 /= null and focus1 /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(get_object(object, "liststore7")), path1);
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer13")), 
		integer'image(integer(Gtk.Tree_Model.Get_int(Gtk_Tree_Model(get_object(object, "liststore7")), tree_iter, 0))));
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer14")), 
		integer'image(integer(Gtk.Tree_Model.Get_int(Gtk_Tree_Model(get_object(object, "liststore7")), tree_iter, 1))));

	end if;
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Show_Lign_offsets;


procedure Show_Lign_parameters (Object : access Gtkada_Builder_Record'Class) is 
	tree_iter: Gtk_Tree_Iter;
	path1: Gtk_Tree_Path;
	focus1: Gtk_Tree_View_Column;
	combo_text: unbounded_string;
begin
	Get_Cursor(Gtk_Tree_View(get_object(object, "treeview3")), path1, focus1);
	if path1 /= null and focus1 /= null then 
		tree_iter:= get_iter(Gtk_Tree_Model(get_object(object, "liststore8")), path1);
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer17")), 
		Gtk.Tree_Model.Get_string(Gtk_Tree_Model(get_object(object, "liststore8")), tree_iter, 0));
		combo_text := to_unbounded_string(Gtk.Tree_Model.Get_String(Gtk_Tree_Model(get_object(object, "liststore8")), 
		tree_iter, 1));
		Set_Active(Gtk_Combo_Box(get_object(object, "combobox6")), 
		Gint(get_int_combo(Gtk_Combo_Box(get_object(object, "combobox6")), combo_text)));
		Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer18")), 
		Gtk.Tree_Model.Get_string(Gtk_Tree_Model(get_object(object, "liststore8")), tree_iter, 2));

	end if;
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end Show_Lign_parameters;


procedure randomize_seed (Object : access Gtkada_Builder_Record'Class) is 
      Seed : Generator;
      Nmb  : Integer;
begin
      -- Give random value for the tasks seed
      --
      Reset (Seed);

      -- Range from integer'first .. integer'last
      --
      Nmb :=
        Integer (Double (Random (Seed)) *
                 Double (Integer'Last - Integer'First) +
                 Double (Integer'First));
	Set_Text (Gtk_Text_Buffer(Get_Object(Object, "textbuffer15")), Nmb'Img);
exception
 when others =>
        Show_Message_Box (Exception_Name & " : " & Exception_Message);
        return;  
end randomize_seed;



package Task_callbacks is new generic_callbacks(Number_of_widget => Task_table_length, Initialize => initialize_Task, 
	Check => Check_Task, Add_In_Sys => Add_In_Sys_Task, Deleted_In_Sys => Deleted_In_Sys_Task, 
	Modify_In_Sys => Modify_In_Sys_Task, Show_In_Sys => Show_In_Sys_Task);  use Task_callbacks;



procedure Show_Task_Widget is
 
	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;
	My_Iterator : Tasks_Iterator;
	A_Task: Generic_Task_Ptr;
	tree_iter: Gtk_Tree_Iter;
	liststore: GObject;

	A_Processor : Generic_Processor_Ptr;
	My_Processor_Iterator : Processors_Iterator;
	my_combo_cpu: GObject;

	An_Adress_Space : Address_Space_Ptr;
	My_Adress_Space_Iterator : Address_Spaces_Iterator;
	my_combo_adress_space: GObject;

begin
	Gtk.Main.Init;
        Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "task.glade");

	if Error /= null then
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;

	initialize(sys_buffer);
        duplicate(sys, Sys_buffer);

	my_combo_cpu := Get_Object(Builder, "combobox3");
	reset_iterator (sys.Processors, My_Processor_Iterator);
	if not is_empty (sys.Processors) then 
	loop
	current_element (sys.Processors, A_Processor, My_Processor_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_cpu), To_String(A_Processor.name));
	exit when is_last_element (sys.Processors, My_Processor_Iterator);
		next_element (sys.Processors, My_Processor_Iterator);
	end loop;
	else
	Show_Message_Box
	(Lb_Define_processors_Before (Current_Language));
	end if;

	my_combo_adress_space := Get_Object(Builder, "combobox2");
	reset_iterator (sys.Address_Spaces, My_Adress_Space_Iterator);
	if not is_empty (sys.Address_Spaces) then 
	loop
	current_element (sys.Address_Spaces, An_Adress_Space, My_Adress_Space_Iterator);
	
	Append_Text (Gtk_Combo_Box(my_combo_adress_space), To_String(An_Adress_Space.name));
	exit when is_last_element (sys.Address_Spaces, My_Adress_Space_Iterator);
		next_element (sys.Address_Spaces, My_Adress_Space_Iterator);
	end loop;
	else
	Show_Message_Box
	(Lb_Define_Address_Spaces_Before (Current_Language));
	end if;

	Set_Text (Gtk_Text_Buffer(Get_Object(Builder, "textbuffer12")), "1");

	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox1")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox3")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox2")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox5")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox6")), 0);
	Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox7")), 1);

	liststore:= Get_Object(Builder, "liststore4");

	reset_iterator (sys.Tasks, My_Iterator);
	if not is_empty (sys.Tasks) then
		loop
		current_element (sys.Tasks, A_Task, My_Iterator);
		Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (A_Task.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, To_String (A_Task.cpu_name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 3, To_String (A_Task.address_space_name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 4, Gint(A_Task.capacity));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 5, Gint(A_Task.deadline));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 6, Gint(A_Task.start_time));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 7, Gint(A_Task.priority));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 8, Gint(A_Task.blocking_time));

		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 9, 
		to_string(to_label(new t_all_enumeration_type'(Policies_enum, A_Task.policy))));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 10, Gint(A_Task.text_memory_size));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 11, Gint(A_Task.stack_memory_size));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 12, Gint(A_Task.criticality));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 18, Gint(A_Task.context_switch_overhead));

		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 13, Gint(0));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 14, Gint(0));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 15, "");
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 16, to_string(to_label(new t_all_enumeration_type'(Predictables, False))));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 17, 0);
		
		case A_Task.task_type is
		when Aperiodic_Type =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, "Aperiodic");
		when Periodic_Type =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, "Periodic");

		when Poisson_Type =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, "Poisson");
		when  Scheduling_Task_type  =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, "Scheduling");

		when Sporadic_Type =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, "Sporadic");
                 
		when Parametric_Type =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, "Parametric");

		when frame_task_type =>
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, "Multi Frame");

	end case;


	 if	A_Task.task_type = Parametric_Type or 
		A_Task.task_type = Periodic_Type or 
		A_Task.task_type = Sporadic_Type or 
		A_Task.task_type = Poisson_Type or 
		A_Task.task_type = Scheduling_Task_type then
	        	Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 13, Gint (Periodic_Task_Ptr (A_Task).jitter));
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 14, Gint (Periodic_Task_Ptr (A_Task).period));
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 19, Gint(Periodic_Task_Ptr (A_Task).every));
	end if;

        if A_Task.task_type  =  Poisson_Type or 
		A_Task.task_type = Scheduling_Task_type  then
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 16, to_string(to_label(new t_all_enumeration_type'(Predictables, Poisson_Task_Ptr (A_Task).predictable))));
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 17, Gint(Poisson_Task_Ptr (A_Task).seed));
	end if;
                 
        if A_Task.task_type  =  Parametric_Type  then
			Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 15, To_String(Parametric_Task_Ptr (A_Task).activation_rule));
	end if;



	exit when is_last_element (sys.Tasks, My_Iterator);
	next_element (sys.Tasks, My_Iterator);
	         end loop;
      end if;

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview1_cursor_changed", 
	Handler      => Task_callbacks.Show_Lign_Selected'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button6_clicked", 
	Handler      => Task_callbacks.Add_View'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button5_clicked", 
	Handler      => Task_callbacks.Modify_Lign'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button4_clicked", 
	Handler      => Task_callbacks.Delected_Lign_Selected'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => Task_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => Task_callbacks.Close_View'Access);  

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button7_clicked", 
	Handler      => Add_View_offsets'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button8_clicked", 
	Handler      => Delete_View_offsets'Access);  

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button9_clicked", 
	Handler      => Add_View_parameters'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button10_clicked", 
	Handler      => delete_View_parameters'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button3_clicked", 
	Handler      => randomize_seed'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview2_cursor_changed", 
	Handler      => show_lign_offsets'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview3_cursor_changed", 
	Handler      => show_lign_parameters'Access);
	
	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
	Gtk.Main.Main;
	Unref (Builder);
 
end Show_Task_Widget;

end graphical_editor.tasks;
