------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.About_Dialog; use Gtk.About_Dialog;
with Gtk.Label;        use Gtk.Label;
with Glib.Object; use Glib.Object;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Gtk.Combo_Box; use Gtk.Combo_Box;
with Gtk.Text_Iter;    use Gtk.Text_Iter;
with Gtk.List_Store;   use Gtk.List_Store;
with Gtk.Tree_Model;  use Gtk.Tree_Model;
with Gtk.Tree_View;  use Gtk.Tree_View;
with Gtk.Tree_View_Column;  use Gtk.Tree_View_Column;
with Glib.Values; use Glib.Values;
with Glib; use Glib;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
with Ada.Strings.Unbounded;    use Ada.Strings.Unbounded;
with Gtkada.Builder; use Gtkada.Builder;
with graphical_editor.manual_callbacks; use graphical_editor.manual_callbacks;
with convert_unbounded_strings; 
with task_dependencies; use task_dependencies;
use task_dependencies.Half_Dep_Set;
with dependencies; use dependencies;
with task_set; use task_set;
with tasks; use tasks;
with graphical_editor.convert_text; use graphical_editor.convert_text;


package body graphical_editor.time_triggered_communication_dependency is

time_triggered_table_length : constant natural :=6;

procedure Initialize_time_triggered(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
ob1: GObject:= Get_Object(Object, "combobox1");
ob2: GObject:= Get_Object(Object, "combobox2");
ob3: GObject:= Get_Object(Object, "combobox3");
ob4: GObject:= Get_Object(Object, "liststore1");
ob5: GObject:= Get_Object(Object, "treeview1");
wid1: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob1));
wid2: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob2));
wid3: t_widget := (ptype => combo, combo_value => Gtk_Combo_Box(ob3));
wid4: t_widget := (ptype => lists, list_value => Gtk_List_Store(ob4));
wid5: t_widget := (ptype => view, view_value => Gtk_Tree_View(ob5));
wid6: t_widget := (ptype => my_window, window_value => time_triggered_dependency_window);

begin
	my_tab(1..time_triggered_table_length) := (new t_widget'(wid1), new t_widget'(wid2), new t_widget'(wid3), new t_widget'(wid4), new t_widget'(wid5), new t_widget'(wid6)  );
end initialize_time_triggered;


-- Data retrieved from widget
--
time_triggered_sink : unbounded_string;
time_triggered_source: unbounded_string;
time_triggered_protocol : Time_Triggered_Communication_Timing_Property_Type;


procedure Check_time_triggered(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 

begin
time_triggered_sink:=to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(1).combo_value)));
time_triggered_source:=to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(2).combo_value)));
time_triggered_protocol:=to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(3).combo_value)))).trigerres_value;
end check_time_triggered;

procedure Add_In_Sys_time_triggered(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
	 add_one_task_dependency_time_triggered(sys.dependencies,
                                    search_task(sys.tasks, time_triggered_source),
                                    search_task(sys.tasks, time_triggered_sink),
					time_triggered_protocol );

end Add_In_Sys_time_triggered;

procedure Deleted_In_Sys_time_triggered(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
time_triggered_sink:=to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox1"))));
time_triggered_source:=to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(Get_Object(Object, "combobox2"))));
time_triggered_protocol:=to_type(to_unbounded_string(Get_Active_Text(Gtk_Combo_Box(my_tab(3).combo_value)))).trigerres_value;
      
	 delete_one_task_dependency_time_triggered(sys.dependencies,
                                    search_task(sys.tasks, time_triggered_source),
                                    search_task(sys.tasks, time_triggered_sink),
					time_triggered_protocol );
end Deleted_In_Sys_time_triggered;

procedure Show_In_Sys_time_triggered(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
	null;
end Show_In_Sys_time_triggered;

procedure Modify_In_Sys_time_triggered(my_tab: in out table_widget_id ; Object : access Gtkada_Builder_Record'Class) is 
begin
	null;
end Modify_In_Sys_time_triggered;

package time_triggered_callbacks is new generic_callbacks(Number_of_widget => time_triggered_table_length, Initialize => Initialize_time_triggered, 
	Check => Check_time_triggered, Add_In_Sys => Add_In_Sys_time_triggered, Deleted_In_Sys => Deleted_In_Sys_time_triggered, 
	Modify_In_Sys => Modify_In_Sys_time_triggered, Show_In_Sys => Show_In_Sys_time_triggered);  use time_triggered_callbacks;


procedure Show_time_triggered_Widget is
 
   Builder : Gtkada_Builder;
   Error   : Glib.Error.GError;
        liststore: GObject;
        tree_iter: Gtk_Tree_Iter;

My_Iterator : Tasks_Dependencies_Iterator;
      A_Half_Dep   : Dependency_Ptr;

  combo_task_sink, combo_task_source : GObject; 
        My_Task_Iterator : Tasks_Iterator;
        A_Task : Generic_Task_Ptr;


begin
   Gtk.Main.Init;
                         
   Gtk_New (Builder);
   Error := Add_From_File (Builder, to_string(Glade_path) & "time_triggered_communication_dependency.glade");
   if Error /= null then
      Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
      Error_Free (Error);
      return;
   end if;


        initialize(Sys_buffer);
        duplicate(sys, Sys_buffer);


        combo_task_sink := Get_Object(Builder, "combobox1");
        combo_task_source := Get_Object(Builder, "combobox2");


        reset_iterator (sys.Tasks, My_Task_Iterator);
        if not is_empty (sys.Tasks) then
        loop
        	current_element (sys.Tasks, A_Task, My_Task_Iterator);

        	Append_Text (Gtk_Combo_Box(combo_task_sink), To_String(A_Task.name));
        	Append_Text (Gtk_Combo_Box(combo_task_source), To_String(A_Task.name));

        	exit when is_last_element (sys.Tasks, My_Task_Iterator);
                next_element (sys.Tasks, My_Task_Iterator);
        end loop;
        end if;

        Set_Active (Gtk_Combo_Box(combo_task_source), 0);
	Set_Active (Gtk_Combo_Box(combo_task_sink), 0);
        Set_Active (Gtk_Combo_Box(Get_Object(Builder, "combobox3")), 0);


      if not is_empty (sys.Dependencies.Depends) then

	liststore:= Get_Object(Builder, "liststore1");
         reset_iterator (sys.Dependencies.Depends, My_Iterator);

         loop
            current_element
              (sys.Dependencies.Depends,
               A_Half_Dep,
               My_Iterator);

            if A_Half_Dep.type_of_dependency = dependencies.Time_Triggered_Communication_Dependency then
 Append(Gtk_List_Store(liststore), tree_iter);
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 0, To_String (A_Half_Dep.time_triggered_communication_sink.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 1, To_String (A_Half_Dep.time_triggered_communication_source.name));
		Gtk.List_Store.Set(Gtk_List_Store(liststore), tree_iter, 2, To_String(to_label(new t_all_enumeration_type'(trigerres, A_Half_Dep.timing_property))) );
            end if;


  	exit when is_last_element (sys.Dependencies.Depends, My_Iterator);

            next_element (sys.Dependencies.Depends, My_Iterator);
         end loop;
      end if;



	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button2_clicked", 
	Handler      => time_triggered_callbacks.Add_View'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button4_clicked", 
	Handler      => time_triggered_callbacks.Delected_Lign_Selected'Access); 

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button3_clicked", 
	Handler      => time_triggered_callbacks.Cancel_View'Access);

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_treeview1_cursor_changed", 
	Handler      => time_triggered_callbacks.Show_Lign_Selected'Access);  

	Gtkada.Builder.Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => time_triggered_callbacks.Close_View'Access);


    Do_Connect (Builder);

   
  Gtk.Widget.Show_All (Get_Widget (Builder, "window1"));
  
   Gtk.Main.Main;
 
   Unref (Builder);
 
end Show_time_triggered_Widget;

end graphical_editor.time_triggered_communication_dependency;

