------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk;            use Gtk;
with Gtk.Main;       use Gtk.Main;
with Glib.Error;     use Glib.Error;
with Gtk.Widget;     use Gtk.Widget;
with Gtk.Object;     use Gtk.Object;
with Gtk.Dialog;     use Gtk.Dialog;
with Gtk.Message_Dialog; use Gtk.Message_Dialog;
with Gtk.Label;        use Gtk.Label;
with Ada.Text_IO;
with Gtkada.Builder; use Gtkada.Builder;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with unbounded_strings;     use unbounded_strings;
use unbounded_strings.strings_table_package;
with Glib;      use Glib;
with Glib.Object; use Glib.Object;
with graphical_editor.user_message_callbacks; use graphical_editor.user_message_callbacks;

package body graphical_editor.user_message is

procedure Show_Message_Box (Msg : Unbounded_String) is

	Builder : Gtkada_Builder;
	Error   : Glib.Error.GError;
	message_dialog : GObject;

begin

	Gtk.Main.Init;
	Gtk_New (Builder);
	Error := Add_From_File (Builder, to_string(Glade_path) & "user_message.glade");
	if Error /= null then 
		Ada.Text_IO.Put_Line ("Error : " & Get_Message (Error));
		Error_Free (Error);
		return;
	end if;

	message_dialog:=Get_Object(Builder, "messagedialog1");
	Set_Markup(Gtk_Message_Dialog(message_dialog), To_String(unbounded_lf & Msg & unbounded_lf));

 	Register_Handler
	(Builder      => Builder,
	Handler_Name => "on_button1_clicked", 
	Handler      => graphical_editor.user_message_callbacks.Close_Message_Box'Access);
	Do_Connect (Builder);
	Gtk.Widget.Show_All (Get_Widget (Builder, "messagedialog1"));
	Gtk.Main.Main;
	Unref (Builder);
 
end Show_Message_Box;

procedure Show_Message_Box (Msg : String) is
begin
	Show_Message_Box (To_Unbounded_String (Msg));
end Show_Message_Box;


procedure Show_Feature_Not_Available is 
begin
	Show_Message_Box ("Feature not allowed with this Cheddar release");
end Show_Feature_Not_Available;

end graphical_editor.user_message;
