------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Call_Scheduling_Framework;         use Call_Scheduling_Framework;
with Framework;                  use Framework;
with Framework_Config;                  use Framework_Config;
with Translate;                         use Translate;
with Multiprocessor_Services;           use Multiprocessor_Services;
with Multiprocessor_Services_Interface; use Multiprocessor_Services_Interface;
use Multiprocessor_Services_Interface.Scheduling_Result_Per_Processor_Package;
with Multiprocessor_Services;           use Multiprocessor_Services;
with Gtkada.Dialogs;                    use Gtkada.Dialogs;
with Gtk;                               use Gtk;
with Gtk.Main;                          use Gtk.Main;
with Ada.Text_IO;                use Ada.Text_IO;


package body graphical_editor is

	
   procedure Do_New is
   begin

        if (not Is_Saved) then
           if (not Is_New) then
               if Message_Dialog
                     (To_String
                        (Lb_Not_Saved_Project (Current_Language) &
                         ASCII.LF &
                          Lb_Continue (Current_Language)),
                    Confirmation,
                     Button_Yes or Button_Cancel,
                     Button_Cancel, 
                     "", 
                      "") /=
                  Button_Yes
               then
                  return;
              end if;
          end if;
         end if;



      Initialize (Sys);
      Xml_Project_File_Name := To_Unbounded_String ("");
      initialize (Aadl_Project_File_Name);
      Xml_Event_Table_File_Name :=
         To_Unbounded_String ("default_event_table.xml");

      Is_Saved := False;
      Is_New   := True;

      Event_Table_File_Name := empty_string;
      initialize (Sched.all);

   end Do_New;

   procedure Do_Open is
   begin

      if (not Is_Saved) then
        if (not Is_New) then
            if Message_Dialog
                 (To_String
                    (Lb_Not_Saved_Project (Current_Language) &
                      ASCII.LF &
                      Lb_Continue (Current_Language)),
                  Confirmation,
                   Button_Yes or Button_Cancel,
                   Button_Cancel,
                  "", 
                  "") /=
              Button_Yes
           then
               return;
           end if;
        end if;
     end if;

      Initialize (Sys);
      Xml_Project_File_Name := To_Unbounded_String ("");
      initialize (Aadl_Project_File_Name);
      Xml_Event_Table_File_Name :=
         To_Unbounded_String ("default_event_table.xml");

      Is_Saved := False;
      Is_New   := True;

      Event_Table_File_Name := empty_string;
      initialize (Sched.all);

   end Do_Open;

   procedure Do_Save is
   begin

      if (Xml_Project_File_Name /= To_Unbounded_String ("")) then
         Systems.Write_To_Xml_File (Sys, Xml_Project_File_Name);

         Is_Saved := True;
      end if;

   end Do_Save;

   procedure Do_Aadl_Export is
   begin

	null;

   end Do_Aadl_Export;


   procedure Do_Aadl_Import is
   begin

     if (not Is_Saved) then
       if (not Is_New) then
           if Message_Dialog
                 (To_String
                      (Lb_Not_Saved_Project (Current_Language) &
                       ASCII.LF &
                       Lb_Continue (Current_Language)),
                   Confirmation,
                   Button_Yes or Button_Cancel,
                   Button_Cancel, 
                   "",
                   "") /=
               Button_Yes
            then
               return;
            end if;
         end if;
      end if;

   end Do_Aadl_Import;

--procedure Do_Save_As is
--begin
--	null;
--end Do_Save_As;

procedure Do_Exit is
begin
	if (not Is_Saved) then
		if (not Is_New) then
			if Message_Dialog
			(To_String
			(Lb_Not_Saved_Project (Current_Language) &
			ASCII.LF &
			Lb_Continue (Current_Language)),
			Confirmation,
			Button_Yes or Button_Cancel,
			Button_Cancel, 
			"",
			"") = Button_Yes
			then
			Gtk.Main.Gtk_Exit (0);
			else
			return;
			end if;
		end if;
	end if;
	Gtk.Main.Gtk_Exit (0);
end Do_Exit;

end graphical_editor;
