------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances 
-- of real time architectures.
--
-- Copyright (C) 2002-2015, by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by 
-- Frank Singhoff, Lab-STICC UMR 6285 laboratory, Université de Bretagne Occidentale
-- 
-- Since 2008, Ellidiss technologies also contributes to the development of 
-- Cheddar and provides industrial support.
--
-- The full list of contributors and sponsors are given into AUTHORS.txt and SPONSORS.txt
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--           
------------------------------------------------------------------------------
-- Last update : 
--    $Rev: 1249 $
--    $Date: 2014-08-28 07:02:15 +0200 (Fri, 28 Aug 2014) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Systems;               use Systems;
with Framework; use Framework;
with unbounded_strings;     use unbounded_strings;
use unbounded_strings.unbounded_string_list_package;
with Gtk.Text_Buffer; use Gtk.Text_Buffer;
with Gtk.Widget; use Gtk.Widget;
with Gtk.Text_Tag_Table; use Gtk.Text_Tag_Table;
with Gtk.Handlers;     use Gtk.Handlers;
with Gtk.Check_Button; use Gtk.Check_Button;
with gtk.button; use gtk.button;

package graphical_editor is 

package Button_Callback is new Gtk.Handlers.Callback (Gtk_Button_Record);

	-- Directory where to read glade files
	--
	Glade_path: Unbounded_String := install_path & "glade_files/";

	-- The working architecture project
	--
	Sys : System;

	-- This system is used during project editing 
	-- in order to save sys before editing (to allow the user to cancel his modifications)
	--
	Sys_buffer : System;

	-- List of directory where to read AADL or XML files
	--
	Project_File_Dir_List : unbounded_string_list;

	-- File name of the working XML/AADL project files
	--
	Xml_Project_File_Name  : Unbounded_String;
	Aadl_Project_File_Name : unbounded_string_list;

	-- File name of the working XML event table file
	--
	Xml_Event_Table_File_Name : Unbounded_String;

	-- Is the XML/AADL project modified ? should we have to save it ?
	--
	Is_Saved : Boolean := true;
	Is_New   : Boolean := false;

	-- Processor name on which we should compute something
	--
	Last_Selected_Processor : Unbounded_String;

	-- The name of the file in which the event table has to be saved
	--
	Event_Table_File_Name : Unbounded_String;

	-- Interface for cheddar mmi
	--
	buffer_editor_cheddar: Gtk_Text_Buffer;
	my_area : Gtk_Widget;
	
	-- Are we calling a customized scheduling simulation or not ?
	--
	Customized_Simulation : boolean := False;

	-- Sub programs do project file management
	--
	procedure Do_New;
	procedure Do_Open;
	procedure Do_Save;
	--procedure Do_Save_As;
	procedure Do_Exit;
	procedure Do_Aadl_Export;
	procedure Do_Aadl_Import;

end graphical_editor;
