--------------------------------------------------------------
------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 14:09:39 +0100 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_IO; use Text_IO;

package body fifos is

   procedure reset (my_fifo : in out fifo) is
   begin
      my_fifo.queue := 0;
      my_fifo.head  := 0;
      my_fifo.size  := 0;
   end reset;

   procedure put (my_fifo : in fifo) is

      index : element_range := my_fifo.queue;

   begin
      Put_Line
        ("Fifo : queue = " &
         my_fifo.queue'Img &
         "; head = " &
         my_fifo.head'Img &
         "; size = " &
         my_fifo.size'Img);

      while (index /= my_fifo.head) loop
         put (my_fifo.data (index));
         New_Line;

         if (index = element_range'Last) then
            index := element_range'First;
         else
            index := index + 1;
         end if;
      end loop;

      New_Line;
      New_Line;

   end put;

   procedure insert (my_fifo : in out fifo; e : in element) is

      tmp : element_range;
   begin
      if (my_fifo.head = element_range'Last) then
         tmp := element_range'First;
      else
         tmp := my_fifo.head + 1;
      end if;
      if (my_fifo.queue = tmp) then
         raise full_fifo;
      end if;

      my_fifo.data (my_fifo.head) := e;
      if (my_fifo.head = element_range'Last) then
         my_fifo.head := element_range'First;
      else
         my_fifo.head := my_fifo.head + 1;
      end if;

      my_fifo.size := my_fifo.size + 1;
   end insert;

   procedure extract (my_fifo : in out fifo; e : in out element) is
   begin
      if is_empty (my_fifo) then
         raise empty_fifo;
      end if;

      e := my_fifo.data (my_fifo.queue);

      if (my_fifo.queue = element_range'Last) then
         my_fifo.queue := element_range'First;
      else
         my_fifo.queue := my_fifo.queue + 1;
      end if;

      my_fifo.size := my_fifo.size - 1;
   end extract;

   procedure extract_any_where (my_fifo : in out fifo; e : in element) is

      index : element_range := my_fifo.queue;

   begin
      if is_empty (my_fifo) then
         raise empty_fifo;
      end if;

      -- Looking for the element
      --
      while (index /= my_fifo.head) loop

         if e = my_fifo.data (index) then
            while index /= my_fifo.head loop
               if index = element_range'Last then
                  my_fifo.data (index) := my_fifo.data (element_range'First);
               else
                  my_fifo.data (index) := my_fifo.data (index + 1);
               end if;

               if (index = element_range'Last) then
                  index := element_range'First;
               else
                  index := index + 1;
               end if;
            end loop;

            if (my_fifo.head = element_range'First) then
               my_fifo.head := element_range'Last;
            else
               my_fifo.head := my_fifo.head - 1;
            end if;
            my_fifo.size := my_fifo.size - 1;
            return;
         end if;

         if (index = element_range'Last) then
            index := element_range'First;
         else
            index := index + 1;
         end if;
      end loop;

      raise element_not_found;

   end extract_any_where;

   function consult (my_fifo : in fifo) return element is
   begin
      if is_empty (my_fifo) then
         raise empty_fifo;
      end if;

      return (my_fifo.data (my_fifo.queue));
   end consult;

   function is_empty (my_fifo : in fifo) return Boolean is
   begin
      return (my_fifo.queue = my_fifo.head);
   end is_empty;

   function get_size (my_fifo : in fifo) return Natural is
   begin
      return (my_fifo.size);
   end get_size;

end fifos;
