---------------------------------------------------------------------
-----------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 14:09:39 +0100 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
generic

   max_element : Natural := 100;
   type element is private;
   with procedure put (a_element : in element);

package fifos is

   type fifo is tagged private;
   type element_range is new Natural range 0 .. max_element;
   type fifo_ptr is access all fifo'Class;

   full_fifo : exception;
   empty_fifo : exception;
   element_not_found : exception;

   -- Clear the fifo
   --
   procedure reset (my_fifo : in out fifo);

   -- Put the content of the fifo in the
   -- screen
   --
   procedure put (my_fifo : in fifo);

   -- Insert a element into the fifo
   -- We insert with the head pointer
   --
   procedure insert (my_fifo : in out fifo; e : in element);

   -- Extract a element from the fifo
   -- We extract with the queue pointer
   --
   procedure extract (my_fifo : in out fifo; e : in out element);

   -- Extract "Element" from any where
   --
   procedure extract_any_where (my_fifo : in out fifo; e : in element);

   -- Read (but not extract) a element from the fifo
   -- We read with the queue pointer
   --
   function consult (my_fifo : in fifo) return element;

   -- Get the number of elements currently present
   -- in the fifo
   --
   function get_size (my_fifo : in fifo) return Natural;

   -- Is my_fifo empty ?
   --
   function is_empty (my_fifo : in fifo) return Boolean;

private

   type element_table is array (element_range) of element;

   type fifo is tagged record
      size  : Natural := 0;
      data  : element_table;
      queue : element_range;
      head  : element_range;
   end record;

end fifos;
