---------------------------------------------------------------------
-----------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 14:09:39 +0100 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

generic
   size : Positive := 100;
   type data_type is private;

   with function compute_hash (index : in Unbounded_String) return Natural;
   with procedure put (d : data_type);

package hashs is

   type hash is private;

   procedure initialize (h : in out hash);

   procedure add
     (h     : in out hash;
      index : in Unbounded_String;
      d     : in data_type);

   function consult
     (h     : in hash;
      index : in Unbounded_String)
      return  data_type;

   procedure put (h : in hash);

   data_not_found : exception;

private

   type cell;
   type link is access cell;

   type cell is record
      next   : link;
      index  : Unbounded_String;
      a_data : data_type;
   end record;

   type hash is array (1 .. size) of link;

end hashs;
