------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 97 $
--    $Date: 2007-07-20 15:17:15 +0200 (Fri, 20 Jul 2007) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with text_io;
use text_io;
with ada.strings.unbounded;
use ada.strings.unbounded;

with Unchecked_Deallocation;


package id_generators is

   
   type id_value_type is new integer range 0..integer'last;

   type id_kind_type is (bounded_value_type, unbounded_value_type);

   type id is record
	value : id_value_type := id_value_type'first;
	kind : id_kind_type := bounded_value_type;
   end record;


   package id_value_type_io is new integer_io(id_value_type);
   use id_value_type_io;
   package id_kind_type_io is new enumeration_io(id_kind_type);
   use id_kind_type_io;

   type id_ptr is access id;


   exceeded_id_value : exception;


   -- Display an id
   --
   procedure put (e : in id);
   procedure put (e : in id_ptr);


   -- Initialize an id
   --
   procedure initialize (a : out id); 


-- Free memory
--
   procedure free is new Unchecked_Deallocation (id, id_ptr);


   -- Generate a unique value from an id
   --
   procedure generate_id(an_id : in out id; current_id : out id_value_type );
   procedure generate_id(an_id : in out id; current_id : out unbounded_string);


   
end id_generators;

