with Ada.Text_IO;		use Ada.Text_IO;
with Strings;               	use Strings;
with Ada.Strings.Unbounded; 	use Ada.Strings.Unbounded;
with Ada.Strings;		use Ada.Strings;
with Ada.Unchecked_Deallocation;
with GNAT.String_Split;


package body Integer_Arrays is

   procedure Initialize (arr : in out Integer_Array)
   is
   begin
      arr.Elements := new Integer_Arr(0..0);
      arr.Size := 0;
   end Initialize;

   procedure Initialize (arr 	   : in out Integer_Array;
                         Elements  : in STRING;
                         Separator : in STRING)
   is
      Subs : GNAT.String_Split.Slice_Set;
      Sub  : Unbounded_String;
   begin
      arr.Elements := new Integer_Arr(0..0);
      arr.Size := 0;

      GNAT.String_Split.Create (Subs, Elements, Separator, Mode => GNAT.String_Split.Multiple);
      for I in 1 .. GNAT.String_Split.Slice_Count (Subs) loop
         declare
            Sub : String := GNAT.String_Split.Slice (Subs, I);
         begin
            if(Sub'Length>0)then
               Add(arr,Integer'Value(Sub));
            end if;
         end;
      end loop;
   end Initialize;

   procedure Add
     (arr 	: in out Integer_Array;
      element 	: in Integer)
   is
      temp_arr 	: Integer_Array;
   begin

      if(arr.Size = 0) then
         arr.Size := arr.Size + 1;
         arr.Elements(0) := element;
      else
         temp_arr.Size := arr.Size+1;
         temp_arr.Elements := new Integer_Arr(0..temp_arr.Size);
         for i in 0..arr.Size-1 loop
            temp_arr.Elements(i) := arr.Elements(i);
         end loop;

         Free(arr.Elements);

         temp_arr.Elements(temp_arr.Size-1) := element;
         arr := temp_arr;
      end if;
   end Add;

   procedure Add
     (arr 	: in out Integer_Array;
      element_identical : in Boolean;
      element 	: in Integer)
   is
      temp_arr 	: Integer_Array;
   begin

      if(element_identical) then
         for i in 0..arr.Size -1 loop
            if(element = arr.Elements(i)) then
               return;
            end if;
         end loop;
      end if;

      if(arr.Size = 0) then
         arr.Size := arr.Size + 1;
         arr.Elements(0) := element;
      else
         temp_arr.Size := arr.Size+1;
         temp_arr.Elements := new Integer_Arr(0..temp_arr.Size);
         for i in 0..arr.Size-1 loop
            temp_arr.Elements(i) := arr.Elements(i);
         end loop;

         Free(arr.Elements);

         temp_arr.Elements(temp_arr.Size-1) := element;
         arr := temp_arr;
      end if;
   end Add;

   procedure Remove
     (arr 	: in out Integer_Array;
      index 	: Integer)
   is
      temp_arr 	: Integer_Array;
   begin
      if(arr.Size > 1) then
         for i in index..arr.Size-2 loop
               arr.Elements(i) := arr.Elements(i+1);
         end loop;
         arr.Size := arr.Size -1;
      else
         arr.Size := 0;
      end if;
   end Remove;

   procedure Remove_By_Value
     (arr	: in out Integer_Array;
      value	: Integer)
   is
   begin
      for i in 0..arr.Size-1 loop
         if(arr.Elements(i) = value) then
            Remove(arr, i);
         end if;
      end loop;
   end Remove_By_Value;

   function Remove_By_Value
     (arr	: in out Integer_Array;
      value	: Integer)
      return Boolean
   is
   begin
      for i in 0..arr.Size-1 loop
         if(arr.Elements(i) = value) then
            Remove(arr, i);
            return true;
         end if;
      end loop;
      return false;
   end Remove_By_Value;

   procedure Union
     (arr_a	: in out Integer_Array;
      arr_b 	: in Integer_Array)
   is
      s_a, s_b	: Integer;
      flag	: boolean;
   begin
      s_a := arr_a.Size-1;
      s_b := arr_b.Size-1;
      for j in 0..s_b loop
         flag := true;
         for i in 0..s_a loop
            if((arr_a.Elements(i) = arr_b.Elements(j)) or (arr_b.Elements(j) = -1)) then
               flag := false;
            end if;
         end loop;

         if(flag) then
            Add(arr     => arr_a,
                element => arr_b.Elements(j));
         end if;
      end loop;

   end Union;

   procedure Union
     (arr_a	: in Integer_Array;
      arr_b 	: in Integer_Array;
      arr_c	: out Integer_Array)
   is
      s_a, s_b	: Integer;
      flag	: boolean;
   begin
      arr_c.Elements := new Integer_Arr(0..0);
      arr_c.Size := 0;

      s_a := arr_a.Size-1;
      s_b := arr_b.Size-1;

      for j in 0..s_a loop
         Add(arr_c, arr_a.Elements(j));
      end loop;

      for j in 0..s_b loop
         flag := true;
         for i in 0..s_a loop
            if((arr_a.Elements(i) = arr_b.Elements(j)) or (arr_b.Elements(j) = -1)) then
               flag := false;
            end if;
         end loop;

         if(flag) then
            Add(arr     => arr_c,
                element => arr_b.Elements(j));
         end if;
      end loop;
   end Union;

   procedure Intersect
     (arr_a	: in Integer_Array;
      arr_b 	: in Integer_Array;
      arr_c	: out Integer_Array;
      n : out Integer)
   is
      flag : Boolean := false;
   begin
      n := 0;
      Initialize(arr_c);

      for i in 0 .. arr_a.Size-1 loop
         flag := false;
         for j in 0 .. arr_b.Size-1 loop
            if (arr_a.Elements(i) = arr_b.Elements(j) AND arr_a.Elements(i) > -1 AND To_Unbounded_String(arr_a.Elements(i)'Img) /= "") then
               flag := true;
            end if;
         end loop;

         if(flag)then
            Integer_Arrays.Add(arr     => arr_c,
                              element => arr_a.Elements(i));
              n := n+1;
         end if;
      end loop;

   end Intersect;

   procedure Intersect
     (arr_a	: in Integer_Array;
      arr_b 	: in Integer_Array;
      n : out Integer)
   is
      flag : Boolean := false;
   begin
      n := 0;
      for i in 0 .. arr_a.Size-1 loop
         flag := false;
         for j in 0 .. arr_b.Size-1 loop
            if (arr_a.Elements(i) = arr_b.Elements(j) AND arr_a.Elements(i) > -1 AND To_Unbounded_String(arr_a.Elements(i)'Img) /= "") then
               flag := true;
            end if;
         end loop;

         if(flag)then
              n := n+1;
         end if;
      end loop;
   end Intersect;

   procedure Sort_DESC
     (arr_a	: in out Integer_Array)
   is
      finished : boolean;
      temp : integer;
   begin
      loop
         finished := true;
         for j in 0 .. arr_a.Size-2 loop
            if(arr_a.Elements(j+1) > arr_a.Elements(j)) then
               finished := false;
               temp := arr_a.Elements(j+1);
               arr_a.Elements(j+1) := arr_a.Elements(j);
               arr_a.Elements(j) := temp;
            end if;
         end loop;

         exit when finished;
      end loop;
   end Sort_DESC;

   procedure Sort_ASC
     (arr_a	: in out Integer_Array)
   is
      finished : boolean;
      temp : integer;
   begin
      loop
         finished := true;
         for j in 0 .. arr_a.Size-2 loop
            if(arr_a.Elements(j+1) < arr_a.Elements(j)) then
               finished := false;
               temp := arr_a.Elements(j+1);
               arr_a.Elements(j+1) := arr_a.Elements(j);
               arr_a.Elements(j) := temp;
            end if;
         end loop;

         exit when finished;
      end loop;
   end Sort_ASC;

   procedure Print(arr : in Integer_Array)
   is
   begin
      for i in 0..arr.Size-1 loop
         Put(arr.Elements(i)'Img);
      end loop;
      Put_Line("");
   end Print;

   procedure Put(arr : in Integer_Array)
   is
   begin
      for i in 0..arr.Size-1 loop
         Put(arr.Elements(i)'Img);
      end loop;
      Put_Line("");
   end Put;

   function XML_String (obj : in Integer_Array)
   return  Unbounded_String
   is
   begin
      return To_Unbounded_String("NOT IMPLEMENTED YET");
   end XML_String;

   procedure Free (arr : in out Integer_Array)
   is
   begin
      Free(arr.Elements);
      arr.Size := 0;
   end Free;

end Integer_Arrays;
