with Ada.Text_IO;	use Ada.Text_IO;
with Ada.Unchecked_Deallocation;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package Integer_Arrays is

   ------------------------------------------------------
   -- This package implements an array which is
   -- - Not bounded
   -- - Extendable
   -- - Can perform some methods such as sort, union, intersection.
   ------------------------------------------------------

   type Integer_Arr is array (Integer range <>) of Integer;
   type Integer_Arr_Ptr is access Integer_Arr;

   procedure Free is new Ada.Unchecked_Deallocation(Object => Integer_Arr,
                                                    Name   => Integer_Arr_Ptr);

   type Integer_Array is record
      Size 	  : Integer := 0;
      Elements    : Integer_Arr_Ptr;
   end record;

   procedure Free (arr : in out Integer_Array);

   procedure Initialize (arr : in out Integer_Array);

   procedure Initialize (arr 	   : in out Integer_Array;
                         Elements  : in STRING;
                         Separator : in STRING);

   -----------------------------------------------------------------------------

   procedure Print(arr : in Integer_Array);

   procedure Put(arr : in Integer_Array);

   function XML_String (obj   : in Integer_Array)
      return  Unbounded_String;

   procedure Add
     (arr 	: in out Integer_Array;
      element 	: in Integer);

   procedure Add
     (arr 		: in out Integer_Array;
      element_identical : in Boolean;
      element 		: in Integer);
   -- Add an element to the Array

   procedure Remove
     (arr 	: in out Integer_Array;
      index 	: Integer);
   -- Remove an element at index index

   procedure Remove_By_Value
     (arr	: in out Integer_Array;
      value	: Integer);
   -- Remove an element which has value value

   function Remove_By_Value
     (arr	: in out Integer_Array;
      value	: Integer)
      return Boolean;
   -- Remove an element which has value value, return true if a element is removed

   procedure Union
     (arr_a	: in out Integer_Array;
      arr_b 	: in Integer_Array);
   -- Union the arr_b with the arr_a
   -- The arr_a is the result of the union.

   procedure Intersect
     (arr_a	: in Integer_Array;
      arr_b 	: in Integer_Array;
      arr_c	: out Integer_Array;
      n 	: out Integer);
   --Intersect arr_a with arr_b
   --Result in arr_c
   --n is the number of intersection.

   procedure Intersect
     (arr_a	: in Integer_Array;
      arr_b 	: in Integer_Array;
      n 	: out Integer);
   --Intersect arr_a with arr_b
   --n is the number of intersection.

   procedure Sort_DESC
     (arr_a	: in out Integer_Array);

   procedure Sort_ASC
     (arr_a	: in out Integer_Array);

end Integer_Arrays;
