------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 14:09:39 +0100 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package io_tools is

   type file_mode is (save_in_new_file, append_to_old_file);

   can_not_open_file_for_reading : exception;
   can_not_open_file_for_writing : exception;

   -- Read a ASCII file and store it in a
   -- ADA string
   -- I/O Exceptions can be raised in case of I/O error
   --
   function read_sequential_file
     (file_name : Unbounded_String)
      return      Unbounded_String;
   function read_sequential_file
     (file_name : String)
      return      Unbounded_String;

   -- Read a ASCII file and store it in a
   -- ADA string
   -- I/O Exceptions can be raised in case of I/O error
   --
   procedure write_sequential_file
     (file_name : in Unbounded_String;
      data      : in Unbounded_String);
   procedure write_sequential_file
     (file_name : in String;
      data      : in Unbounded_String);

   -- Check is a file can be read
   -- Return True if the file can be read
   --
   function can_be_read (file_name : Unbounded_String) return Boolean;

end io_tools;
