------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 548 $
--    $Date: 2012-10-12 01:48:51 +0200 (Fri, 12 Oct 2012) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Ada.Numerics.Aux;       use Ada.Numerics.Aux;
with Unchecked_Deallocation;


package natural_util is

   type natural_ptr is access Natural;

   -- Convert and format a natural to
   -- an unbounded_string
   --
   function format (f : in Natural) return Unbounded_String;

   -- Display a natural
   --
   procedure put (e : in Natural);
   procedure put (e : in natural_ptr);
   procedure put_line (e : in Natural);

   -- Initialize a natural
   --
   procedure initialize (a : out Natural);

   -- Compute Great common Divisor
   -- of two natural
   --
   function gcd (a : in Natural; b : in Natural) return Natural;

   -- Compute Last common multiplier
   -- of two natural
   --
   function lcm (a : in Natural; b : in Natural) return Natural;
   function lcm (a : in Natural; b : in Natural) return Double;

   procedure free is new Unchecked_Deallocation (Natural, natural_ptr);

   function copy (e : in natural_ptr) return natural_ptr;


end natural_util;
