---------------------------------------------------------------
-----------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 14:09:39 +0100 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_IO; use Text_IO;

package body real_util is

   package real_element_io is new Text_IO.Float_IO (real_element);

   function equal
     (num1, num2 : real_element;
      accuracy   : Integer := 10000)
      return       Boolean
   is
      my_epsi : constant real_element := 1.0 / real_element (accuracy);
      diff    : constant real_element := abs (num1 - num2);
   begin
      return (diff < my_epsi);
   end equal;

   function format
     (f     : in real_element;
      after : in Natural := 5)
      return  Unbounded_String
   is
      s      : String (1 .. 10000);
      result : Unbounded_String;
   begin
      real_element_io.Put (s, f, after, Exp => 0);

      -- suppress blank
      for i in 1 .. s'Length loop
         if s (i) /= ' ' then
            Append (result, s (i));
         end if;
      end loop;

      return (result);
   end format;

end real_util;
