-------------------------------------------------------
-------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 14:09:39 +0100 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_IO; use Text_IO;

package body stacks is

   procedure reset (a_stack : in out stack) is

   begin
      a_stack.first_free := 0;
   end reset;

   function get_size (my_stack : in stack) return element_range is
   begin
      return my_stack.first_free;
   end get_size;

   -- Is my_stack empty ?
   --
   function is_empty (my_stack : in stack) return Boolean is
   begin
      return my_stack.first_free = 0;
   end is_empty;

   -- Put the content of the stack in the
   -- screen
   --
   procedure put (my_stack : in stack) is

   begin
      Put_Line ("Number of elements : " & my_stack.first_free'Img);
      for i in 0 .. my_stack.first_free - 1 loop
         put (my_stack.data (i));
      end loop;
      New_Line;
   end put;

   -- Get the last inserted element
   --
   function get_top (my_stack : in stack) return element is
   begin
      if my_stack.first_free = 0 then
         raise empty_stack;
      end if;
      return my_stack.data (my_stack.first_free - 1);
   end get_top;

   -- Push a new element
   --
   procedure push (my_stack : in out stack; a_element : in element) is
   begin
      if my_stack.first_free = element_range'Last then
         raise full_stack;
      end if;
      my_stack.data (my_stack.first_free) := a_element;
      my_stack.first_free                 := my_stack.first_free + 1;

   end push;

   -- Pop a element
   --
   procedure pop (my_stack : in out stack; a_element : in out element) is

   begin
      if my_stack.first_free = 0 then
         raise empty_stack;
      end if;
      a_element           := my_stack.data (my_stack.first_free - 1);
      my_stack.first_free := my_stack.first_free - 1;
   end pop;

end stacks;
