--------------------------------------------------------------------
------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 523 $
--    $Date: 2012-09-26 14:09:39 +0100 (Wed, 26 Sep 2012) $
--    $Author: fotsing $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
generic
   max_element : Natural := 100;
   type element is private;
   with procedure put (a_element : in element);

package stacks is

   type stack is tagged private;
   type element_range is new Natural range 0 .. max_element;
   type stack_ptr is access all stack'Class;

   empty_stack, full_stack : exception;

   -- Clear the stack
   --
   procedure reset (a_stack : in out stack);

   -- Get the number of elements currently present
   -- in the stack
   --
   function get_size (my_stack : in stack) return element_range;

   -- Is my_stack empty ?
   --
   function is_empty (my_stack : in stack) return Boolean;

   -- Put the content of the stack in the
   -- screen
   --
   procedure put (my_stack : in stack);

   -- Get the last inserted element
   --
   function get_top (my_stack : in stack) return element;

   -- Push a new element
   --
   procedure push (my_stack : in out stack; a_element : in element);

   -- Pop a element
   --
   procedure pop (my_stack : in out stack; a_element : in out element);

private

   type element_table is array (element_range) of element;

   type stack is tagged record
      first_free : element_range := 0;
      data       : element_table;
   end record;

end stacks;
