------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 541 $
--    $Date: 2012-10-08 22:23:24 +0200 (Mon, 08 Oct 2012) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with ada.strings.unbounded;
use ada.strings.unbounded;
with Unchecked_Deallocation;


generic
   type Element is private;

   Size : Natural := 100;
   with procedure Put (E : in Element);
   with function xml_string (E : in Element)
	return unbounded_string;
   with function xml_ref_string (E : in Element)
        return unbounded_string;


package Tables is

   type Table_Range is new Natural range 0 .. Size;
   type Table_Range_ptr is access table_range;

   type Data_Table is array (Table_Range) of Element;

   type Table is record
      Nb_Entries : Table_Range := 0;
      Entries    : Data_Table;
   end record;

  type Table_Ptr is access Table;


   procedure Put (A_Table : in Table);
   procedure Put (A_Table : in Table_ptr);

   procedure Initialize (A_Table : out Table);

   procedure Delete (A_Table : in out Table; A_Item : Table_Range);

   procedure Add (A_Table : in out Table; A_Element : Element);

  function XML_String(obj : in Table) return Unbounded_String;
  function XML_String(obj : in Table_ptr) return Unbounded_String;
  function XML_root_String(obj : in Table) return Unbounded_String;
  function XML_root_String(obj : in Table_ptr) return Unbounded_String;


   procedure Free is new Unchecked_Deallocation (
      Table,
      Table_Ptr);

   pragma Inline (Add);
   pragma Inline (Delete);
   pragma Inline (Put);
   pragma Inline (Initialize);

end Tables;
