-------------------------------
-------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 97 $
--    $Date: 2007-07-20 15:17:15 +0200 (Fri, 20 Jul 2007) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with unbounded_strings;         use unbounded_strings;
with Version;                   use Version;
with Translate.Francais_Labels; use Translate.Francais_Labels;
with Translate.English_Labels;  use Translate.English_Labels;

package body Translate.AADL_Labels is

   procedure Initialize is

   begin

      if (Current_Language = English) then

         Lb_Task (Current_Language) :=
            To_Unbounded_String ("Thread component");

         Lb_Compute_Buffer (Current_Language) :=
            To_Unbounded_String
              ("Event data port analysis with feasibility tests");
         Lb_Draw_Buffer (Current_Language)    :=
            To_Unbounded_String
              ("Event data port analysis from scheduling simulation");

         Lb_Compute_And_Update (Current_Language)             :=
            To_Unbounded_String ("Compute and Update thread components");
         Lb_Compute_And_Update_One_Step (Current_Language)    :=
            To_Unbounded_String
              ("Compute and Update thread components : one step ");
         Lb_Check_And_Update_Address_Space (Current_Language) :=
            To_Unbounded_String ("Check and update process component");
         Lb_Check_Address_Space (Current_Language)            :=
            To_Unbounded_String ("Check process component");

         Lb_Update_Task (Current_Language)          :=
            To_Unbounded_String ("Update thread components");
         Lb_Update_Address_Space (Current_Language) :=
            To_Unbounded_String ("Update process components");
         Lb_Update_Resource (Current_Language)      :=
            To_Unbounded_String ("Update data components");
         Lb_Update_Buffer (Current_Language)        :=
            To_Unbounded_String ("Update event data ports");
         Lb_Update_Processor (Current_Language)     :=
            To_Unbounded_String ("Update processor components");

         Lb_Resource_Not_Found (Current_Language)      :=
            To_Unbounded_String ("Data component not found");
         Lb_Task_Not_Found (Current_Language)          :=
            To_Unbounded_String ("Thread component not found");
         Lb_Address_Space_Not_Found (Current_Language) :=
            To_Unbounded_String ("Process component not found");
         Lb_Buffer_Not_Found (Current_Language)        :=
            To_Unbounded_String ("Event data port not found");
         Lb_Processor_Not_Found (Current_Language)     :=
            To_Unbounded_String ("Processor component not found");

         Lb_Define_Tasks_Before (Current_Language)          :=
            To_Unbounded_String ("Define a thread component before");
         Lb_Define_Address_Spaces_Before (Current_Language) :=
            To_Unbounded_String ("Define a process component before");
         Lb_Define_Resources_Before (Current_Language)      :=
            To_Unbounded_String ("Define a data component before");
         Lb_Define_Buffers_Before (Current_Language)        :=
            To_Unbounded_String ("Define an event data port before");
         Lb_Define_Processors_Before (Current_Language)     :=
            To_Unbounded_String ("Define a processor component before");

         Lb_Can_Not_Define_More_Tasks (Current_Language)          :=
            To_Unbounded_String ("Can not define more thread components");
         Lb_Can_Not_Define_More_Address_Spaces (Current_Language) :=
            To_Unbounded_String ("Can not define more process components");
         Lb_Can_Not_Define_More_Resources (Current_Language)      :=
            To_Unbounded_String ("Can not define more data components");
         Lb_Can_Not_Define_More_Buffers (Current_Language)        :=
            To_Unbounded_String ("Can not define more event data ports");
         Lb_Can_Not_Define_More_Processors (Current_Language)     :=
            To_Unbounded_String ("Can not define more processor components");

         --
         --  Processors widgets
         --
         Lb_Scheduler (Current_Language)              :=
            To_Unbounded_String ("Scheduling protocol");
         Lb_Select_Processor_First (Current_Language) :=
            To_Unbounded_String ("Select a processor component first");
         Lb_File_Name_Control (Current_Language)      :=
            To_Unbounded_String
              ("Invalid file name with the selected scheduling protocol");
         Lb_Quantum_Control (Current_Language)        :=
            To_Unbounded_String
              ("Invalid quantum with selected scheduling protocol");
         Lb_Processor (Current_Language)              :=
            To_Unbounded_String ("Processor component ");
         Lb_Processor_Name (Current_Language)         :=
            To_Unbounded_String ("Processor component name");
         Lb_Scheduler (Current_Language)              :=
            To_Unbounded_String ("Scheduling protocol");

         --
         --  Address space widgets
         --
         Lb_Address_Space_Name (Current_Language)         :=
            To_Unbounded_String ("Process component name");
         Lb_Address_Space (Current_Language)              :=
            To_Unbounded_String ("Process component");
         Lb_Select_Address_Space_First (Current_Language) :=
            To_Unbounded_String ("Select a process component first");
         Lb_Invalid_Scheduler (Current_Language)          :=
            To_Unbounded_String ("Invalid scheduling protocol");

         --
         --  Tasks widgets
         --
         Lb_Task_Name (Current_Language)                        :=
            To_Unbounded_String ("Thread component name");
         Lb_Task (Current_Language)                             :=
            To_Unbounded_String ("Thread component");
         Lb_Task_Type (Current_Language)                        :=
            To_Unbounded_String ("Dispatch protocol");
         Lb_Capacity (Current_Language)                         :=
            To_Unbounded_String ("Compute execution time");
         Lb_Rule_For_Parametric_Only (Current_Language)         :=
            To_Unbounded_String
              ("Activation rule only with user-defined thread component");
         Lb_Select_Task_First (Current_Language)                :=
            To_Unbounded_String ("Select a thread component first");
         Lb_Task_Not_Permitted (Current_Language)               :=
            To_Unbounded_String
              ("Operation not permitted with the selected thread component");
         Lb_Have_To_Be_On_The_Same_Processor (Current_Language) :=
            To_Unbounded_String
              ("have to be on the same processor component");

         --
         --  Buffers widgets
         --
         Lb_Buffer (Current_Language)              :=
            To_Unbounded_String ("Event data port");
         Lb_Buffer_Name (Current_Language)         :=
            To_Unbounded_String ("Event data port name");
         Lb_Select_Buffer_First (Current_Language) :=
            To_Unbounded_String ("Select an event data port first");
         Lb_Bound_Buffer (Current_Language)        :=
            To_Unbounded_String ("Bound on event data ports");
         Lb_Buffer_Information (Current_Language)  :=
            To_Unbounded_String ("Event data port information ");
         Lb_Buffer_List (Current_Language)         :=
            To_Unbounded_String ("Event data port list ");
         Lb_Role_List (Current_Language)           :=
            To_Unbounded_String ("Threads using the event data port ");

         --
         --  Resources widgets
         --
         Lb_Resource_Name (Current_Language)           :=
            To_Unbounded_String ("Data component name");
         Lb_Resource (Current_Language)                :=
            To_Unbounded_String ("Data component");
         Lb_Select_Resource_First (Current_Language)   :=
            To_Unbounded_String ("Select a data component first");
         Lb_Update_Resource_Control (Current_Language) :=
            To_Unbounded_String ("Invalid selected thread component");
         Lb_Resource_Filter (Current_Language)         :=
            To_Unbounded_String ("Select a data component");
         Lb_Protocol (Current_Language)                :=
            To_Unbounded_String ("Concurrency control protocol");
         Lb_Used_Protocol (Current_Language)           :=
            To_Unbounded_String ("Used concurrency control protocol : ");
         Lb_Resource_State (Current_Language)          :=
            To_Unbounded_String ("Data component state (Integer) : ");
         Lb_Resources (Current_Language)               :=
            To_Unbounded_String ("Data components");
         Lb_Resource_Affect (Current_Language)         :=
            To_Unbounded_String ("Data component shared by threads ");
         Lb_Resource_Information (Current_Language)    :=
            To_Unbounded_String ("Data component information ");
         Lb_Invalid_Protocol (Current_Language)        :=
            To_Unbounded_String
              (" Invalid data component concurrency control protocol ");
         Lb_Cpu_Filter (Current_Language)              :=
            To_Unbounded_String ("Select a processor component");

         --
         --  Dependency graph widgets
         --
         Lb_New_Buffer_Role (Current_Language) :=
            To_Unbounded_String ("New event data port role");

         --
         -- AADL options widget
         --
         Lb_Import_Aadl_Event_To_Precedencies (Current_Language)      :=
            To_Unbounded_String
              ("Transform event port to thread component precedencies");
         Lb_Import_Aadl_Event_Data_To_Precedencies (Current_Language) :=
            To_Unbounded_String
              ("Transform event data port to thread component precedencies");
         Lb_Export_Aadl_Precedencies_To_Event (Current_Language)      :=
            To_Unbounded_String(
"Transform thread component precedencies to event port connections");
         Lb_Export_Aadl_Precedencies_To_Event_Data (Current_Language) :=
            To_Unbounded_String(
"Transform thread component precedencies to event data port connections");
         Lb_Export_Aadl_Precedencies_To_Properties (Current_Language) :=
            To_Unbounded_String(
"Transform thread component precedencies to Cheddar's properties");

         --
         -- System entity checks
         --
         Lb_are_unconsistent (Current_Language) :=
            To_Unbounded_String (" are unconsistent");

         ----------------------------------------------------
         --   Seed options
         ----------------------------------------------------
         Lb_Task_Specific (Current_Language)     :=
            To_Unbounded_String ("Thread component specific seed");
         Lb_All_Predictable (Current_Language)   :=
            To_Unbounded_String ("Predictable for all thread components");
         Lb_All_Unpredictable (Current_Language) :=
            To_Unbounded_String ("Unpredictable for all thread components");

         --
         -- Shared resources logic
         --
         Lb_Compute_Blocking_Error1 (Current_Language)      :=
            To_Unbounded_String(
"All data components have to use the same concurrency control protocol.");
         Lb_Compute_Blocking_Error2 (Current_Language)      :=
            To_Unbounded_String(
"- Concurrency control protocol error : shared data components use protocol which does not work with the selected scheduling protocol."
);
         Lb_Compute_Blocking_Error3 (Current_Language)      :=
            To_Unbounded_String(
"Can not compute bound on blocking time with such shared data component concurrency control protocol."
);
         Lb_Has_Priority_Inversion (Current_Language)       :=
            To_Unbounded_String
              ("  has a priority inversion on the data component ");
         Lb_Blocking_Time_Inject_Success (Current_Language) :=
            To_Unbounded_String
              ("- Thread component blocking times are updated.");
         Lb_Blocking_Time_Inject_Failed (Current_Language)  :=
            To_Unbounded_String
              ("- Thread component blocking times are not updated.");

         --
         -- Scheduling logic
         --
         Lb_Feasibility_Number2 (Current_Language) :=
            To_Unbounded_String(
"2) Feasibility test based on worst case thread component response time : ");
         Lb_Priorities_Error1 (Current_Language) :=
            To_Unbounded_String
              ("- Thread components must be periodic : can not set priorities."
);
         Lb_Priorities_Error2 (Current_Language) :=
            To_Unbounded_String
              ("- Processor without tasks : can not set priorities.");
         Lb_Sched_Explanation1 (Current_Language) :=
            To_Unbounded_String(
"the thread component set is schedulable because the processor utilization factor "
);
         Lb_Sched_Explanation2 (Current_Language) :=
            To_Unbounded_String(
"the thread component set is not schedulable because the processor utilization factor "
);
         Lb_Sched_Explanation3 (Current_Language) :=
            To_Unbounded_String(
"we can not prove that the thread component set is schedulable because the processor utilization factor "
);
         Lb_Sched_Explanation5 (Current_Language) :=
            To_Unbounded_String(
"the thread component set is schedulable because for each thread component ");
         Lb_Sched_Explanation6 (Current_Language) :=
            To_Unbounded_String
              ("the thread component set is not schedulable because ");
         Lb_Sched_Explanation62 (Current_Language) :=
            To_Unbounded_String(
"one or many thread components doesn't/don't satisfy the processor utilization factor feasibility test"
);
         Lb_Sched_Explanation7 (Current_Language) :=
            To_Unbounded_String
              ("  List of thread components not schedulable : ");
         Lb_Sched_Explanation8 (Current_Language) :=
            To_Unbounded_String(
"we can not prove that thread components are schedulable because ");
         Lb_Sched_Explanation81 (Current_Language) :=
            To_Unbounded_String(
"one or many thread components doesn't/don't satisfy the processor utilization factor feasibility test"
);
         Lb_Sched_Explanation82 (Current_Language) :=
            To_Unbounded_String ("  List of concerned thread components : ");
         Lb_No_Deadline_Missed_In_The_Computed_Scheduling (Current_Language)
            :=
            To_Unbounded_String(
"- No deadline missed in the computed scheduling : the thread component is schedulable if you computed the simulation on the hyperperiod."
);
Lb_Cannot_Say_If_Deadline_will_be_Missed_In_The_Computed_Scheduling
(current_language)    :=
        To_Unbounded_String(
                            "- One or several tasks did not complete their execution : for those tasks, it is not possible to say if their deadline can be missed or not."
                           );
         Lb_No_Deadline_Will_Be_Missed_Task_Are_Schedulable (Current_Language)
            :=
            To_Unbounded_String(
"- All thread component deadlines will be met : the thread component set is schedulable."
);
         Lb_Deadline_Will_Be_Missed_Task_Are_Not_Schedulable (Current_Language)
            :=
            To_Unbounded_String(
"- Some thread component deadlines will be missed : the thread component set is not schedulable."
);
         Lb_worst_case_Task_Response_Time (Current_Language) :=
            To_Unbounded_String ("Worst case thread component response time");
         Lb_Task_Response_Time (Current_Language) :=
            To_Unbounded_String ("Thread component response time");
         Lb_Task_Response_Time_From_Simulation (Current_Language) :=
            To_Unbounded_String
              ("Thread component response time computed from simulation");
         Lb_Partitioning_Result (Current_Language) :=
            To_Unbounded_String
              ("Thread component assignement after partitioning ");
         Lb_Compute_Scheduling_Error_1 (Current_Language) :=
            To_Unbounded_String(
"- Thread component model error : can not apply the selected scheduling protocol with this thread component set."
);
         Lb_Compute_Scheduling_Error_2 (Current_Language) :=
            To_Unbounded_String(
"- Thread components must be periodic : can not compute hyperperiod with this thread component set. "
);
         Lb_Compute_Scheduling_Error_3 (Current_Language) :=
            To_Unbounded_String(
"- Thread component model error : can not compute hyperperiod with this thread component set. "
);
         Lb_Compute_Scheduling_Error_4 (Current_Language) :=
            To_Unbounded_String(
"- Thread components must have period equal to deadline : can not compute bound on response time with this thread component set. "
);
         Lb_Compute_Scheduling_Error_5 (Current_Language) :=
            To_Unbounded_String(
"- Thread components must be periodic : can not compute bound on response time with this thread component set."
);
         Lb_Compute_Scheduling_Error_6 (Current_Language) :=
            To_Unbounded_String(
"- Thread component model error : can not compute bound on response time with this thread component set."
);
         Lb_Compute_Scheduling_Error_7 (Current_Language) :=
            To_Unbounded_String(
"- Processor utilization exceeded : can not compute bound on response time with this thread component set. "
);
         Lb_Compute_Scheduling_Error_8 (Current_Language) :=
            To_Unbounded_String(
"- Internal Error : can not compute bound on response time with this thread component set."
);
         Lb_Compute_Scheduling_Error_9 (Current_Language) :=
            To_Unbounded_String(
"- Thread components must be periodic : can not compute bound on processor utilization factor with period on this thread component set. "
);
         Lb_Compute_Scheduling_Error_10 (Current_Language) :=
            To_Unbounded_String(
"- Thread component model error : can not compute processor utilization factor with period. "
);
         Lb_Compute_Scheduling_Error_11 (Current_Language) :=
            To_Unbounded_String(
"- Several thread components have same priority level : can not apply the selected scheduling protocol with this thread component set."
);
         Lb_Compute_Scheduling_Error_12 (Current_Language) :=
            To_Unbounded_String(
"- Thread component with non zero start time : can not compute bound on response time with this thread component set."
);
         Lb_Compute_Scheduling_Error_13 (Current_Language) :=
            To_Unbounded_String(
"- Thread component with non zero offset : can not compute bound on response time with this thread component set."
);
         Lb_Compute_Scheduling_Error_14 (Current_Language) :=
            To_Unbounded_String(
"- Thread component must have period equal to deadline : can not apply the selected scheduling protocol with this thread component set. "
);
         Lb_Compute_Scheduling_Error_15 (Current_Language) :=
            To_Unbounded_String(
"- Thread component must be periodic : can not apply the selected scheduling protocol with this thread component set. "
);
         Lb_Compute_Scheduling_Error_21 (Current_Language) :=
            To_Unbounded_String(
"- Thread components must have deadline > 0 : can not compute processor utilization factor with deadline on this thread component set. "
);
         Lb_Periodic (Current_Language) :=
            To_Unbounded_String ("Periodic thread component ");
         Lb_Aperiodic (Current_Language) :=
            To_Unbounded_String ("Aperiodic thread component ");
         Lb_Poisson_Process_Task (Current_Language) :=
            To_Unbounded_String ("Poisson thread component ");
         Lb_Sporadic_Task (Current_Language) :=
            To_Unbounded_String ("Sporadic thread component ");
         Lb_Parametric_Task (Current_Language) :=
            To_Unbounded_String ("User-defined thread component ");
         lb_task_is_not_over_response_time_is_not_computed (Current_Language)
            :=
            To_Unbounded_String(
" response time not computed since the thread component did not run all its compute execution time"
);
         Lb_Run_Simulation_On_One_Processor_Only (Current_Language) :=
            To_Unbounded_String ("Schedule all processor components");
         Lb_Run_Feasibility_Tests_On_One_Processor_Only (Current_Language) :=
            To_Unbounded_String (" Feasibility on all processor components");
         Lb_Partition_Error1 (Current_Language) :=
            To_Unbounded_String(
"- Can not do partitioning : all processor components must have the same scheduling protocol."
);
         Lb_Partition_Error2 (Current_Language) :=
            To_Unbounded_String(
"- Can not do partitioning : you must define more processor components.");
         Lb_Partition_Error3 (Current_Language) :=
            To_Unbounded_String(
"- Invalid scheduling protocol : can not run scheduling partition. ");

         --
         -- Buffer logic
         --
         Lb_Buffer_Utilization (Current_Language)     :=
            To_Unbounded_String ("Event data port utilization factor : ");
         Lb_Compute_Buffer_Error_1 (Current_Language) :=
            To_Unbounded_String(
"- Data flow constraint not respected : can not compute event data port bound with this thread component set. "
);
         Lb_Compute_Buffer_Error_2 (Current_Language) :=
            To_Unbounded_String(
"- Thread components must be periodic : can not compute event data port bound with this thread component set. "
);
         Lb_Compute_Buffer_Error_3 (Current_Language) :=
            To_Unbounded_String(
"- Thread component model error : can not compute bound with several consumers. "
);

         Lb_Maximum_Buffer_Size (Current_Language) :=
            To_Unbounded_String
              ("- Maximum number of messages in the event data port : ");
         Lb_Average_Buffer_Size (Current_Language) :=
            To_Unbounded_String
              ("- Average number of messages in the event data port : ");

         --
         -- Dependencies logic
         --
         Lb_Precedencies_Period_Error (Current_Language) :=
            To_Unbounded_String(
"- Can not schedule with precedencies : thread component precedencies required that thread components have the same period."
);
         Lb_Chetto_Error2 (Current_Language)             :=
            To_Unbounded_String(
"- Thread components must have the same period : can not apply Chetto/Blazewicz method on this thread component set."
);
         Lb_Chetto_Error3 (Current_Language)             :=
            To_Unbounded_String(
"- Thread components model error : can not apply Chetto/Blazewicz method on this thread component set. "
);

      else
         if (Current_Language = Francais) then
            Lb_Task (Current_Language) :=
               To_Unbounded_String ("Composant thread");

            Lb_Compute_Buffer (Current_Language) :=
               To_Unbounded_String
                 ("Analyse des event data ports par tests de faisabilit");
            Lb_Draw_Buffer (Current_Language)    :=
               To_Unbounded_String(
"Analyse des event data ports par simulation d'ordonnancement");

            Lb_Compute_And_Update (Current_Language)             :=
               To_Unbounded_String
                 ("Calculer et mettre  jour les composants thread");
            Lb_Compute_And_Update_One_Step (Current_Language)    :=
               To_Unbounded_String
                 ("Calculer et mettre  jour les composants thread : un pas ");
            Lb_Check_And_Update_Address_Space (Current_Language) :=
               To_Unbounded_String
                 ("Vrification et mise jour des composants process");
            Lb_Check_Address_Space (Current_Language)            :=
               To_Unbounded_String ("Vrification des composants process");

            Lb_Update_Task (Current_Language)          :=
               To_Unbounded_String ("Mise  jour des composants thread");
            Lb_Update_Address_Space (Current_Language) :=
               To_Unbounded_String ("Mise  jour des composants process");
            Lb_Update_Resource (Current_Language)      :=
               To_Unbounded_String ("Mise  jour des composants data");
            Lb_Update_Buffer (Current_Language)        :=
               To_Unbounded_String ("Mise  jour des event data ports");
            Lb_Update_Processor (Current_Language)     :=
               To_Unbounded_String ("Mise  jour des composants processor");

            Lb_Resource_Not_Found (Current_Language)      :=
               To_Unbounded_String ("Composant data inconnue");
            Lb_Task_Not_Found (Current_Language)          :=
               To_Unbounded_String ("Composant thread inconnu");
            Lb_Address_Space_Not_Found (Current_Language) :=
               To_Unbounded_String ("Composant process inconnu");
            Lb_Buffer_Not_Found (Current_Language)        :=
               To_Unbounded_String ("Event data port inconnu");
            Lb_Processor_Not_Found (Current_Language)     :=
               To_Unbounded_String ("Composant processor inconnu");

            Lb_Define_Tasks_Before (Current_Language)          :=
               To_Unbounded_String ("Dfinir un composant thread d'abord");
            Lb_Define_Address_Spaces_Before (Current_Language) :=
               To_Unbounded_String ("Dfinir un composant process d'abord");
            Lb_Define_Resources_Before (Current_Language)      :=
               To_Unbounded_String ("Dfinir un composant data d'abord");
            Lb_Define_Buffers_Before (Current_Language)        :=
               To_Unbounded_String ("Dfinir un event data port d'abord");
            Lb_Define_Processors_Before (Current_Language)     :=
               To_Unbounded_String ("Dfinir un composant processor d'abord");

            Lb_Can_Not_Define_More_Tasks (Current_Language)          :=
               To_Unbounded_String
                 ("Impossible de dfinir d'autres composants thread");
            Lb_Can_Not_Define_More_Address_Spaces (Current_Language) :=
               To_Unbounded_String
                 ("Impossible de dfinir d'autres composants process");
            Lb_Can_Not_Define_More_Resources (Current_Language)      :=
               To_Unbounded_String
                 ("Impossible de dfinir d'autres composants data");
            Lb_Can_Not_Define_More_Buffers (Current_Language)        :=
               To_Unbounded_String
                 ("Impossible de dfinir d'autres event data ports");
            Lb_Can_Not_Define_More_Processors (Current_Language)     :=
               To_Unbounded_String
                 ("Impossible de dfinir d'autres composants processor");

            -- widget
            Lb_Scheduler (Current_Language) :=
               To_Unbounded_String ("Scheduling protocol");

            --
            --  Processors widgets
            --
            Lb_Select_Processor_First (Current_Language) :=
               To_Unbounded_String
                 ("Choisissez un composant processor d'abord");
            Lb_File_Name_Control (Current_Language)      :=
               To_Unbounded_String(
"Nom de fichier invalide avec le scheduling protocol slectionn");
            Lb_Quantum_Control (Current_Language)        :=
               To_Unbounded_String(
"Valeur du quantum invalide avec le scheduling protocol slectionn");
            Lb_Processor (Current_Language)              :=
               To_Unbounded_String ("Composant processor ");
            Lb_Processor_Name (Current_Language)         :=
               To_Unbounded_String ("Nom du composant processor");
            Lb_Scheduler (Current_Language)              :=
               To_Unbounded_String ("Scheduling protocol");

            --
            --  Address space widgets
            --
            Lb_Address_Space_Name (Francais)         :=
               To_Unbounded_String ("Nom du composant process");
            Lb_Address_Space (Francais)              :=
               To_Unbounded_String ("Composant process");
            Lb_Invalid_Scheduler (Francais)          :=
               To_Unbounded_String ("Scheduling protocol invalide");
            Lb_Select_Address_Space_First (Francais) :=
               To_Unbounded_String
                 ("Choisissez un composant process d'abord");

            --
            --  Tasks widgets
            --
            Lb_Task_Name (Current_Language)                        :=
               To_Unbounded_String ("Nom du composant thread");
            Lb_Task (Current_Language)                             :=
               To_Unbounded_String ("Composant thread");
            Lb_Task_Type (Current_Language)                        :=
               To_Unbounded_String ("Dispatch protocol");
            Lb_Capacity (Current_Language)                         :=
               To_Unbounded_String ("Compute execution time");
            Lb_Rule_For_Parametric_Only (Current_Language)         :=
               To_Unbounded_String(
"Rgle d'activation seulement avec un composant thread paramtrique");
            Lb_Select_Task_First (Current_Language)                :=
               To_Unbounded_String ("Choisissez un composant thread d'abord");
            Lb_Task_Not_Permitted (Current_Language)               :=
               To_Unbounded_String
                 ("Operation impossible pour le composant thread selectionn");
            lb_context_switch_overhead (Current_Language)          :=
               To_Unbounded_String ("Context switch overhead");
            Lb_Have_To_Be_On_The_Same_Processor (Current_Language) :=
               To_Unbounded_String
                 ("doivent tre positionns sur le mme composant processor");

            --
            --  Buffers widgets
            --
            Lb_Buffer (Current_Language)              :=
               To_Unbounded_String ("Event data port");
            Lb_Buffer_Name (Current_Language)         :=
               To_Unbounded_String ("Nom de l'event data port");
            Lb_Select_Buffer_First (Current_Language) :=
               To_Unbounded_String ("Choisissez un event data port d'abord");
            Lb_Bound_Buffer (Current_Language)        :=
               To_Unbounded_String ("Borne sur les event data port");
            Lb_Buffer_Information (Current_Language)  :=
               To_Unbounded_String ("Information sur l'event data port ");
            Lb_Buffer_List (Current_Language)         :=
               To_Unbounded_String ("Liste des event data port ");
            Lb_Role_List (Current_Language)           :=
               To_Unbounded_String
                 ("Composants thread utilisant l'event data port ");

            --
            --  Resources widgets
            --
            Lb_Resource_Name (Current_Language)           :=
               To_Unbounded_String ("Nom du composant data");
            Lb_Resource (Current_Language)                :=
               To_Unbounded_String ("Composant data");
            Lb_Select_Resource_First (Current_Language)   :=
               To_Unbounded_String ("Choisissez un composant data d'abord");
            Lb_Update_Resource_Control (Current_Language) :=
               To_Unbounded_String ("Composant thread selectionn invalide");
            Lb_Resource_Filter (Current_Language)         :=
               To_Unbounded_String ("Selectionner un composant data");
            Lb_Protocol (Current_Language)                :=
               To_Unbounded_String ("Concurrency control protocol");
            Lb_Used_Protocol (Current_Language)           :=
               To_Unbounded_String
                 ("Concurrency control protocol utilis : ");
            Lb_Resource_State (Current_Language)          :=
               To_Unbounded_String ("Etat du composant data (Entier) : ");
            Lb_Resources (Current_Language)               :=
               To_Unbounded_String ("Composants data");
            Lb_Resource_Affect (Current_Language)         :=
               To_Unbounded_String
                 ("Composant data partage par les thread ");
            Lb_Resource_Information (Current_Language)    :=
               To_Unbounded_String ("Information sur le composant data ");
            Lb_Invalid_Protocol (Current_Language)        :=
               To_Unbounded_String(
" Concurrency control protocol de partage de composant data invalide ");
            Lb_Cpu_Filter (Current_Language)              :=
               To_Unbounded_String ("Selectionner un composant processor");

            --
            --  Dependency graph widgets
            --
            Lb_New_Buffer_Role (Current_Language) :=
               To_Unbounded_String ("Nouveau rle event data port");

            --
            -- AADL options widget
            --
            Lb_Import_Aadl_Event_To_Precedencies (Current_Language)      :=
               To_Unbounded_String(
"Transformer les event ports en prcdence de composants thread");
            Lb_Import_Aadl_Event_Data_To_Precedencies (Current_Language) :=
               To_Unbounded_String(
"Transformer les event data ports en prcdence de composants thread");
            Lb_Export_Aadl_Precedencies_To_Event (Current_Language)      :=
               To_Unbounded_String(
"Transformer les prcdences de composants thread en event ports");
            Lb_Export_Aadl_Precedencies_To_Event_Data (Current_Language) :=
               To_Unbounded_String(
"Transformer les prcdences de composants thread en event data ports");
            Lb_Export_Aadl_Precedencies_To_Properties (Current_Language) :=
               To_Unbounded_String(
"Transformer les prcdences de composants thread en proprits Cheddar");

            ----------------------------------------------------
            --   Seed options
            ----------------------------------------------------
            Lb_Task_Specific (Current_Language)     :=
               To_Unbounded_String ("Spcifiques aux composants thread");
            Lb_All_Predictable (Current_Language)   :=
               To_Unbounded_String
                 ("Prvisible pour toutes les composants thread");
            Lb_All_Unpredictable (Current_Language) :=
               To_Unbounded_String
                 ("Imprvisible pour toutes les composants thread");

            --
            -- Shared resources logic
            --
            Lb_Compute_Blocking_Error1 (Current_Language) :=
               To_Unbounded_String(
"Toutes les composants data doivent utiliser le mme concurrency control protocol."
);
            Lb_Compute_Blocking_Error2 (Current_Language) :=
               To_Unbounded_String(
"- Erreur de concurrency control protocol : protocole de partage de composant data incompatible avec le scheduling protocol slectionn."
);
            Lb_Compute_Blocking_Error3 (Current_Language) :=
               To_Unbounded_String(
"Impossible de calculer une borne sur les temps de blocage avec ce concurrency control protocol."
);
            Lb_Has_Priority_Inversion (Current_Language)  :=
               To_Unbounded_String(
"  est victime d'une inversion de priorit sur le composant data ");

            --
            -- Scheduling logic
            --
            Lb_Priorities_Error1 (Current_Language)   :=
               To_Unbounded_String(
"- Composants thread non priodiques : impossible de modifier les priorits.");
            Lb_Priorities_Error2 (Current_Language)   :=
               To_Unbounded_String(
"- Composant processor sans threads : impossible de modifier les priorits.");
            Lb_Sched_Explanation1 (Current_Language)  :=
               To_Unbounded_String(
"les composants thread sont ordonnancables car le taux d'utilisation processeur "
);
            Lb_Sched_Explanation2 (Current_Language)  :=
               To_Unbounded_String(
"les composants thread ne sont pas ordonnancables car le taux d'utilisation processeur "
);
            Lb_Sched_Explanation3 (Current_Language)  :=
               To_Unbounded_String(
"il n'est pas possible de prouver que les composants thread sont ordonnancables car le taux d'utilisation processeur "
);
            Lb_Sched_Explanation5 (Current_Language)  :=
               To_Unbounded_String(
"le jeu de composants thread est ordonnancable car pour chaque composant thread "
);
            Lb_Sched_Explanation6 (Current_Language)  :=
               To_Unbounded_String
                 ("le jeu de composants thread n'est pas ordonnancable car ");
            Lb_Sched_Explanation62 (Current_Language) :=
               To_Unbounded_String(
"un ou plusieurs composants thread ne remplissent pas le test de faisabilit sur le taux d'utilisation processeur"
);
            Lb_Sched_Explanation7 (Current_Language)  :=
               To_Unbounded_String
                 ("  Liste des composants thread non ordonnancables : ");
            Lb_Sched_Explanation8 (Current_Language)  :=
               To_Unbounded_String(
"il n'est pas possible de prouver que le jeu de composants thread est ordonnancable car "
);
            Lb_Sched_Explanation81 (Current_Language) :=
               To_Unbounded_String(
"un ou plusieurs composants thread ne remplissent pas le test de faisabilit sur le taux d'utilisation processeur"
);
            Lb_Sched_Explanation82 (Current_Language) :=
               To_Unbounded_String
                 ("  Liste des composants thread concerns : ");
            Lb_No_Deadline_Missed_In_The_Computed_Scheduling (Current_Language)
               :=
               To_Unbounded_String(
"- Aucune chance de manque dans la simulation d'ordonnancement : le jeu de composants thread semble tre ordonnanable."
);
            Lb_No_Deadline_Will_Be_Missed_Task_Are_Schedulable (
              Current_Language)                       :=
               To_Unbounded_String(
"- Toutes les chances seront respectes : le jeu de composants thread est ordonnanable."
);
            Lb_Deadline_Will_Be_Missed_Task_Are_Not_Schedulable (
              Current_Language)                       :=
               To_Unbounded_String(
"- Certaines chances ne seront pas respectes : le jeu de composants thread n'est pas ordonnanable."
);
            Lb_worst_case_Task_Response_Time (Current_Language) :=
               To_Unbounded_String
                 ("Pire temps de rponse des composants thread");
            Lb_Task_Response_Time (Current_Language)  :=
               To_Unbounded_String
                 ("- Temps de rponse des composants thread : ");
            Lb_Task_Response_Time (Current_Language)  :=
               To_Unbounded_String ("Temps de rponse des composants thread");
            Lb_Task_Response_Time_From_Simulation (Current_Language) :=
               To_Unbounded_String(
"Temps de rponse des composants thread calcul  partir de la simulation");
            Lb_Partitioning_Result (Current_Language) :=
               To_Unbounded_String
                 ("Placement des composants thread aprs partitionnement");
            Lb_Compute_Scheduling_Error_1 (Current_Language) :=
               To_Unbounded_String(
"- Erreur sur le modle de composant thread : le scheduling protocol slectionne est incompatible."
);
            Lb_Compute_Scheduling_Error_2 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread non priodiques : impossible de calculer la priode d'tude avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_3 (Current_Language) :=
               To_Unbounded_String(
"- Erreur sur le modle de composant thread : impossible de calculer la priode d'tude avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_4 (Current_Language) :=
               To_Unbounded_String(
"- Composant thread avec priodes diffrentes des chances : impossible de calculer les bornes des temps de rponse avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_5 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread non priodiques : impossible de calculer les bornes des temps de rponse avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_6 (Current_Language) :=
               To_Unbounded_String(
"- Erreur sur le modle de composant thread : impossible de calculer les bornes des temps de rponse avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_7 (Current_Language) :=
               To_Unbounded_String(
"- Taux d'utilisation trp lev : impossible de calculer les bornes des temps de rponse avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_8 (Current_Language) :=
               To_Unbounded_String(
"- Erreur interne : impossible de calculer les bornes des temps de rponse avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_9 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread non priodiques : impossible de calculer un taux d'utilisation bas sur les priodes avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_10 (Current_Language) :=
               To_Unbounded_String(
"- Erreur sur le modle de composant thread : impossible de calculer un taux d'utilisation avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_11 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread avec priorit identique : le scheduling protocol slectionne est incompatible."
);
            Lb_Compute_Scheduling_Error_12 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread avec date d'arrive >  zro : impossible de calculer les bornes des temps de rponse avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_13 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread avec offset diffrents de zro : impossible de calculer les bornes des temps de rponse avec ce jeu de composants thread."
);
            Lb_Compute_Scheduling_Error_14 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread avec priodes diffrentes des chances : le scheduling protocol slectionn est incompatible."
);
            Lb_Compute_Scheduling_Error_15 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread non priodiques : le scheduling protocol slectionn est incompatible."
);
            Lb_Compute_Scheduling_Error_21 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread avec chance nulle : impossible de calculer un taux d'utilisation bas sur les chances avec ce jeu de composants thread."
);
            Lb_Periodic (Current_Language)            :=
               To_Unbounded_String ("Le composant thread priodique ");
            Lb_Aperiodic (Current_Language)           :=
               To_Unbounded_String ("Le composant thread apriodique ");
            Lb_Poisson_Process_Task (Current_Language) :=
               To_Unbounded_String
                 ("Le composant thread processus de Poisson ");
            Lb_Sporadic_Task (Current_Language)       :=
               To_Unbounded_String ("Le composant thread sporadique ");
            Lb_Parametric_Task (Current_Language)     :=
               To_Unbounded_String ("Le composant thread paramtrique ");
            lb_task_is_not_over_response_time_is_not_computed (
              Current_Language)                       :=
               To_Unbounded_String(
" temps de reponse non calcule car le composant thread n'a pas termin son compute execution time"
);
            Lb_Run_Simulation_On_One_Processor_Only (Current_Language) :=
               To_Unbounded_String
                 ("Ordonnance tous les composants processor");
            Lb_Run_Feasibility_Tests_On_One_Processor_Only (Current_Language)
               :=
               To_Unbounded_String
                 (" Faisabilit sur tous les composants processor");
            Lb_Partition_Error1 (Current_Language)    :=
               To_Unbounded_String(
"- Impossible de partitionner : tous les composants processeur doivent avoir le mme scheduling protocol."
);
            Lb_Partition_Error2 (Current_Language)    :=
               To_Unbounded_String(
"- Impossible de partitionner : nombre de composant processeur insuffisant.");
            Lb_Partition_Error3 (Current_Language)    :=
               To_Unbounded_String(
"- Scheduling protocol invalide : impossible de calculer le partitionnement.");

            --
            -- Buffer logic
            --
            Lb_Compute_Buffer_Error_1 (Current_Language) :=
               To_Unbounded_String(
"- Contrainte de dbit non respecte : impossible de calculer la borne de l'event data port tampon avec ce jeu de composants thread."
);
            Lb_Compute_Buffer_Error_2 (Current_Language) :=
               To_Unbounded_String(
"- Composants thread non priodiques : impossible de calculer la borne de l'event data port avec ce jeu de composants thread."
);
            Lb_Compute_Buffer_Error_3 (Current_Language) :=
               To_Unbounded_String(
"- Erreur sur le modle de composant thread : impossible de calculer la borne de l'event data port avec plusieurs consommateurs. "
);
            Lb_Maximum_Buffer_Size (Current_Language)    :=
               To_Unbounded_String
                 ("- Nombre maximum de message dans l'event data port : ");
            Lb_Average_Buffer_Size (Current_Language)    :=
               To_Unbounded_String
                 ("- Nombre moyen de message dans l'event data port : ");

            --
            -- Dependencies logic
            --
            Lb_Precedencies_Period_Error (Current_Language) :=
               To_Unbounded_String(
"- Imposible d'ordonnancer avec les contraintes de precedence : les composants thread ayant des contraintes de prcdence doivent avoir une priode identique."
);
            Lb_Chetto_Error2 (Current_Language)             :=
               To_Unbounded_String(
"- Composants thread avec des priodes diffrentes : impossible d'appliquer la mthode de Chetto/Blazewicz. "
);
            Lb_Chetto_Error3 (Current_Language)             :=
               To_Unbounded_String(
"- Erreur sur le modle de composant thread : impossible d'appliquer la mthode de Chetto/Blazewicz. "
);
         end if;
      end if;
   end Initialize;

end Translate.AADL_Labels;
